@@ -1,3 +1,6 @@
+// Copyright (c) HashiCorp, Inc.
+// SPDX-License-Identifier: MPL-2.0
+
 package api
 
 import (
@@ -61,10 +64,11 @@ func (cc *ConsulConnect) Canonicalize() {
 // ConsulSidecarService represents a Consul Connect SidecarService jobspec
 // block.
 type ConsulSidecarService struct {
-	Tags                   []string     `hcl:"tags,optional"`
-	Port                   string       `hcl:"port,optional"`
-	Proxy                  *ConsulProxy `hcl:"proxy,block"`
-	DisableDefaultTCPCheck bool         `mapstructure:"disable_default_tcp_check" hcl:"disable_default_tcp_check,optional"`
+	Tags                   []string          `hcl:"tags,optional"`
+	Port                   string            `hcl:"port,optional"`
+	Proxy                  *ConsulProxy      `hcl:"proxy,block"`
+	DisableDefaultTCPCheck bool              `mapstructure:"disable_default_tcp_check" hcl:"disable_default_tcp_check,optional"`
+	Meta                   map[string]string `hcl:"meta,block"`
 }
 
 func (css *ConsulSidecarService) Canonicalize() {
@@ -76,6 +80,10 @@ func (css *ConsulSidecarService) Canonicalize() {
 		css.Tags = nil
 	}
 
+	if len(css.Meta) == 0 {
+		css.Meta = nil
+	}
+
 	css.Proxy.Canonicalize()
 }
 
