@@ -1,3 +1,6 @@
+// Copyright (c) HashiCorp, Inc.
+// SPDX-License-Identifier: MPL-2.0
+
 package allocrunner
 
 import (
@@ -11,25 +14,29 @@ import (
 
 	"github.com/hashicorp/consul/api"
 	multierror "github.com/hashicorp/go-multierror"
+	"github.com/shoenig/test/must"
+	"github.com/shoenig/test/wait"
+	"github.com/stretchr/testify/require"
+
 	"github.com/hashicorp/nomad/ci"
 	"github.com/hashicorp/nomad/client/allochealth"
+	"github.com/hashicorp/nomad/client/allocrunner/interfaces"
+	arstate "github.com/hashicorp/nomad/client/allocrunner/state"
 	"github.com/hashicorp/nomad/client/allocrunner/tasklifecycle"
 	"github.com/hashicorp/nomad/client/allocrunner/taskrunner"
 	"github.com/hashicorp/nomad/client/allocwatcher"
 	"github.com/hashicorp/nomad/client/serviceregistration"
 	regMock "github.com/hashicorp/nomad/client/serviceregistration/mock"
 	"github.com/hashicorp/nomad/client/state"
+	cstructs "github.com/hashicorp/nomad/client/structs"
 	"github.com/hashicorp/nomad/helper/uuid"
 	"github.com/hashicorp/nomad/nomad/mock"
 	"github.com/hashicorp/nomad/nomad/structs"
 	"github.com/hashicorp/nomad/testutil"
-	"github.com/shoenig/test/must"
-	"github.com/shoenig/test/wait"
-	"github.com/stretchr/testify/require"
 )
 
 // destroy does a blocking destroy on an alloc runner
-func destroy(ar *allocRunner) {
+func destroy(ar interfaces.AllocRunner) {
 	ar.Destroy()
 	<-ar.DestroyCh()
 }
@@ -45,7 +52,7 @@ func TestAllocRunner_AllocState_Initialized(t *testing.T) {
 	defer cleanup()
 
 	ar, err := NewAllocRunner(conf)
-	require.NoError(t, err)
+	must.NoError(t, err)
 
 	allocState := ar.AllocState()
 
@@ -86,7 +93,7 @@ func TestAllocRunner_TaskLeader_KillTG(t *testing.T) {
 	conf, cleanup := testAllocRunnerConfig(t, alloc)
 	defer cleanup()
 	ar, err := NewAllocRunner(conf)
-	require.NoError(t, err)
+	must.NoError(t, err)
 	defer destroy(ar)
 	go ar.Run()
 
@@ -169,7 +176,7 @@ func TestAllocRunner_Lifecycle_Poststart(t *testing.T) {
 	conf, cleanup := testAllocRunnerConfig(t, alloc)
 	defer cleanup()
 	ar, err := NewAllocRunner(conf)
-	require.NoError(t, err)
+	must.NoError(t, err)
 	defer destroy(ar)
 	go ar.Run()
 
@@ -306,7 +313,7 @@ func TestAllocRunner_TaskMain_KillTG(t *testing.T) {
 	conf, cleanup := testAllocRunnerConfig(t, alloc)
 	defer cleanup()
 	ar, err := NewAllocRunner(conf)
-	require.NoError(t, err)
+	must.NoError(t, err)
 	defer destroy(ar)
 	go ar.Run()
 
@@ -428,7 +435,7 @@ func TestAllocRunner_Lifecycle_Poststop(t *testing.T) {
 	conf, cleanup := testAllocRunnerConfig(t, alloc)
 	defer cleanup()
 	ar, err := NewAllocRunner(conf)
-	require.NoError(t, err)
+	must.NoError(t, err)
 	defer destroy(ar)
 	go ar.Run()
 
@@ -513,13 +520,13 @@ func TestAllocRunner_Lifecycle_Restart(t *testing.T) {
 		taskDefs      []mock.LifecycleTaskDef
 		isBatch       bool
 		hasLeader     bool
-		action        func(*allocRunner, *structs.Allocation) error
+		action        func(interfaces.AllocRunner, *structs.Allocation) error
 		expectedErr   string
 		expectedAfter map[string]structs.TaskState
 	}{
 		{
 			name: "restart entire allocation",
-			action: func(ar *allocRunner, alloc *structs.Allocation) error {
+			action: func(ar interfaces.AllocRunner, alloc *structs.Allocation) error {
 				return ar.RestartAll(ev)
 			},
 			expectedAfter: map[string]structs.TaskState{
@@ -533,7 +540,7 @@ func TestAllocRunner_Lifecycle_Restart(t *testing.T) {
 		},
 		{
 			name: "restart only running tasks",
-			action: func(ar *allocRunner, alloc *structs.Allocation) error {
+			action: func(ar interfaces.AllocRunner, alloc *structs.Allocation) error {
 				return ar.RestartRunning(ev)
 			},
 			expectedAfter: map[string]structs.TaskState{
@@ -556,7 +563,7 @@ func TestAllocRunner_Lifecycle_Restart(t *testing.T) {
 				{Name: "poststop", RunFor: "1s", ExitCode: 0, Hook: "poststop", IsSidecar: false},
 			},
 			isBatch: true,
-			action: func(ar *allocRunner, alloc *structs.Allocation) error {
+			action: func(ar interfaces.AllocRunner, alloc *structs.Allocation) error {
 				return ar.RestartAll(ev)
 			},
 			expectedAfter: map[string]structs.TaskState{
@@ -579,7 +586,7 @@ func TestAllocRunner_Lifecycle_Restart(t *testing.T) {
 				{Name: "poststop", RunFor: "1s", ExitCode: 0, Hook: "poststop", IsSidecar: false},
 			},
 			isBatch: true,
-			action: func(ar *allocRunner, alloc *structs.Allocation) error {
+			action: func(ar interfaces.AllocRunner, alloc *structs.Allocation) error {
 				return ar.RestartRunning(ev)
 			},
 			expectedAfter: map[string]structs.TaskState{
@@ -594,7 +601,7 @@ func TestAllocRunner_Lifecycle_Restart(t *testing.T) {
 		{
 			name:      "restart entire allocation with leader",
 			hasLeader: true,
-			action: func(ar *allocRunner, alloc *structs.Allocation) error {
+			action: func(ar interfaces.AllocRunner, alloc *structs.Allocation) error {
 				return ar.RestartAll(ev)
 			},
 			expectedAfter: map[string]structs.TaskState{
@@ -608,7 +615,7 @@ func TestAllocRunner_Lifecycle_Restart(t *testing.T) {
 		},
 		{
 			name: "stop from server",
-			action: func(ar *allocRunner, alloc *structs.Allocation) error {
+			action: func(ar interfaces.AllocRunner, alloc *structs.Allocation) error {
 				stopAlloc := alloc.Copy()
 				stopAlloc.DesiredStatus = structs.AllocDesiredStatusStop
 				ar.Update(stopAlloc)
@@ -625,7 +632,7 @@ func TestAllocRunner_Lifecycle_Restart(t *testing.T) {
 		},
 		{
 			name: "restart main task",
-			action: func(ar *allocRunner, alloc *structs.Allocation) error {
+			action: func(ar interfaces.AllocRunner, alloc *structs.Allocation) error {
 				return ar.RestartTask("main", ev)
 			},
 			expectedAfter: map[string]structs.TaskState{
@@ -640,7 +647,7 @@ func TestAllocRunner_Lifecycle_Restart(t *testing.T) {
 		{
 			name:      "restart leader main task",
 			hasLeader: true,
-			action: func(ar *allocRunner, alloc *structs.Allocation) error {
+			action: func(ar interfaces.AllocRunner, alloc *structs.Allocation) error {
 				return ar.RestartTask("main", ev)
 			},
 			expectedAfter: map[string]structs.TaskState{
@@ -662,7 +669,7 @@ func TestAllocRunner_Lifecycle_Restart(t *testing.T) {
 				{Name: "poststart-sidecar", RunFor: "100s", ExitCode: 0, Hook: "poststart", IsSidecar: true},
 				{Name: "poststop", RunFor: "1s", ExitCode: 0, Hook: "poststop", IsSidecar: false},
 			},
-			action: func(ar *allocRunner, alloc *structs.Allocation) error {
+			action: func(ar interfaces.AllocRunner, alloc *structs.Allocation) error {
 				time.Sleep(3 * time.Second) // make sure main task has exited
 				return nil
 			},
@@ -686,7 +693,7 @@ func TestAllocRunner_Lifecycle_Restart(t *testing.T) {
 				{Name: "poststop", RunFor: "1s", ExitCode: 0, Hook: "poststop", IsSidecar: false},
 			},
 			hasLeader: true,
-			action: func(ar *allocRunner, alloc *structs.Allocation) error {
+			action: func(ar interfaces.AllocRunner, alloc *structs.Allocation) error {
 				time.Sleep(3 * time.Second) // make sure main task has exited
 				return nil
 			},
@@ -709,7 +716,7 @@ func TestAllocRunner_Lifecycle_Restart(t *testing.T) {
 				{Name: "poststart-sidecar", RunFor: "100s", ExitCode: 0, Hook: "poststart", IsSidecar: true},
 				{Name: "poststop", RunFor: "1s", ExitCode: 0, Hook: "poststop", IsSidecar: false},
 			},
-			action: func(ar *allocRunner, alloc *structs.Allocation) error {
+			action: func(ar interfaces.AllocRunner, alloc *structs.Allocation) error {
 				time.Sleep(3 * time.Second) // make sure main task has exited
 				return nil
 			},
@@ -732,7 +739,7 @@ func TestAllocRunner_Lifecycle_Restart(t *testing.T) {
 				{Name: "poststart-sidecar", RunFor: "100s", ExitCode: 0, Hook: "poststart", IsSidecar: true},
 				{Name: "poststop", RunFor: "1s", ExitCode: 0, Hook: "poststop", IsSidecar: false},
 			},
-			action: func(ar *allocRunner, alloc *structs.Allocation) error {
+			action: func(ar interfaces.AllocRunner, alloc *structs.Allocation) error {
 				time.Sleep(3 * time.Second) // make sure main task has exited
 				return nil
 			},
@@ -755,7 +762,7 @@ func TestAllocRunner_Lifecycle_Restart(t *testing.T) {
 				{Name: "poststart-sidecar", RunFor: "100s", ExitCode: 0, Hook: "poststart", IsSidecar: true},
 				{Name: "poststop", RunFor: "1s", ExitCode: 0, Hook: "poststop", IsSidecar: false},
 			},
-			action: func(ar *allocRunner, alloc *structs.Allocation) error {
+			action: func(ar interfaces.AllocRunner, alloc *structs.Allocation) error {
 				// make sure main task has had a chance to restart once on its
 				// own and fail again before we try to manually restart it
 				time.Sleep(5 * time.Second)
@@ -773,7 +780,7 @@ func TestAllocRunner_Lifecycle_Restart(t *testing.T) {
 		},
 		{
 			name: "restart prestart-sidecar task",
-			action: func(ar *allocRunner, alloc *structs.Allocation) error {
+			action: func(ar interfaces.AllocRunner, alloc *structs.Allocation) error {
 				return ar.RestartTask("prestart-sidecar", ev)
 			},
 			expectedAfter: map[string]structs.TaskState{
@@ -787,7 +794,7 @@ func TestAllocRunner_Lifecycle_Restart(t *testing.T) {
 		},
 		{
 			name: "restart poststart-sidecar task",
-			action: func(ar *allocRunner, alloc *structs.Allocation) error {
+			action: func(ar interfaces.AllocRunner, alloc *structs.Allocation) error {
 				return ar.RestartTask("poststart-sidecar", ev)
 			},
 			expectedAfter: map[string]structs.TaskState{
@@ -829,7 +836,7 @@ func TestAllocRunner_Lifecycle_Restart(t *testing.T) {
 			conf, cleanup := testAllocRunnerConfig(t, alloc)
 			defer cleanup()
 			ar, err := NewAllocRunner(conf)
-			require.NoError(t, err)
+			must.NoError(t, err)
 			defer destroy(ar)
 			go ar.Run()
 
@@ -988,7 +995,7 @@ func TestAllocRunner_TaskGroup_ShutdownDelay(t *testing.T) {
 	conf, cleanup := testAllocRunnerConfig(t, alloc)
 	defer cleanup()
 	ar, err := NewAllocRunner(conf)
-	require.NoError(t, err)
+	must.NoError(t, err)
 	defer destroy(ar)
 	go ar.Run()
 
@@ -1113,7 +1120,7 @@ func TestAllocRunner_TaskLeader_StopTG(t *testing.T) {
 	conf, cleanup := testAllocRunnerConfig(t, alloc)
 	defer cleanup()
 	ar, err := NewAllocRunner(conf)
-	require.NoError(t, err)
+	must.NoError(t, err)
 	defer destroy(ar)
 	go ar.Run()
 
@@ -1211,15 +1218,15 @@ func TestAllocRunner_TaskLeader_StopRestoredTG(t *testing.T) {
 	conf.StateDB = state.NewMemDB(conf.Logger)
 
 	ar, err := NewAllocRunner(conf)
-	require.NoError(t, err)
+	must.NoError(t, err)
 
 	// Mimic Nomad exiting before the leader stopping is able to stop other tasks.
-	ar.tasks["leader"].UpdateState(structs.TaskStateDead, structs.NewTaskEvent(structs.TaskKilled))
-	ar.tasks["follower1"].UpdateState(structs.TaskStateRunning, structs.NewTaskEvent(structs.TaskStarted))
+	ar.(*allocRunner).tasks["leader"].UpdateState(structs.TaskStateDead, structs.NewTaskEvent(structs.TaskKilled))
+	ar.(*allocRunner).tasks["follower1"].UpdateState(structs.TaskStateRunning, structs.NewTaskEvent(structs.TaskStarted))
 
 	// Create a new AllocRunner to test RestoreState and Run
 	ar2, err := NewAllocRunner(conf)
-	require.NoError(t, err)
+	must.NoError(t, err)
 	defer destroy(ar2)
 
 	if err := ar2.Restore(); err != nil {
@@ -1263,8 +1270,9 @@ func TestAllocRunner_Restore_LifecycleHooks(t *testing.T) {
 	// Use a memory backed statedb
 	conf.StateDB = state.NewMemDB(conf.Logger)
 
-	ar, err := NewAllocRunner(conf)
-	require.NoError(t, err)
+	arIface, err := NewAllocRunner(conf)
+	must.NoError(t, err)
+	ar := arIface.(*allocRunner)
 
 	go ar.Run()
 	defer destroy(ar)
@@ -1287,9 +1295,10 @@ func TestAllocRunner_Restore_LifecycleHooks(t *testing.T) {
 	ar.tasks["web"].UpdateState(structs.TaskStateRunning, structs.NewTaskEvent(structs.TaskStarted))
 
 	// Create a new AllocRunner to test Restore and Run.
-	ar2, err := NewAllocRunner(conf)
-	require.NoError(t, err)
-	require.NoError(t, ar2.Restore())
+	arIface2, err := NewAllocRunner(conf)
+	must.NoError(t, err)
+	ar2 := arIface2.(*allocRunner)
+	must.NoError(t, ar2.Restore())
 
 	go ar2.Run()
 	defer destroy(ar2)
@@ -1322,8 +1331,9 @@ func TestAllocRunner_Update_Semantics(t *testing.T) {
 	conf, cleanup := testAllocRunnerConfig(t, alloc)
 	defer cleanup()
 
-	ar, err := NewAllocRunner(conf)
-	require.NoError(err)
+	arIface, err := NewAllocRunner(conf)
+	must.NoError(t, err)
+	ar := arIface.(*allocRunner)
 
 	upd1 := updatedAlloc(alloc)
 	ar.Update(upd1)
@@ -1383,7 +1393,7 @@ func TestAllocRunner_DeploymentHealth_Healthy_Migration(t *testing.T) {
 	defer cleanup()
 
 	ar, err := NewAllocRunner(conf)
-	require.NoError(t, err)
+	must.NoError(t, err)
 	go ar.Run()
 	defer destroy(ar)
 
@@ -1437,7 +1447,7 @@ func TestAllocRunner_DeploymentHealth_Healthy_NoChecks(t *testing.T) {
 	defer cleanup()
 
 	ar, err := NewAllocRunner(conf)
-	require.NoError(t, err)
+	must.NoError(t, err)
 
 	start, done := time.Now(), time.Time{}
 	go ar.Run()
@@ -1531,7 +1541,7 @@ func TestAllocRunner_DeploymentHealth_Unhealthy_Checks(t *testing.T) {
 	}
 
 	ar, err := NewAllocRunner(conf)
-	require.NoError(t, err)
+	must.NoError(t, err)
 	go ar.Run()
 	defer destroy(ar)
 
@@ -1580,7 +1590,7 @@ func TestAllocRunner_Destroy(t *testing.T) {
 	conf.StateDB = state.NewMemDB(conf.Logger)
 
 	ar, err := NewAllocRunner(conf)
-	require.NoError(t, err)
+	must.NoError(t, err)
 	go ar.Run()
 
 	// Wait for alloc to be running
@@ -1620,8 +1630,8 @@ func TestAllocRunner_Destroy(t *testing.T) {
 	require.Nil(t, ts)
 
 	// Assert the alloc directory was cleaned
-	if _, err := os.Stat(ar.allocDir.AllocDir); err == nil {
-		require.Fail(t, "alloc dir still exists: %v", ar.allocDir.AllocDir)
+	if _, err := os.Stat(ar.(*allocRunner).allocDir.AllocDir); err == nil {
+		require.Fail(t, "alloc dir still exists: %v", ar.(*allocRunner).allocDir.AllocDir)
 	} else if !os.IsNotExist(err) {
 		require.Failf(t, "expected NotExist error", "found %v", err)
 	}
@@ -1635,7 +1645,7 @@ func TestAllocRunner_SimpleRun(t *testing.T) {
 	conf, cleanup := testAllocRunnerConfig(t, alloc)
 	defer cleanup()
 	ar, err := NewAllocRunner(conf)
-	require.NoError(t, err)
+	must.NoError(t, err)
 	go ar.Run()
 	defer destroy(ar)
 
@@ -1670,7 +1680,8 @@ func TestAllocRunner_MoveAllocDir(t *testing.T) {
 	conf, cleanup := testAllocRunnerConfig(t, alloc)
 	defer cleanup()
 	ar, err := NewAllocRunner(conf)
-	require.NoError(t, err)
+	must.NoError(t, err)
+
 	ar.Run()
 	defer destroy(ar)
 
@@ -1678,9 +1689,9 @@ func TestAllocRunner_MoveAllocDir(t *testing.T) {
 
 	// Step 2. Modify its directory
 	task := alloc.Job.TaskGroups[0].Tasks[0]
-	dataFile := filepath.Join(ar.allocDir.SharedDir, "data", "data_file")
+	dataFile := filepath.Join(ar.GetAllocDir().SharedDir, "data", "data_file")
 	os.WriteFile(dataFile, []byte("hello world"), os.ModePerm)
-	taskDir := ar.allocDir.TaskDirs[task.Name]
+	taskDir := ar.GetAllocDir().TaskDirs[task.Name]
 	taskLocalFile := filepath.Join(taskDir.LocalDir, "local_file")
 	os.WriteFile(taskLocalFile, []byte("good bye world"), os.ModePerm)
 
@@ -1697,7 +1708,7 @@ func TestAllocRunner_MoveAllocDir(t *testing.T) {
 	})
 	defer cleanup()
 	ar2, err := NewAllocRunner(conf2)
-	require.NoError(t, err)
+	must.NoError(t, err)
 
 	ar2.Run()
 	defer destroy(ar2)
@@ -1705,11 +1716,11 @@ func TestAllocRunner_MoveAllocDir(t *testing.T) {
 	WaitForClientState(t, ar, structs.AllocClientStatusComplete)
 
 	// Ensure that data from ar was moved to ar2
-	dataFile = filepath.Join(ar2.allocDir.SharedDir, "data", "data_file")
+	dataFile = filepath.Join(ar2.GetAllocDir().SharedDir, "data", "data_file")
 	fileInfo, _ := os.Stat(dataFile)
 	require.NotNilf(t, fileInfo, "file %q not found", dataFile)
 
-	taskDir = ar2.allocDir.TaskDirs[task.Name]
+	taskDir = ar2.GetAllocDir().TaskDirs[task.Name]
 	taskLocalFile = filepath.Join(taskDir.LocalDir, "local_file")
 	fileInfo, _ = os.Stat(taskLocalFile)
 	require.NotNilf(t, fileInfo, "file %q not found", dataFile)
@@ -1752,7 +1763,8 @@ func TestAllocRunner_HandlesArtifactFailure(t *testing.T) {
 	conf, cleanup := testAllocRunnerConfig(t, alloc)
 	defer cleanup()
 	ar, err := NewAllocRunner(conf)
-	require.NoError(t, err)
+	must.NoError(t, err)
+
 	go ar.Run()
 	defer destroy(ar)
 
@@ -1856,7 +1868,8 @@ func TestAllocRunner_TaskFailed_KillTG(t *testing.T) {
 	}
 
 	ar, err := NewAllocRunner(conf)
-	require.NoError(t, err)
+	must.NoError(t, err)
+
 	defer destroy(ar)
 	go ar.Run()
 	upd := conf.StateUpdater.(*MockStateUpdater)
@@ -1926,7 +1939,8 @@ func TestAllocRunner_TerminalUpdate_Destroy(t *testing.T) {
 	conf, cleanup := testAllocRunnerConfig(t, alloc)
 	defer cleanup()
 	ar, err := NewAllocRunner(conf)
-	require.NoError(t, err)
+	must.NoError(t, err)
+
 	defer destroy(ar)
 	go ar.Run()
 	upd := conf.StateUpdater.(*MockStateUpdater)
@@ -1946,7 +1960,7 @@ func TestAllocRunner_TerminalUpdate_Destroy(t *testing.T) {
 
 	// Update the alloc to be terminal which should cause the alloc runner to
 	// stop the tasks and wait for a destroy.
-	update := ar.alloc.Copy()
+	update := ar.Alloc().Copy()
 	update.DesiredStatus = structs.AllocDesiredStatusStop
 	ar.Update(update)
 
@@ -1962,8 +1976,8 @@ func TestAllocRunner_TerminalUpdate_Destroy(t *testing.T) {
 		}
 
 		// Check the alloc directory still exists
-		if _, err := os.Stat(ar.allocDir.AllocDir); err != nil {
-			return false, fmt.Errorf("alloc dir destroyed: %v", ar.allocDir.AllocDir)
+		if _, err := os.Stat(ar.GetAllocDir().AllocDir); err != nil {
+			return false, fmt.Errorf("alloc dir destroyed: %v", ar.GetAllocDir().AllocDir)
 		}
 
 		return true, nil
@@ -1986,8 +2000,8 @@ func TestAllocRunner_TerminalUpdate_Destroy(t *testing.T) {
 		}
 
 		// Check the alloc directory was cleaned
-		if _, err := os.Stat(ar.allocDir.AllocDir); err == nil {
-			return false, fmt.Errorf("alloc dir still exists: %v", ar.allocDir.AllocDir)
+		if _, err := os.Stat(ar.GetAllocDir().AllocDir); err == nil {
+			return false, fmt.Errorf("alloc dir still exists: %v", ar.GetAllocDir().AllocDir)
 		} else if !os.IsNotExist(err) {
 			return false, fmt.Errorf("stat err: %v", err)
 		}
@@ -2010,7 +2024,8 @@ func TestAllocRunner_PersistState_Destroyed(t *testing.T) {
 
 	defer cleanup()
 	ar, err := NewAllocRunner(conf)
-	require.NoError(t, err)
+	must.NoError(t, err)
+
 	defer destroy(ar)
 
 	go ar.Run()
@@ -2106,12 +2121,12 @@ func TestAllocRunner_Reconnect(t *testing.T) {
 			defer cleanup()
 
 			ar, err := NewAllocRunner(conf)
-			require.NoError(t, err)
+			must.NoError(t, err)
 			defer destroy(ar)
 
 			go ar.Run()
 
-			for _, taskRunner := range ar.tasks {
+			for _, taskRunner := range ar.(*allocRunner).tasks {
 				taskRunner.UpdateState(tc.taskState, tc.taskEvent)
 			}
 
@@ -2185,7 +2200,8 @@ func TestAllocRunner_Lifecycle_Shutdown_Order(t *testing.T) {
 	conf, cleanup := testAllocRunnerConfig(t, alloc)
 	defer cleanup()
 	ar, err := NewAllocRunner(conf)
-	require.NoError(t, err)
+	must.NoError(t, err)
+
 	defer destroy(ar)
 	go ar.Run()
 
@@ -2386,8 +2402,9 @@ func TestHasSidecarTasks(t *testing.T) {
 			arConf, cleanup := testAllocRunnerConfig(t, alloc)
 			defer cleanup()
 
-			ar, err := NewAllocRunner(arConf)
-			require.NoError(t, err)
+			arIface, err := NewAllocRunner(arConf)
+			must.NoError(t, err)
+			ar := arIface.(*allocRunner)
 
 			require.Equal(t, tc.hasSidecars, hasSidecarTasks(ar.tasks), "sidecars")
 
@@ -2423,8 +2440,9 @@ func TestAllocRunner_PreKill_RunOnDone(t *testing.T) {
 	conf, cleanup := testAllocRunnerConfig(t, alloc.Copy())
 	t.Cleanup(cleanup)
 
-	ar, err := NewAllocRunner(conf)
+	arIface, err := NewAllocRunner(conf)
 	must.NoError(t, err)
+	ar := arIface.(*allocRunner)
 
 	// set our custom prekill hook
 	hook := new(allocPreKillHook)
@@ -2440,3 +2458,65 @@ func TestAllocRunner_PreKill_RunOnDone(t *testing.T) {
 		wait.Gap(500*time.Millisecond),
 	))
 }
+
+func TestAllocRunner_GetUpdatePriority(t *testing.T) {
+	ci.Parallel(t)
+
+	alloc := mock.Alloc()
+	task := alloc.Job.TaskGroups[0].Tasks[0]
+	task.Driver = "mock_driver"
+	task.Config = map[string]interface{}{"run_for": "2ms"}
+	alloc.DesiredStatus = "stop"
+
+	conf, cleanup := testAllocRunnerConfig(t, alloc.Copy())
+	t.Cleanup(cleanup)
+
+	arIface, err := NewAllocRunner(conf)
+	must.NoError(t, err)
+	ar := arIface.(*allocRunner)
+
+	ar.SetNetworkStatus(&structs.AllocNetworkStatus{
+		InterfaceName: "eth0",
+		Address:       "192.168.1.1",
+		DNS:           &structs.DNSConfig{},
+	})
+
+	calloc := ar.clientAlloc(map[string]*structs.TaskState{})
+	ar.AcknowledgeState(&arstate.State{
+		ClientStatus:      calloc.ClientStatus,
+		ClientDescription: calloc.ClientDescription,
+		DeploymentStatus:  calloc.DeploymentStatus,
+		TaskStates:        calloc.TaskStates,
+		NetworkStatus:     calloc.NetworkStatus,
+	})
+
+	must.Eq(t, cstructs.AllocUpdatePriorityNone, ar.GetUpdatePriority(calloc))
+
+	// clientAlloc mutates the state, so verify this doesn't break the check
+	// without state having been updated
+	calloc = ar.clientAlloc(map[string]*structs.TaskState{})
+	must.Eq(t, cstructs.AllocUpdatePriorityNone, ar.GetUpdatePriority(calloc))
+
+	// make a no-op state update
+	ar.SetNetworkStatus(&structs.AllocNetworkStatus{
+		InterfaceName: "eth0",
+		Address:       "192.168.1.1",
+		DNS:           &structs.DNSConfig{},
+	})
+	calloc = ar.clientAlloc(map[string]*structs.TaskState{})
+	must.Eq(t, cstructs.AllocUpdatePriorityNone, ar.GetUpdatePriority(calloc))
+
+	// make a low priority state update
+	ar.SetNetworkStatus(&structs.AllocNetworkStatus{
+		InterfaceName: "eth0",
+		Address:       "192.168.1.2",
+		DNS:           &structs.DNSConfig{},
+	})
+	calloc = ar.clientAlloc(map[string]*structs.TaskState{})
+	must.Eq(t, cstructs.AllocUpdatePriorityTypical, ar.GetUpdatePriority(calloc))
+
+	// make a state update that should be detected as high priority
+	ar.SetClientStatus(structs.AllocClientStatusFailed)
+	calloc = ar.clientAlloc(map[string]*structs.TaskState{})
+	must.Eq(t, cstructs.AllocUpdatePriorityUrgent, ar.GetUpdatePriority(calloc))
+}
