@@ -1,22 +1,14 @@
+// Copyright (c) HashiCorp, Inc.
+// SPDX-License-Identifier: MPL-2.0
+
 //go:build !linux && !windows
 
 package getter
 
 import (
 	"path/filepath"
-	"syscall"
 )
 
-// attributes is not implemented by default
-func attributes() *syscall.SysProcAttr {
-	return nil
-}
-
-// credentials is not implemented by default
-func credentials() (uint32, uint32) {
-	return 0, 0
-}
-
 // lockdown is not implemented by default
 func lockdown(string, string) error {
 	return nil
