@@ -1,3 +1,6 @@
+// Copyright (c) HashiCorp, Inc.
+// SPDX-License-Identifier: MPL-2.0
+
 //go:build windows
 
 package getter
@@ -5,19 +8,8 @@ package getter
 import (
 	"os"
 	"path/filepath"
-	"syscall"
 )
 
-// attributes is not implemented on Windows
-func attributes() *syscall.SysProcAttr {
-	return nil
-}
-
-// credentials is not implemented on Windows
-func credentials() (uint32, uint32) {
-	return 0, 0
-}
-
 // lockdown is not implemented on Windows
 func lockdown(string, string) error {
 	return nil
