@@ -1,3 +1,6 @@
+// Copyright (c) HashiCorp, Inc.
+// SPDX-License-Identifier: MPL-2.0
+
 package client
 
 import (
@@ -11,7 +14,15 @@ import (
 	"time"
 
 	memdb "github.com/hashicorp/go-memdb"
+	"github.com/shoenig/test"
+	"github.com/shoenig/test/must"
+	"github.com/shoenig/test/wait"
+	"github.com/stretchr/testify/assert"
+	"github.com/stretchr/testify/require"
+
 	"github.com/hashicorp/nomad/ci"
+	"github.com/hashicorp/nomad/client/allocrunner"
+	"github.com/hashicorp/nomad/client/allocrunner/interfaces"
 	trstate "github.com/hashicorp/nomad/client/allocrunner/taskrunner/state"
 	"github.com/hashicorp/nomad/client/config"
 	"github.com/hashicorp/nomad/client/fingerprint"
@@ -30,8 +41,6 @@ import (
 	"github.com/hashicorp/nomad/plugins/device"
 	psstructs "github.com/hashicorp/nomad/plugins/shared/structs"
 	"github.com/hashicorp/nomad/testutil"
-	"github.com/stretchr/testify/assert"
-	"github.com/stretchr/testify/require"
 )
 
 func testACLServer(t *testing.T, cb func(*nomad.Config)) (*nomad.Server, string, *structs.ACLToken, func()) {
@@ -249,9 +258,9 @@ func TestClient_MixedTLS(t *testing.T) {
 	ci.Parallel(t)
 
 	const (
-		cafile  = "../helper/tlsutil/testdata/ca.pem"
-		foocert = "../helper/tlsutil/testdata/nomad-foo.pem"
-		fookey  = "../helper/tlsutil/testdata/nomad-foo-key.pem"
+		cafile        = "../helper/tlsutil/testdata/nomad-agent-ca.pem"
+		fooservercert = "../helper/tlsutil/testdata/regionFoo-server-nomad.pem"
+		fooserverkey  = "../helper/tlsutil/testdata/regionFoo-server-nomad-key.pem"
 	)
 	s1, addr, cleanupS1 := testServer(t, func(c *nomad.Config) {
 		c.TLSConfig = &nconfig.TLSConfig{
@@ -259,8 +268,8 @@ func TestClient_MixedTLS(t *testing.T) {
 			EnableRPC:            true,
 			VerifyServerHostname: true,
 			CAFile:               cafile,
-			CertFile:             foocert,
-			KeyFile:              fookey,
+			CertFile:             fooservercert,
+			KeyFile:              fooserverkey,
 		}
 	})
 	defer cleanupS1()
@@ -271,6 +280,9 @@ func TestClient_MixedTLS(t *testing.T) {
 	})
 	defer cleanup()
 
+	// tell the client we've registered to unblock the RPC we test below
+	c1.registeredOnce.Do(func() { close(c1.registeredCh) })
+
 	req := structs.NodeSpecificRequest{
 		NodeID:       c1.Node().ID,
 		QueryOptions: structs.QueryOptions{Region: "global"},
@@ -279,7 +291,7 @@ func TestClient_MixedTLS(t *testing.T) {
 	testutil.AssertUntil(100*time.Millisecond,
 		func() (bool, error) {
 			err := c1.RPC("Node.GetNode", &req, &out)
-			if err == nil {
+			if err == nil || structs.IsErrPermissionDenied(err) {
 				return false, fmt.Errorf("client RPC succeeded when it should have failed:\n%+v", out)
 			}
 			return true, nil
@@ -297,12 +309,12 @@ func TestClient_BadTLS(t *testing.T) {
 	ci.Parallel(t)
 
 	const (
-		cafile  = "../helper/tlsutil/testdata/ca.pem"
-		foocert = "../helper/tlsutil/testdata/nomad-foo.pem"
-		fookey  = "../helper/tlsutil/testdata/nomad-foo-key.pem"
-		badca   = "../helper/tlsutil/testdata/ca-bad.pem"
-		badcert = "../helper/tlsutil/testdata/nomad-bad.pem"
-		badkey  = "../helper/tlsutil/testdata/nomad-bad-key.pem"
+		cafile        = "../helper/tlsutil/testdata/nomad-agent-ca.pem"
+		fooclientcert = "../helper/tlsutil/testdata/regionFoo-client-nomad.pem"
+		fooclientkey  = "../helper/tlsutil/testdata/regionFoo-client-nomad-key.pem"
+		badca         = "../helper/tlsutil/testdata/bad-agent-ca.pem"
+		badcert       = "../helper/tlsutil/testdata/badRegion-client-bad.pem"
+		badkey        = "../helper/tlsutil/testdata/badRegion-client-bad-key.pem"
 	)
 	s1, addr, cleanupS1 := testServer(t, func(c *nomad.Config) {
 		c.TLSConfig = &nconfig.TLSConfig{
@@ -310,8 +322,8 @@ func TestClient_BadTLS(t *testing.T) {
 			EnableRPC:            true,
 			VerifyServerHostname: true,
 			CAFile:               cafile,
-			CertFile:             foocert,
-			KeyFile:              fookey,
+			CertFile:             fooclientcert,
+			KeyFile:              fooclientkey,
 		}
 	})
 	defer cleanupS1()
@@ -330,6 +342,9 @@ func TestClient_BadTLS(t *testing.T) {
 	})
 	defer cleanupC1()
 
+	// tell the client we've registered to unblock the RPC we test below
+	c1.registeredOnce.Do(func() { close(c1.registeredCh) })
+
 	req := structs.NodeSpecificRequest{
 		NodeID:       c1.Node().ID,
 		QueryOptions: structs.QueryOptions{Region: "global"},
@@ -338,7 +353,7 @@ func TestClient_BadTLS(t *testing.T) {
 	testutil.AssertUntil(100*time.Millisecond,
 		func() (bool, error) {
 			err := c1.RPC("Node.GetNode", &req, &out)
-			if err == nil {
+			if err == nil || structs.IsErrPermissionDenied(err) {
 				return false, fmt.Errorf("client RPC succeeded when it should have failed:\n%+v", out)
 			}
 			return true, nil
@@ -382,6 +397,82 @@ func TestClient_Register(t *testing.T) {
 	})
 }
 
+func TestClient_Register_NodePool(t *testing.T) {
+	ci.Parallel(t)
+
+	s1, _, cleanupS1 := testServer(t, nil)
+	defer cleanupS1()
+	testutil.WaitForLeader(t, s1.RPC)
+
+	// Create client with a node pool configured.
+	c1, cleanupC1 := TestClient(t, func(c *config.Config) {
+		c.RPCHandler = s1
+		c.Node.NodePool = "dev"
+	})
+	defer cleanupC1()
+
+	// Create client with no node pool configured.
+	c2, cleanupC2 := TestClient(t, func(c *config.Config) {
+		c.RPCHandler = s1
+		c.Node.NodePool = ""
+	})
+	defer cleanupC2()
+
+	nodeReq := structs.NodeSpecificRequest{
+		QueryOptions: structs.QueryOptions{Region: "global"},
+	}
+	var nodeResp structs.SingleNodeResponse
+
+	poolReq := structs.NodePoolSpecificRequest{
+		Name:         "dev",
+		QueryOptions: structs.QueryOptions{Region: "global"},
+	}
+	var poolResp structs.SingleNodePoolResponse
+
+	// Register should succeed and node pool should be created.
+	// Client without node pool configuration should be in the default pool.
+	testutil.WaitForResult(func() (bool, error) {
+		// Fetch node1.
+		nodeReq.NodeID = c1.Node().ID
+		err := s1.RPC("Node.GetNode", &nodeReq, &nodeResp)
+		if err != nil {
+			return false, err
+		}
+		if nodeResp.Node == nil {
+			return false, fmt.Errorf("c1 is missing")
+		}
+		if nodeResp.Node.NodePool != "dev" {
+			return false, fmt.Errorf("c1 has wrong node pool")
+		}
+
+		// Fetch node1 node pool.
+		err = s1.RPC("NodePool.GetNodePool", &poolReq, &poolResp)
+		if err != nil {
+			return false, err
+		}
+		if poolResp.NodePool == nil {
+			return false, fmt.Errorf("dev node pool is nil")
+		}
+
+		// Fetch node2.
+		nodeReq.NodeID = c2.Node().ID
+		err = s1.RPC("Node.GetNode", &nodeReq, &nodeResp)
+		if err != nil {
+			return false, err
+		}
+		if nodeResp.Node == nil {
+			return false, fmt.Errorf("c2 is missing")
+		}
+		if nodeResp.Node.NodePool != structs.NodePoolDefault {
+			return false, fmt.Errorf("c2 has wrong node pool")
+		}
+
+		return true, nil
+	}, func(err error) {
+		t.Fatalf("err: %v", err)
+	})
+}
+
 func TestClient_Heartbeat(t *testing.T) {
 	ci.Parallel(t)
 
@@ -478,7 +569,7 @@ func TestClient_WatchAllocs(t *testing.T) {
 	alloc2.Job = job
 
 	state := s1.State()
-	if err := state.UpsertJob(structs.MsgTypeTestSetup, 100, job); err != nil {
+	if err := state.UpsertJob(structs.MsgTypeTestSetup, 100, nil, job); err != nil {
 		t.Fatal(err)
 	}
 	if err := state.UpsertJobSummary(101, mock.JobSummary(alloc1.JobID)); err != nil {
@@ -546,67 +637,175 @@ func waitTilNodeReady(client *Client, t *testing.T) {
 	})
 }
 
+// TestClient_SaveRestoreState exercises the allocrunner restore code paths
+// after a client restart. It runs several jobs in different states and asserts
+// the expected final state and server updates.
 func TestClient_SaveRestoreState(t *testing.T) {
 	ci.Parallel(t)
 
 	s1, _, cleanupS1 := testServer(t, nil)
-	defer cleanupS1()
+	t.Cleanup(cleanupS1)
 	testutil.WaitForLeader(t, s1.RPC)
 
 	c1, cleanupC1 := TestClient(t, func(c *config.Config) {
 		c.DevMode = false
 		c.RPCHandler = s1
 	})
-	defer cleanupC1()
+	t.Cleanup(func() {
+		for _, ar := range c1.getAllocRunners() {
+			ar.Destroy()
+		}
+		for _, ar := range c1.getAllocRunners() {
+			<-ar.DestroyCh()
+		}
+		cleanupC1()
+	})
 
 	// Wait until the node is ready
 	waitTilNodeReady(c1, t)
 
-	// Create mock allocations
-	job := mock.Job()
-	alloc1 := mock.Alloc()
-	alloc1.NodeID = c1.Node().ID
-	alloc1.Job = job
-	alloc1.JobID = job.ID
-	alloc1.Job.TaskGroups[0].Tasks[0].Driver = "mock_driver"
-	alloc1.Job.TaskGroups[0].Tasks[0].Config = map[string]interface{}{
-		"run_for": "10s",
-	}
-	alloc1.ClientStatus = structs.AllocClientStatusRunning
+	migrateStrategy := structs.DefaultMigrateStrategy()
+	migrateStrategy.MinHealthyTime = time.Millisecond
+	migrateStrategy.HealthCheck = structs.MigrateStrategyHealthStates
 
-	state := s1.State()
-	if err := state.UpsertJob(structs.MsgTypeTestSetup, 100, job); err != nil {
-		t.Fatal(err)
-	}
-	if err := state.UpsertJobSummary(101, mock.JobSummary(alloc1.JobID)); err != nil {
-		t.Fatal(err)
-	}
-	if err := state.UpsertAllocs(structs.MsgTypeTestSetup, 102, []*structs.Allocation{alloc1}); err != nil {
-		t.Fatalf("err: %v", err)
+	// Create mock jobs and allocations that will start up fast
+
+	setup := func(id string) *structs.Job {
+		job := mock.MinJob()
+		job.ID = id
+		job.TaskGroups[0].Migrate = migrateStrategy
+		must.NoError(t, s1.RPC("Job.Register", &structs.JobRegisterRequest{
+			Job:          job,
+			WriteRequest: structs.WriteRequest{Region: "global", Namespace: job.Namespace},
+		}, &structs.JobRegisterResponse{}))
+		return job
 	}
 
-	// Allocations should get registered
-	testutil.WaitForResult(func() (bool, error) {
-		c1.allocLock.RLock()
-		ar := c1.allocs[alloc1.ID]
-		c1.allocLock.RUnlock()
-		if ar == nil {
-			return false, fmt.Errorf("nil alloc runner")
-		}
-		if ar.Alloc().ClientStatus != structs.AllocClientStatusRunning {
-			return false, fmt.Errorf("client status: got %v; want %v", ar.Alloc().ClientStatus, structs.AllocClientStatusRunning)
-		}
-		return true, nil
-	}, func(err error) {
-		t.Fatalf("err: %v", err)
-	})
+	// job1: will be left running
+	// job2: will be stopped before shutdown
+	// job3: will be stopped after shutdown
+	// job4: will be stopped and GC'd after shutdown
+	job1, job2, job3, job4 := setup("job1"), setup("job2"), setup("job3"), setup("job4")
 
-	// Shutdown the client, saves state
-	if err := c1.Shutdown(); err != nil {
-		t.Fatalf("err: %v", err)
+	// Allocations should be placed
+	must.Wait(t, wait.InitialSuccess(
+		wait.ErrorFunc(func() error {
+			c1.allocLock.RLock()
+			defer c1.allocLock.RUnlock()
+			if len(c1.allocs) != 4 {
+				return fmt.Errorf("expected 4 alloc runners")
+			}
+			for _, ar := range c1.allocs {
+				if ar.AllocState().ClientStatus != structs.AllocClientStatusRunning {
+					return fmt.Errorf("expected running client status, got %v",
+						ar.AllocState().ClientStatus)
+				}
+			}
+			return nil
+		}),
+		wait.Timeout(time.Second*10),
+		wait.Gap(time.Millisecond*30),
+	))
+
+	store := s1.State()
+
+	allocIDforJob := func(job *structs.Job) string {
+		allocs, err := store.AllocsByJob(nil, job.Namespace, job.ID, false)
+		must.NoError(t, err)
+		must.Len(t, 1, allocs) // we should only ever get 1 in this test
+		return allocs[0].ID
 	}
+	alloc1 := allocIDforJob(job1)
+	alloc2 := allocIDforJob(job2)
+	alloc3 := allocIDforJob(job3)
+	alloc4 := allocIDforJob(job4)
+	t.Logf("alloc1=%s alloc2=%s alloc3=%s alloc4=%s", alloc1, alloc2, alloc3, alloc4)
+
+	// Stop the 2nd job before we shut down
+	must.NoError(t, s1.RPC("Job.Deregister", &structs.JobDeregisterRequest{
+		JobID:        job2.ID,
+		WriteRequest: structs.WriteRequest{Region: "global", Namespace: job2.Namespace},
+	}, &structs.JobDeregisterResponse{}))
+
+	var alloc2ModifyIndex uint64
+	var alloc2AllocModifyIndex uint64
+
+	// Wait till we're sure the client has received the stop and updated the server
+	must.Wait(t, wait.InitialSuccess(
+		wait.ErrorFunc(func() error {
+			alloc, err := store.AllocByID(nil, alloc2)
+			must.NotNil(t, alloc)
+			must.NoError(t, err)
+			if alloc.ClientStatus != structs.AllocClientStatusComplete {
+				// note that the allocrunner is non-nil until it's been
+				// client-GC'd, so we're just looking to make sure the client
+				// has updated the server
+				return fmt.Errorf("alloc2 should have been marked completed")
+			}
+			alloc2ModifyIndex = alloc.ModifyIndex
+			alloc2AllocModifyIndex = alloc.AllocModifyIndex
+			return nil
+		}),
+		wait.Timeout(time.Second*20),
+		wait.Gap(time.Millisecond*30),
+	))
+
+	t.Log("shutting down client")
+	must.NoError(t, c1.Shutdown()) // note: this saves the client state DB
+
+	// Stop the 3rd job while we're down
+	must.NoError(t, s1.RPC("Job.Deregister", &structs.JobDeregisterRequest{
+		JobID:        job3.ID,
+		WriteRequest: structs.WriteRequest{Region: "global", Namespace: job3.Namespace},
+	}, &structs.JobDeregisterResponse{}))
+
+	// Stop and purge the 4th job while we're down
+	must.NoError(t, s1.RPC("Job.Deregister", &structs.JobDeregisterRequest{
+		JobID:        job4.ID,
+		Purge:        true,
+		WriteRequest: structs.WriteRequest{Region: "global", Namespace: job4.Namespace},
+	}, &structs.JobDeregisterResponse{}))
+
+	// Ensure the allocation has been deleted as well
+	must.NoError(t, s1.RPC("Eval.Reap", &structs.EvalReapRequest{
+		Allocs:       []string{alloc4},
+		WriteRequest: structs.WriteRequest{Region: "global"},
+	}, &structs.GenericResponse{}))
+
+	var alloc3AllocModifyIndex uint64
+	var alloc3ModifyIndex uint64
+
+	// Wait till we're sure the scheduler has marked alloc3 for stop and deleted alloc4
+	must.Wait(t, wait.InitialSuccess(
+		wait.ErrorFunc(func() error {
+			alloc, err := store.AllocByID(nil, alloc3)
+			must.NotNil(t, alloc)
+			must.NoError(t, err)
+			if alloc.DesiredStatus != structs.AllocDesiredStatusStop {
+				return fmt.Errorf("alloc3 should have been marked for stop")
+			}
+			alloc3ModifyIndex = alloc.ModifyIndex
+			alloc3AllocModifyIndex = alloc.AllocModifyIndex
+
+			alloc, err = store.AllocByID(nil, alloc4)
+			must.NoError(t, err)
+			if alloc != nil {
+				return fmt.Errorf("alloc4 should have been deleted")
+			}
+			return nil
+		}),
+		wait.Timeout(time.Second*5),
+		wait.Gap(time.Millisecond*30),
+	))
+
+	a1, err := store.AllocByID(nil, alloc1)
+	var alloc1AllocModifyIndex uint64
+	var alloc1ModifyIndex uint64
+	alloc1ModifyIndex = a1.ModifyIndex
+	alloc1AllocModifyIndex = a1.AllocModifyIndex
+
+	t.Log("starting new client")
 
-	// Create a new client
 	logger := testlog.HCLogger(t)
 	c1.config.Logger = logger
 	consulCatalog := consul.NewMockCatalog(logger)
@@ -617,34 +816,105 @@ func TestClient_SaveRestoreState(t *testing.T) {
 	c1.config.PluginSingletonLoader = singleton.NewSingletonLoader(logger, c1.config.PluginLoader)
 
 	c2, err := NewClient(c1.config, consulCatalog, nil, mockService, nil)
-	if err != nil {
-		t.Fatalf("err: %v", err)
-	}
-	defer c2.Shutdown()
+	must.NoError(t, err)
 
-	// Ensure the allocation is running
-	testutil.WaitForResult(func() (bool, error) {
-		c2.allocLock.RLock()
-		ar := c2.allocs[alloc1.ID]
-		c2.allocLock.RUnlock()
-		status := ar.Alloc().ClientStatus
-		alive := status == structs.AllocClientStatusRunning || status == structs.AllocClientStatusPending
-		if !alive {
-			return false, fmt.Errorf("incorrect client status: %#v", ar.Alloc())
+	t.Cleanup(func() {
+		for _, ar := range c2.getAllocRunners() {
+			ar.Destroy()
 		}
-		return true, nil
-	}, func(err error) {
-		t.Fatalf("err: %v", err)
+		for _, ar := range c2.getAllocRunners() {
+			<-ar.DestroyCh()
+		}
+		c2.Shutdown()
 	})
 
-	// Destroy all the allocations
-	for _, ar := range c2.getAllocRunners() {
-		ar.Destroy()
-	}
+	// Ensure only the expected allocation is running
+	must.Wait(t, wait.InitialSuccess(
+		wait.ErrorFunc(func() error {
+			c2.allocLock.RLock()
+			defer c2.allocLock.RUnlock()
+			if len(c2.allocs) != 3 {
+				// the GC'd alloc will not have restored AR
+				return fmt.Errorf("expected 3 alloc runners")
+			}
+			for allocID, ar := range c2.allocs {
+				if ar == nil {
+					return fmt.Errorf("nil alloc runner")
+				}
+				switch allocID {
+				case alloc1:
+					if ar.AllocState().ClientStatus != structs.AllocClientStatusRunning {
+						return fmt.Errorf("expected running client status, got %v",
+							ar.AllocState().ClientStatus)
+					}
+
+				case alloc3:
+					if ar.AllocState().ClientStatus != structs.AllocClientStatusComplete {
+						return fmt.Errorf("expected complete client status, got %v",
+							ar.AllocState().ClientStatus)
+					}
+
+					// because the client's update will be batched, we need to
+					// ensure we wait for the server update too
+					a3, err := store.AllocByID(nil, alloc3)
+					must.NoError(t, err)
+					must.NotNil(t, a3)
+					if alloc3AllocModifyIndex != a3.AllocModifyIndex ||
+						alloc3ModifyIndex >= a3.ModifyIndex {
+						return fmt.Errorf(
+							"alloc %s stopped during shutdown should have updated", a3.ID[:8])
+					}
+
+				default:
+					if ar.AllocState().ClientStatus != structs.AllocClientStatusComplete {
+						return fmt.Errorf("expected complete client status, got %v",
+							ar.AllocState().ClientStatus)
+					}
+				}
+			}
+			return nil
+		}),
+		wait.Timeout(time.Second*10),
+		wait.Gap(time.Millisecond*30),
+	))
+
+	// Because we're asserting that no changes have been made, we have to wait a
+	// sufficient amount of time to verify that
+	must.Wait(t, wait.ContinualSuccess(
+		wait.ErrorFunc(func() error {
+			a1, err = store.AllocByID(nil, alloc1)
+			must.NoError(t, err)
+			must.NotNil(t, a1)
+
+			if alloc1AllocModifyIndex != a1.AllocModifyIndex ||
+				alloc1ModifyIndex != a1.ModifyIndex {
+				return fmt.Errorf("alloc still running should not have updated")
+			}
+
+			a2, err := store.AllocByID(nil, alloc2)
+			must.NoError(t, err)
+			must.NotNil(t, a2)
+			if alloc2AllocModifyIndex != a2.AllocModifyIndex ||
+				alloc2ModifyIndex != a2.ModifyIndex {
+				return fmt.Errorf(
+					"alloc %s stopped before shutdown should not have updated", a2.ID[:8])
+			}
+
+			// TODO: the alloc has been GC'd so the server will reject any
+			// update. It'd be nice if we could instrument the server here to
+			// ensure we didn't send one either.
+			a4, err := store.AllocByID(nil, alloc4)
+			must.NoError(t, err)
+			if a4 != nil {
+				return fmt.Errorf("garbage collected alloc should not exist")
+			}
+
+			return nil
+		}),
+		wait.Timeout(time.Second*3),
+		wait.Gap(time.Millisecond*100),
+	))
 
-	for _, ar := range c2.getAllocRunners() {
-		<-ar.DestroyCh()
-	}
 }
 
 func TestClient_AddAllocError(t *testing.T) {
@@ -681,7 +951,7 @@ func TestClient_AddAllocError(t *testing.T) {
 	alloc1.TaskResources = nil
 
 	state := s1.State()
-	err := state.UpsertJob(structs.MsgTypeTestSetup, 100, job)
+	err := state.UpsertJob(structs.MsgTypeTestSetup, 100, nil, job)
 	require.Nil(err)
 
 	err = state.UpsertJobSummary(101, mock.JobSummary(alloc1.JobID))
@@ -918,9 +1188,9 @@ func TestClient_ReloadTLS_UpgradePlaintextToTLS(t *testing.T) {
 	testutil.WaitForLeader(t, s1.RPC)
 
 	const (
-		cafile  = "../helper/tlsutil/testdata/ca.pem"
-		foocert = "../helper/tlsutil/testdata/nomad-foo.pem"
-		fookey  = "../helper/tlsutil/testdata/nomad-foo-key.pem"
+		cafile        = "../helper/tlsutil/testdata/nomad-agent-ca.pem"
+		fooclientcert = "../helper/tlsutil/testdata/regionFoo-client-nomad.pem"
+		fooclientkey  = "../helper/tlsutil/testdata/regionFoo-client-nomad-key.pem"
 	)
 
 	c1, cleanup := TestClient(t, func(c *config.Config) {
@@ -954,8 +1224,8 @@ func TestClient_ReloadTLS_UpgradePlaintextToTLS(t *testing.T) {
 		EnableRPC:            true,
 		VerifyServerHostname: true,
 		CAFile:               cafile,
-		CertFile:             foocert,
-		KeyFile:              fookey,
+		CertFile:             fooclientcert,
+		KeyFile:              fooclientkey,
 	}
 
 	err := c1.reloadTLSConnections(newConfig)
@@ -994,9 +1264,9 @@ func TestClient_ReloadTLS_DowngradeTLSToPlaintext(t *testing.T) {
 	testutil.WaitForLeader(t, s1.RPC)
 
 	const (
-		cafile  = "../helper/tlsutil/testdata/ca.pem"
-		foocert = "../helper/tlsutil/testdata/nomad-foo.pem"
-		fookey  = "../helper/tlsutil/testdata/nomad-foo-key.pem"
+		cafile        = "../helper/tlsutil/testdata/nomad-agent-ca.pem"
+		fooclientcert = "../helper/tlsutil/testdata/regionFoo-client-nomad.pem"
+		fooclientkey  = "../helper/tlsutil/testdata/regionFoo-client-nomad-key.pem"
 	)
 
 	c1, cleanup := TestClient(t, func(c *config.Config) {
@@ -1006,12 +1276,15 @@ func TestClient_ReloadTLS_DowngradeTLSToPlaintext(t *testing.T) {
 			EnableRPC:            true,
 			VerifyServerHostname: true,
 			CAFile:               cafile,
-			CertFile:             foocert,
-			KeyFile:              fookey,
+			CertFile:             fooclientcert,
+			KeyFile:              fooclientkey,
 		}
 	})
 	defer cleanup()
 
+	// tell the client we've registered to unblock the RPC we test below
+	c1.registeredOnce.Do(func() { close(c1.registeredCh) })
+
 	// assert that when one node is running in encrypted mode, a RPC request to a
 	// node running in plaintext mode should fail
 	{
@@ -1022,7 +1295,7 @@ func TestClient_ReloadTLS_DowngradeTLSToPlaintext(t *testing.T) {
 		testutil.WaitForResult(func() (bool, error) {
 			var out structs.SingleNodeResponse
 			err := c1.RPC("Node.GetNode", &req, &out)
-			if err == nil {
+			if err == nil || structs.IsErrPermissionDenied(err) {
 				return false, fmt.Errorf("client RPC succeeded when it should have failed :\n%+v", err)
 			}
 			return true, nil
@@ -1736,7 +2009,7 @@ func TestClient_ReconnectAllocs(t *testing.T) {
 	runningAlloc.ClientStatus = structs.AllocClientStatusPending
 
 	state := s1.State()
-	err := state.UpsertJob(structs.MsgTypeTestSetup, 100, job)
+	err := state.UpsertJob(structs.MsgTypeTestSetup, 100, nil, job)
 	require.NoError(t, err)
 
 	err = state.UpsertJobSummary(101, mock.JobSummary(runningAlloc.JobID))
@@ -1772,7 +2045,7 @@ func TestClient_ReconnectAllocs(t *testing.T) {
 	c1.runAllocs(updates)
 
 	invalid := false
-	var runner AllocRunner
+	var runner interfaces.AllocRunner
 	var finalAlloc *structs.Allocation
 	// Ensure the allocation is not invalid on the client and has been marked
 	// running on the server with the new modify index
@@ -1793,3 +2066,89 @@ func TestClient_ReconnectAllocs(t *testing.T) {
 	require.False(t, invalid, "expected alloc to not be marked invalid")
 	require.Equal(t, unknownAlloc.AllocModifyIndex, finalAlloc.AllocModifyIndex)
 }
+
+// TestClient_AllocPrerunErrorDuringRestore ensures that a running allocation,
+// which fails Prerun during Restore on client restart, should be killed.
+func TestClient_AllocPrerunErrorDuringRestore(t *testing.T) {
+	ci.Parallel(t)
+
+	logger := testlog.HCLogger(t)
+
+	// set up server
+	server, _, cleanS1 := testServer(t, nil)
+	t.Cleanup(cleanS1)
+	testutil.WaitForLeader(t, server.RPC)
+
+	// set up first client, which will initially start the job cleanly
+	c1, cleanC1 := TestClient(t, func(c *config.Config) {
+		c.DevMode = false // so state persists to client 2
+		c.RPCHandler = server
+	})
+	t.Cleanup(func() {
+		test.NoError(t, cleanC1())
+	})
+	waitTilNodeReady(c1, t)
+
+	// register a happy job to run until we cause it to fail
+	job := mock.MinJob()
+	testutil.RegisterJob(t, server.RPC, job)
+
+	// wait for our alloc to be running
+	testutil.WaitForJobAllocStatus(t, server.RPC, job, map[string]int{
+		structs.AllocClientStatusRunning: 1,
+	})
+	t.Logf("job %s allocs running 👍", job.ID)
+
+	// stop client 1, shutdown will dump state to disk but leave allocs running
+	must.NoError(t, c1.Shutdown())
+
+	// make a new client, using parts from the old one to be able to restore state
+	restoreClient := func() {
+		conf := c1.config.Copy()
+		// we want the prerun hook to fail
+		hook := allocrunner.NewFailHook(logger, t.Name())
+		hook.Fail.Prerun = true
+		conf.ExtraAllocHooks = []interfaces.RunnerHook{hook}
+
+		// this is so in-memory driver handles from client 1 can be restored by client 2
+		conf.PluginSingletonLoader = singleton.NewSingletonLoader(logger, c1.config.PluginLoader)
+
+		// actually make and start the client
+		c2, err := NewClient(conf, c1.consulCatalog, nil, c1.consulService, nil)
+		must.NoError(t, err)
+		t.Cleanup(func() {
+			test.NoError(t, c2.Shutdown())
+		})
+	}
+	restoreClient()
+
+	// wait for the client to pick up the alloc and fail prerun hook
+	testutil.WaitForJobAllocStatus(t, server.RPC, job, map[string]int{
+		structs.AllocClientStatusFailed: 1,
+	})
+	t.Logf("job %s allocs failed 👍", job.ID)
+
+	// ok, final assertions
+	allocs, err := server.State().AllocsByJob(nil, job.Namespace, job.ID, true)
+	must.NoError(t, err)
+
+	ts := allocs[0].TaskStates["t"]
+	test.True(t, ts.Failed)
+	test.Eq(t, structs.TaskStateDead, ts.State)
+
+	expectEvents := []string{
+		// initial successful setup
+		structs.TaskReceived,
+		structs.TaskSetup,
+		structs.TaskStarted,
+		// after prerun error during restore
+		structs.TaskSetupFailure,
+		structs.TaskTerminated, // this whole test is to ensure this happens.
+	}
+	var actual []string
+	for _, event := range ts.Events {
+		actual = append(actual, event.Type)
+	}
+	must.Eq(t, expectEvents, actual)
+	test.StrContains(t, ts.Events[3].DisplayMessage, allocrunner.ErrFailHookError.Error())
+}
