@@ -1,3 +1,6 @@
+// Copyright (c) HashiCorp, Inc.
+// SPDX-License-Identifier: MPL-2.0
+
 package state
 
 import (
@@ -8,11 +11,13 @@ import (
 	"time"
 
 	hclog "github.com/hashicorp/go-hclog"
+	arstate "github.com/hashicorp/nomad/client/allocrunner/state"
 	trstate "github.com/hashicorp/nomad/client/allocrunner/taskrunner/state"
 	dmstate "github.com/hashicorp/nomad/client/devicemanager/state"
 	"github.com/hashicorp/nomad/client/dynamicplugins"
 	driverstate "github.com/hashicorp/nomad/client/pluginmanager/drivermanager/state"
 	"github.com/hashicorp/nomad/client/serviceregistration/checks"
+	cstructs "github.com/hashicorp/nomad/client/structs"
 	"github.com/hashicorp/nomad/helper/boltdd"
 	"github.com/hashicorp/nomad/nomad/structs"
 	"go.etcd.io/bbolt"
@@ -29,6 +34,8 @@ allocations/
    |--> alloc          -> allocEntry{*structs.Allocation}
 	 |--> deploy_status  -> deployStatusEntry{*structs.AllocDeploymentStatus}
 	 |--> network_status -> networkStatusEntry{*structs.AllocNetworkStatus}
+	 |--> acknowledged_state -> acknowledgedStateEntry{*arstate.State}
+	 |--> alloc_volumes -> allocVolumeStatesEntry{arstate.AllocVolumes}
    |--> task-<name>/
       |--> local_state -> *trstate.LocalState # Local-only state
       |--> task_state  -> *structs.TaskState  # Syncs to servers
@@ -46,6 +53,9 @@ dynamicplugins/
 
 nodemeta/
 |--> meta -> map[string]*string
+
+node/
+|--> registration -> *cstructs.NodeRegistration
 */
 
 var (
@@ -80,6 +90,11 @@ var (
 	// stored under
 	allocNetworkStatusKey = []byte("network_status")
 
+	// acknowledgedStateKey is the key *arstate.State is stored under
+	acknowledgedStateKey = []byte("acknowledged_state")
+
+	allocVolumeKey = []byte("alloc_volume")
+
 	// checkResultsBucket is the bucket name in which check query results are stored
 	checkResultsBucket = []byte("check_results")
 
@@ -112,6 +127,12 @@ var (
 
 	// nodeMetaKey is the key at which dynamic node metadata is stored.
 	nodeMetaKey = []byte("meta")
+
+	// nodeBucket is the bucket name in which data about the node is stored.
+	nodeBucket = []byte("node")
+
+	// nodeRegistrationKey is the key at which node registration data is stored.
+	nodeRegistrationKey = []byte("node_registration")
 )
 
 // taskBucketName returns the bucket name for the given task name.
@@ -389,6 +410,107 @@ func (s *BoltStateDB) GetNetworkStatus(allocID string) (*structs.AllocNetworkSta
 	return entry.NetworkStatus, nil
 }
 
+// PutAcknowledgedState stores an allocation's last acknowledged state or
+// returns an error if it could not be stored.
+func (s *BoltStateDB) PutAcknowledgedState(allocID string, state *arstate.State, opts ...WriteOption) error {
+	return s.updateWithOptions(opts, func(tx *boltdd.Tx) error {
+		allocBkt, err := getAllocationBucket(tx, allocID)
+		if err != nil {
+			return err
+		}
+
+		entry := acknowledgedStateEntry{
+			State: state,
+		}
+		return allocBkt.Put(acknowledgedStateKey, &entry)
+	})
+}
+
+// GetAcknowledgedState retrieves an allocation's last acknowledged state
+func (s *BoltStateDB) GetAcknowledgedState(allocID string) (*arstate.State, error) {
+	var entry acknowledgedStateEntry
+
+	err := s.db.View(func(tx *boltdd.Tx) error {
+		allAllocsBkt := tx.Bucket(allocationsBucketName)
+		if allAllocsBkt == nil {
+			// No state, return
+			return nil
+		}
+
+		allocBkt := allAllocsBkt.Bucket([]byte(allocID))
+		if allocBkt == nil {
+			// No state for alloc, return
+			return nil
+		}
+
+		return allocBkt.Get(acknowledgedStateKey, &entry)
+	})
+
+	// It's valid for this field to be nil/missing
+	if boltdd.IsErrNotFound(err) {
+		return nil, nil
+	}
+
+	if err != nil {
+		return nil, err
+	}
+
+	return entry.State, nil
+}
+
+type allocVolumeStatesEntry struct {
+	State *arstate.AllocVolumes
+}
+
+// PutAllocVolumes stores stubs of an allocation's dynamic volume mounts so they
+// can be restored.
+func (s *BoltStateDB) PutAllocVolumes(allocID string, state *arstate.AllocVolumes, opts ...WriteOption) error {
+	return s.updateWithOptions(opts, func(tx *boltdd.Tx) error {
+		allocBkt, err := getAllocationBucket(tx, allocID)
+		if err != nil {
+			return err
+		}
+
+		entry := allocVolumeStatesEntry{
+			State: state,
+		}
+		return allocBkt.Put(allocVolumeKey, &entry)
+	})
+}
+
+// GetAllocVolumes retrieves stubs of an allocation's dynamic volume mounts so
+// they can be restored.
+func (s *BoltStateDB) GetAllocVolumes(allocID string) (*arstate.AllocVolumes, error) {
+	var entry allocVolumeStatesEntry
+
+	err := s.db.View(func(tx *boltdd.Tx) error {
+		allAllocsBkt := tx.Bucket(allocationsBucketName)
+		if allAllocsBkt == nil {
+			// No state, return
+			return nil
+		}
+
+		allocBkt := allAllocsBkt.Bucket([]byte(allocID))
+		if allocBkt == nil {
+			// No state for alloc, return
+			return nil
+		}
+
+		return allocBkt.Get(allocVolumeKey, &entry)
+	})
+
+	// It's valid for this field to be nil/missing
+	if boltdd.IsErrNotFound(err) {
+		return nil, nil
+	}
+
+	if err != nil {
+		return nil, err
+	}
+
+	return entry.State, nil
+}
+
 // GetTaskRunnerState returns the LocalState and TaskState for a
 // TaskRunner. LocalState or TaskState will be nil if they do not exist.
 //
@@ -841,6 +963,34 @@ func getNodeMeta(b *boltdd.Bucket) (map[string]*string, error) {
 	return m, nil
 }
 
+func (s *BoltStateDB) PutNodeRegistration(reg *cstructs.NodeRegistration) error {
+	return s.db.Update(func(tx *boltdd.Tx) error {
+		b, err := tx.CreateBucketIfNotExists(nodeBucket)
+		if err != nil {
+			return err
+		}
+
+		return b.Put(nodeRegistrationKey, reg)
+	})
+}
+
+func (s *BoltStateDB) GetNodeRegistration() (*cstructs.NodeRegistration, error) {
+	var reg cstructs.NodeRegistration
+	err := s.db.View(func(tx *boltdd.Tx) error {
+		b := tx.Bucket(nodeBucket)
+		if b == nil {
+			return nil
+		}
+		return b.Get(nodeRegistrationKey, &reg)
+	})
+
+	if boltdd.IsErrNotFound(err) {
+		return nil, nil
+	}
+
+	return &reg, err
+}
+
 // init initializes metadata entries in a newly created state database.
 func (s *BoltStateDB) init() error {
 	return s.db.Update(func(tx *boltdd.Tx) error {
@@ -848,6 +998,12 @@ func (s *BoltStateDB) init() error {
 	})
 }
 
+// acknowledgedStateEntry wraps values in the acknowledged_state bucket, so we
+// can expand it in the future if need be
+type acknowledgedStateEntry struct {
+	State *arstate.State
+}
+
 // updateWithOptions enables adjustments to db.Update operation, including Batch mode.
 func (s *BoltStateDB) updateWithOptions(opts []WriteOption, updateFn func(tx *boltdd.Tx) error) error {
 	writeOpts := mergeWriteOptions(opts)
