@@ -1,3 +1,6 @@
+// Copyright (c) HashiCorp, Inc.
+// SPDX-License-Identifier: MPL-2.0
+
 package consul
 
 import (
@@ -969,6 +972,7 @@ func TestCreateCheckReg_GRPC(t *testing.T) {
 		PortLabel:     "label",
 		GRPCService:   "foo.Bar",
 		GRPCUseTLS:    true,
+		TLSServerName: "localhost",
 		TLSSkipVerify: true,
 		Timeout:       time.Second,
 		Interval:      time.Minute,
@@ -985,13 +989,14 @@ func TestCreateCheckReg_GRPC(t *testing.T) {
 		AgentServiceCheck: api.AgentServiceCheck{
 			Timeout:       "1s",
 			Interval:      "1m0s",
-			GRPC:          "localhost:8080/foo.Bar",
+			GRPC:          "127.0.0.1:8080/foo.Bar",
 			GRPCUseTLS:    true,
+			TLSServerName: "localhost",
 			TLSSkipVerify: true,
 		},
 	}
 
-	actual, err := createCheckReg(serviceID, checkID, check, "localhost", 8080, "default")
+	actual, err := createCheckReg(serviceID, checkID, check, "127.0.0.1", 8080, "default")
 	must.NoError(t, err)
 	must.Eq(t, expected, actual)
 }
