@@ -1,3 +1,6 @@
+// Copyright (c) HashiCorp, Inc.
+// SPDX-License-Identifier: MPL-2.0
+
 package agent
 
 import (
@@ -63,54 +66,56 @@ func (s *HTTPServer) JobSpecificRequest(resp http.ResponseWriter, req *http.Requ
 	path := strings.TrimPrefix(req.URL.Path, "/v1/job/")
 	switch {
 	case strings.HasSuffix(path, "/evaluate"):
-		jobName := strings.TrimSuffix(path, "/evaluate")
-		return s.jobForceEvaluate(resp, req, jobName)
+		jobID := strings.TrimSuffix(path, "/evaluate")
+		return s.jobForceEvaluate(resp, req, jobID)
 	case strings.HasSuffix(path, "/allocations"):
-		jobName := strings.TrimSuffix(path, "/allocations")
-		return s.jobAllocations(resp, req, jobName)
+		jobID := strings.TrimSuffix(path, "/allocations")
+		return s.jobAllocations(resp, req, jobID)
 	case strings.HasSuffix(path, "/evaluations"):
-		jobName := strings.TrimSuffix(path, "/evaluations")
-		return s.jobEvaluations(resp, req, jobName)
+		jobID := strings.TrimSuffix(path, "/evaluations")
+		return s.jobEvaluations(resp, req, jobID)
 	case strings.HasSuffix(path, "/periodic/force"):
-		jobName := strings.TrimSuffix(path, "/periodic/force")
-		return s.periodicForceRequest(resp, req, jobName)
+		jobID := strings.TrimSuffix(path, "/periodic/force")
+		return s.periodicForceRequest(resp, req, jobID)
 	case strings.HasSuffix(path, "/plan"):
-		jobName := strings.TrimSuffix(path, "/plan")
-		return s.jobPlan(resp, req, jobName)
+		jobID := strings.TrimSuffix(path, "/plan")
+		return s.jobPlan(resp, req, jobID)
 	case strings.HasSuffix(path, "/summary"):
-		jobName := strings.TrimSuffix(path, "/summary")
-		return s.jobSummaryRequest(resp, req, jobName)
+		jobID := strings.TrimSuffix(path, "/summary")
+		return s.jobSummaryRequest(resp, req, jobID)
 	case strings.HasSuffix(path, "/dispatch"):
-		jobName := strings.TrimSuffix(path, "/dispatch")
-		return s.jobDispatchRequest(resp, req, jobName)
+		jobID := strings.TrimSuffix(path, "/dispatch")
+		return s.jobDispatchRequest(resp, req, jobID)
 	case strings.HasSuffix(path, "/versions"):
-		jobName := strings.TrimSuffix(path, "/versions")
-		return s.jobVersions(resp, req, jobName)
+		jobID := strings.TrimSuffix(path, "/versions")
+		return s.jobVersions(resp, req, jobID)
 	case strings.HasSuffix(path, "/revert"):
-		jobName := strings.TrimSuffix(path, "/revert")
-		return s.jobRevert(resp, req, jobName)
+		jobID := strings.TrimSuffix(path, "/revert")
+		return s.jobRevert(resp, req, jobID)
 	case strings.HasSuffix(path, "/deployments"):
-		jobName := strings.TrimSuffix(path, "/deployments")
-		return s.jobDeployments(resp, req, jobName)
+		jobID := strings.TrimSuffix(path, "/deployments")
+		return s.jobDeployments(resp, req, jobID)
 	case strings.HasSuffix(path, "/deployment"):
-		jobName := strings.TrimSuffix(path, "/deployment")
-		return s.jobLatestDeployment(resp, req, jobName)
+		jobID := strings.TrimSuffix(path, "/deployment")
+		return s.jobLatestDeployment(resp, req, jobID)
 	case strings.HasSuffix(path, "/stable"):
-		jobName := strings.TrimSuffix(path, "/stable")
-		return s.jobStable(resp, req, jobName)
+		jobID := strings.TrimSuffix(path, "/stable")
+		return s.jobStable(resp, req, jobID)
 	case strings.HasSuffix(path, "/scale"):
-		jobName := strings.TrimSuffix(path, "/scale")
-		return s.jobScale(resp, req, jobName)
+		jobID := strings.TrimSuffix(path, "/scale")
+		return s.jobScale(resp, req, jobID)
 	case strings.HasSuffix(path, "/services"):
-		jobName := strings.TrimSuffix(path, "/services")
-		return s.jobServiceRegistrations(resp, req, jobName)
+		jobID := strings.TrimSuffix(path, "/services")
+		return s.jobServiceRegistrations(resp, req, jobID)
+	case strings.HasSuffix(path, "/submission"):
+		jobID := strings.TrimSuffix(path, "/submission")
+		return s.jobSubmissionCRUD(resp, req, jobID)
 	default:
 		return s.jobCRUD(resp, req, path)
 	}
 }
 
-func (s *HTTPServer) jobForceEvaluate(resp http.ResponseWriter, req *http.Request,
-	jobName string) (interface{}, error) {
+func (s *HTTPServer) jobForceEvaluate(resp http.ResponseWriter, req *http.Request, jobID string) (interface{}, error) {
 	if req.Method != "PUT" && req.Method != "POST" {
 		return nil, CodedError(405, ErrInvalidMethod)
 	}
@@ -120,7 +125,7 @@ func (s *HTTPServer) jobForceEvaluate(resp http.ResponseWriter, req *http.Reques
 	// COMPAT: For backwards compatibility allow using this endpoint without a payload
 	if req.ContentLength == 0 {
 		args = structs.JobEvaluateRequest{
-			JobID: jobName,
+			JobID: jobID,
 		}
 	} else {
 		if err := decodeBody(req, &args); err != nil {
@@ -130,7 +135,7 @@ func (s *HTTPServer) jobForceEvaluate(resp http.ResponseWriter, req *http.Reques
 			return nil, CodedError(400, "Job ID must be specified")
 		}
 
-		if jobName != "" && args.JobID != jobName {
+		if jobID != "" && args.JobID != jobID {
 			return nil, CodedError(400, "JobID not same as job name")
 		}
 	}
@@ -231,15 +236,14 @@ func (s *HTTPServer) periodicForceRequest(resp http.ResponseWriter, req *http.Re
 	return out, nil
 }
 
-func (s *HTTPServer) jobAllocations(resp http.ResponseWriter, req *http.Request,
-	jobName string) (interface{}, error) {
+func (s *HTTPServer) jobAllocations(resp http.ResponseWriter, req *http.Request, jobID string) (interface{}, error) {
 	if req.Method != "GET" {
 		return nil, CodedError(405, ErrInvalidMethod)
 	}
 	allAllocs, _ := strconv.ParseBool(req.URL.Query().Get("all"))
 
 	args := structs.JobSpecificRequest{
-		JobID: jobName,
+		JobID: jobID,
 		All:   allAllocs,
 	}
 	if s.parse(resp, req, &args.Region, &args.QueryOptions) {
@@ -261,13 +265,12 @@ func (s *HTTPServer) jobAllocations(resp http.ResponseWriter, req *http.Request,
 	return out.Allocations, nil
 }
 
-func (s *HTTPServer) jobEvaluations(resp http.ResponseWriter, req *http.Request,
-	jobName string) (interface{}, error) {
+func (s *HTTPServer) jobEvaluations(resp http.ResponseWriter, req *http.Request, jobID string) (interface{}, error) {
 	if req.Method != "GET" {
 		return nil, CodedError(405, ErrInvalidMethod)
 	}
 	args := structs.JobSpecificRequest{
-		JobID: jobName,
+		JobID: jobID,
 	}
 	if s.parse(resp, req, &args.Region, &args.QueryOptions) {
 		return nil, nil
@@ -285,14 +288,13 @@ func (s *HTTPServer) jobEvaluations(resp http.ResponseWriter, req *http.Request,
 	return out.Evaluations, nil
 }
 
-func (s *HTTPServer) jobDeployments(resp http.ResponseWriter, req *http.Request,
-	jobName string) (interface{}, error) {
+func (s *HTTPServer) jobDeployments(resp http.ResponseWriter, req *http.Request, jobID string) (interface{}, error) {
 	if req.Method != "GET" {
 		return nil, CodedError(405, ErrInvalidMethod)
 	}
 	all, _ := strconv.ParseBool(req.URL.Query().Get("all"))
 	args := structs.JobSpecificRequest{
-		JobID: jobName,
+		JobID: jobID,
 		All:   all,
 	}
 	if s.parse(resp, req, &args.Region, &args.QueryOptions) {
@@ -311,13 +313,12 @@ func (s *HTTPServer) jobDeployments(resp http.ResponseWriter, req *http.Request,
 	return out.Deployments, nil
 }
 
-func (s *HTTPServer) jobLatestDeployment(resp http.ResponseWriter, req *http.Request,
-	jobName string) (interface{}, error) {
+func (s *HTTPServer) jobLatestDeployment(resp http.ResponseWriter, req *http.Request, jobID string) (interface{}, error) {
 	if req.Method != "GET" {
 		return nil, CodedError(405, ErrInvalidMethod)
 	}
 	args := structs.JobSpecificRequest{
-		JobID: jobName,
+		JobID: jobID,
 	}
 	if s.parse(resp, req, &args.Region, &args.QueryOptions) {
 		return nil, nil
@@ -332,24 +333,58 @@ func (s *HTTPServer) jobLatestDeployment(resp http.ResponseWriter, req *http.Req
 	return out.Deployment, nil
 }
 
-func (s *HTTPServer) jobCRUD(resp http.ResponseWriter, req *http.Request,
-	jobName string) (interface{}, error) {
+func (s *HTTPServer) jobSubmissionCRUD(resp http.ResponseWriter, req *http.Request, jobID string) (*structs.JobSubmission, error) {
+	version, err := strconv.ParseUint(req.URL.Query().Get("version"), 10, 64)
+	if err != nil {
+		return nil, CodedError(400, "Unable to parse job submission version parameter")
+	}
 	switch req.Method {
 	case "GET":
-		return s.jobQuery(resp, req, jobName)
-	case "PUT", "POST":
-		return s.jobUpdate(resp, req, jobName)
-	case "DELETE":
-		return s.jobDelete(resp, req, jobName)
+		return s.jobSubmissionQuery(resp, req, jobID, version)
 	default:
 		return nil, CodedError(405, ErrInvalidMethod)
 	}
 }
 
-func (s *HTTPServer) jobQuery(resp http.ResponseWriter, req *http.Request,
-	jobName string) (interface{}, error) {
+func (s *HTTPServer) jobSubmissionQuery(resp http.ResponseWriter, req *http.Request, jobID string, version uint64) (*structs.JobSubmission, error) {
+	args := structs.JobSubmissionRequest{
+		JobID:   jobID,
+		Version: version,
+	}
+
+	if s.parse(resp, req, &args.Region, &args.QueryOptions) {
+		return nil, nil
+	}
+
+	var out structs.JobSubmissionResponse
+	if err := s.agent.RPC("Job.GetJobSubmission", &args, &out); err != nil {
+		return nil, err
+	}
+
+	setMeta(resp, &out.QueryMeta)
+	if out.Submission == nil {
+		return nil, CodedError(404, "job source not found")
+	}
+
+	return out.Submission, nil
+}
+
+func (s *HTTPServer) jobCRUD(resp http.ResponseWriter, req *http.Request, jobID string) (interface{}, error) {
+	switch req.Method {
+	case "GET":
+		return s.jobQuery(resp, req, jobID)
+	case "PUT", "POST":
+		return s.jobUpdate(resp, req, jobID)
+	case "DELETE":
+		return s.jobDelete(resp, req, jobID)
+	default:
+		return nil, CodedError(405, ErrInvalidMethod)
+	}
+}
+
+func (s *HTTPServer) jobQuery(resp http.ResponseWriter, req *http.Request, jobID string) (interface{}, error) {
 	args := structs.JobSpecificRequest{
-		JobID: jobName,
+		JobID: jobID,
 	}
 	if s.parse(resp, req, &args.Region, &args.QueryOptions) {
 		return nil, nil
@@ -379,8 +414,7 @@ func (s *HTTPServer) jobQuery(resp http.ResponseWriter, req *http.Request,
 	return job, nil
 }
 
-func (s *HTTPServer) jobUpdate(resp http.ResponseWriter, req *http.Request,
-	jobName string) (interface{}, error) {
+func (s *HTTPServer) jobUpdate(resp http.ResponseWriter, req *http.Request, jobID string) (interface{}, error) {
 	var args api.JobRegisterRequest
 	if err := decodeBody(req, &args); err != nil {
 		return nil, CodedError(400, err.Error())
@@ -392,7 +426,7 @@ func (s *HTTPServer) jobUpdate(resp http.ResponseWriter, req *http.Request,
 	if args.Job.ID == nil {
 		return nil, CodedError(400, "Job ID hasn't been provided")
 	}
-	if jobName != "" && *args.Job.ID != jobName {
+	if jobID != "" && *args.Job.ID != jobID {
 		return nil, CodedError(400, "Job ID does not match name")
 	}
 
@@ -418,8 +452,12 @@ func (s *HTTPServer) jobUpdate(resp http.ResponseWriter, req *http.Request,
 	}
 
 	sJob, writeReq := s.apiJobAndRequestToStructs(args.Job, req, args.WriteRequest)
+	submission := apiJobSubmissionToStructs(args.Submission)
+
 	regReq := structs.JobRegisterRequest{
-		Job:            sJob,
+		Job:        sJob,
+		Submission: submission,
+
 		EnforceIndex:   args.EnforceIndex,
 		JobModifyIndex: args.JobModifyIndex,
 		PolicyOverride: args.PolicyOverride,
@@ -436,10 +474,11 @@ func (s *HTTPServer) jobUpdate(resp http.ResponseWriter, req *http.Request,
 	return out, nil
 }
 
-func (s *HTTPServer) jobDelete(resp http.ResponseWriter, req *http.Request,
-	jobName string) (interface{}, error) {
+func (s *HTTPServer) jobDelete(resp http.ResponseWriter, req *http.Request, jobID string) (interface{}, error) {
 
-	args := structs.JobDeregisterRequest{JobID: jobName}
+	args := structs.JobDeregisterRequest{
+		JobID: jobID,
+	}
 
 	// Identify the purge query param and parse.
 	purgeStr := req.URL.Query().Get("purge")
@@ -503,24 +542,22 @@ func (s *HTTPServer) jobDelete(resp http.ResponseWriter, req *http.Request,
 	return out, nil
 }
 
-func (s *HTTPServer) jobScale(resp http.ResponseWriter, req *http.Request,
-	jobName string) (interface{}, error) {
+func (s *HTTPServer) jobScale(resp http.ResponseWriter, req *http.Request, jobID string) (interface{}, error) {
 
 	switch req.Method {
 	case "GET":
-		return s.jobScaleStatus(resp, req, jobName)
+		return s.jobScaleStatus(resp, req, jobID)
 	case "PUT", "POST":
-		return s.jobScaleAction(resp, req, jobName)
+		return s.jobScaleAction(resp, req, jobID)
 	default:
 		return nil, CodedError(405, ErrInvalidMethod)
 	}
 }
 
-func (s *HTTPServer) jobScaleStatus(resp http.ResponseWriter, req *http.Request,
-	jobName string) (interface{}, error) {
+func (s *HTTPServer) jobScaleStatus(resp http.ResponseWriter, req *http.Request, jobID string) (interface{}, error) {
 
 	args := structs.JobScaleStatusRequest{
-		JobID: jobName,
+		JobID: jobID,
 	}
 	if s.parse(resp, req, &args.Region, &args.QueryOptions) {
 		return nil, nil
@@ -539,8 +576,7 @@ func (s *HTTPServer) jobScaleStatus(resp http.ResponseWriter, req *http.Request,
 	return out.JobScaleStatus, nil
 }
 
-func (s *HTTPServer) jobScaleAction(resp http.ResponseWriter, req *http.Request,
-	jobName string) (interface{}, error) {
+func (s *HTTPServer) jobScaleAction(resp http.ResponseWriter, req *http.Request, jobID string) (interface{}, error) {
 
 	if req.Method != "PUT" && req.Method != "POST" {
 		return nil, CodedError(405, ErrInvalidMethod)
@@ -552,12 +588,12 @@ func (s *HTTPServer) jobScaleAction(resp http.ResponseWriter, req *http.Request,
 	}
 
 	targetJob := args.Target[structs.ScalingTargetJob]
-	if targetJob != "" && targetJob != jobName {
+	if targetJob != "" && targetJob != jobID {
 		return nil, CodedError(400, "job ID in payload did not match URL")
 	}
 
 	scaleReq := structs.JobScaleRequest{
-		JobID:          jobName,
+		JobID:          jobID,
 		Target:         args.Target,
 		Count:          args.Count,
 		PolicyOverride: args.PolicyOverride,
@@ -577,8 +613,7 @@ func (s *HTTPServer) jobScaleAction(resp http.ResponseWriter, req *http.Request,
 	return out, nil
 }
 
-func (s *HTTPServer) jobVersions(resp http.ResponseWriter, req *http.Request,
-	jobName string) (interface{}, error) {
+func (s *HTTPServer) jobVersions(resp http.ResponseWriter, req *http.Request, jobID string) (interface{}, error) {
 
 	diffsStr := req.URL.Query().Get("diffs")
 	var diffsBool bool
@@ -591,7 +626,7 @@ func (s *HTTPServer) jobVersions(resp http.ResponseWriter, req *http.Request,
 	}
 
 	args := structs.JobVersionsRequest{
-		JobID: jobName,
+		JobID: jobID,
 		Diffs: diffsBool,
 	}
 	if s.parse(resp, req, &args.Region, &args.QueryOptions) {
@@ -611,8 +646,7 @@ func (s *HTTPServer) jobVersions(resp http.ResponseWriter, req *http.Request,
 	return out, nil
 }
 
-func (s *HTTPServer) jobRevert(resp http.ResponseWriter, req *http.Request,
-	jobName string) (interface{}, error) {
+func (s *HTTPServer) jobRevert(resp http.ResponseWriter, req *http.Request, jobID string) (interface{}, error) {
 
 	if req.Method != "PUT" && req.Method != "POST" {
 		return nil, CodedError(405, ErrInvalidMethod)
@@ -625,7 +659,7 @@ func (s *HTTPServer) jobRevert(resp http.ResponseWriter, req *http.Request,
 	if revertRequest.JobID == "" {
 		return nil, CodedError(400, "JobID must be specified")
 	}
-	if revertRequest.JobID != jobName {
+	if revertRequest.JobID != jobID {
 		return nil, CodedError(400, "Job ID does not match")
 	}
 
@@ -640,8 +674,7 @@ func (s *HTTPServer) jobRevert(resp http.ResponseWriter, req *http.Request,
 	return out, nil
 }
 
-func (s *HTTPServer) jobStable(resp http.ResponseWriter, req *http.Request,
-	jobName string) (interface{}, error) {
+func (s *HTTPServer) jobStable(resp http.ResponseWriter, req *http.Request, jobID string) (interface{}, error) {
 
 	if req.Method != "PUT" && req.Method != "POST" {
 		return nil, CodedError(405, ErrInvalidMethod)
@@ -654,7 +687,7 @@ func (s *HTTPServer) jobStable(resp http.ResponseWriter, req *http.Request,
 	if stableRequest.JobID == "" {
 		return nil, CodedError(400, "JobID must be specified")
 	}
-	if stableRequest.JobID != jobName {
+	if stableRequest.JobID != jobID {
 		return nil, CodedError(400, "Job ID does not match")
 	}
 
@@ -669,9 +702,9 @@ func (s *HTTPServer) jobStable(resp http.ResponseWriter, req *http.Request,
 	return out, nil
 }
 
-func (s *HTTPServer) jobSummaryRequest(resp http.ResponseWriter, req *http.Request, name string) (interface{}, error) {
+func (s *HTTPServer) jobSummaryRequest(resp http.ResponseWriter, req *http.Request, jobID string) (interface{}, error) {
 	args := structs.JobSummaryRequest{
-		JobID: name,
+		JobID: jobID,
 	}
 	if s.parse(resp, req, &args.Region, &args.QueryOptions) {
 		return nil, nil
@@ -690,7 +723,7 @@ func (s *HTTPServer) jobSummaryRequest(resp http.ResponseWriter, req *http.Reque
 	return out.JobSummary, nil
 }
 
-func (s *HTTPServer) jobDispatchRequest(resp http.ResponseWriter, req *http.Request, name string) (interface{}, error) {
+func (s *HTTPServer) jobDispatchRequest(resp http.ResponseWriter, req *http.Request, jobID string) (interface{}, error) {
 	if req.Method != "PUT" && req.Method != "POST" {
 		return nil, CodedError(405, ErrInvalidMethod)
 	}
@@ -698,11 +731,11 @@ func (s *HTTPServer) jobDispatchRequest(resp http.ResponseWriter, req *http.Requ
 	if err := decodeBody(req, &args); err != nil {
 		return nil, CodedError(400, err.Error())
 	}
-	if args.JobID != "" && args.JobID != name {
+	if args.JobID != "" && args.JobID != jobID {
 		return nil, CodedError(400, "Job ID does not match")
 	}
 	if args.JobID == "" {
-		args.JobID = name
+		args.JobID = jobID
 	}
 
 	s.parseWriteRequest(req, &args.WriteRequest)
@@ -753,10 +786,14 @@ func (s *HTTPServer) JobsParseRequest(resp http.ResponseWriter, req *http.Reques
 		jobStruct, err = jobspec.Parse(strings.NewReader(args.JobHCL))
 	} else {
 		jobStruct, err = jobspec2.ParseWithConfig(&jobspec2.ParseConfig{
-			Path:    "input.hcl",
-			Body:    []byte(args.JobHCL),
-			AllowFS: false,
+			Path:       "input.hcl",
+			Body:       []byte(args.JobHCL),
+			AllowFS:    false,
+			VarContent: args.Variables,
 		})
+		if err != nil {
+			return nil, CodedError(400, fmt.Sprintf("Failed to parse job: %v", err))
+		}
 	}
 	if err != nil {
 		return nil, CodedError(400, err.Error())
@@ -772,8 +809,7 @@ func (s *HTTPServer) JobsParseRequest(resp http.ResponseWriter, req *http.Reques
 // to the job identifier. It is callable via the
 // /v1/job/:jobID/services HTTP API and uses the
 // structs.JobServiceRegistrationsRPCMethod RPC method.
-func (s *HTTPServer) jobServiceRegistrations(
-	resp http.ResponseWriter, req *http.Request, jobID string) (interface{}, error) {
+func (s *HTTPServer) jobServiceRegistrations(resp http.ResponseWriter, req *http.Request, jobID string) (interface{}, error) {
 
 	// The endpoint only supports GET requests.
 	if req.Method != http.MethodGet {
@@ -801,6 +837,18 @@ func (s *HTTPServer) jobServiceRegistrations(
 	return reply.Services, nil
 }
 
+func apiJobSubmissionToStructs(submission *api.JobSubmission) *structs.JobSubmission {
+	if submission == nil {
+		return nil
+	}
+	return &structs.JobSubmission{
+		Source:        submission.Source,
+		Format:        submission.Format,
+		VariableFlags: submission.VariableFlags,
+		Variables:     submission.Variables,
+	}
+}
+
 // apiJobAndRequestToStructs parses the query params from the incoming
 // request and converts to a structs.Job and WriteRequest with the
 func (s *HTTPServer) apiJobAndRequestToStructs(job *api.Job, req *http.Request, apiReq api.WriteRequest) (*structs.Job, *structs.WriteRequest) {
@@ -917,6 +965,7 @@ func ApiJobToStructJob(job *api.Job) *structs.Job {
 		Priority:       *job.Priority,
 		AllAtOnce:      *job.AllAtOnce,
 		Datacenters:    job.Datacenters,
+		NodePool:       *job.NodePool,
 		Payload:        job.Payload,
 		Meta:           job.Meta,
 		ConsulToken:    *job.ConsulToken,
@@ -980,6 +1029,7 @@ func ApiJobToStructJob(job *api.Job) *structs.Job {
 			r.Name = region.Name
 			r.Count = *region.Count
 			r.Datacenters = region.Datacenters
+			r.NodePool = region.NodePool
 			r.Meta = region.Meta
 			j.Multiregion.Regions = append(j.Multiregion.Regions, r)
 		}
@@ -1194,10 +1244,7 @@ func ApiTaskToStructsTask(job *structs.Job, group *structs.TaskGroup,
 
 	structsTask.Resources = ApiResourcesToStructs(apiTask.Resources)
 
-	structsTask.LogConfig = &structs.LogConfig{
-		MaxFiles:      *apiTask.LogConfig.MaxFiles,
-		MaxFileSizeMB: *apiTask.LogConfig.MaxFileSizeMB,
-	}
+	structsTask.LogConfig = apiLogConfigToStructs(apiTask.LogConfig)
 
 	if len(apiTask.Artifacts) > 0 {
 		structsTask.Artifacts = []*structs.TaskArtifact{}
@@ -1218,6 +1265,7 @@ func ApiTaskToStructsTask(job *structs.Job, group *structs.TaskGroup,
 			Policies:     apiTask.Vault.Policies,
 			Namespace:    *apiTask.Vault.Namespace,
 			Env:          *apiTask.Vault.Env,
+			DisableFile:  *apiTask.Vault.DisableFile,
 			ChangeMode:   *apiTask.Vault.ChangeMode,
 			ChangeSignal: *apiTask.Vault.ChangeSignal,
 		}
@@ -1437,6 +1485,7 @@ func ApiServicesToStructs(in []*api.Service, group bool) []*structs.Service {
 					Interval:               check.Interval,
 					Timeout:                check.Timeout,
 					InitialStatus:          check.InitialStatus,
+					TLSServerName:          check.TLSServerName,
 					TLSSkipVerify:          check.TLSSkipVerify,
 					Header:                 check.Header,
 					Method:                 check.Method,
@@ -1645,6 +1694,7 @@ func apiConnectSidecarServiceToStructs(in *api.ConsulSidecarService) *structs.Co
 		Tags:                   slices.Clone(in.Tags),
 		Proxy:                  apiConnectSidecarServiceProxyToStructs(in.Proxy),
 		DisableDefaultTCPCheck: in.DisableDefaultTCPCheck,
+		Meta:                   maps.Clone(in.Meta),
 	}
 }
 
@@ -1759,12 +1809,21 @@ func apiLogConfigToStructs(in *api.LogConfig) *structs.LogConfig {
 	if in == nil {
 		return nil
 	}
+
 	return &structs.LogConfig{
+		Disabled:      dereferenceBool(in.Disabled),
 		MaxFiles:      dereferenceInt(in.MaxFiles),
 		MaxFileSizeMB: dereferenceInt(in.MaxFileSizeMB),
 	}
 }
 
+func dereferenceBool(in *bool) bool {
+	if in == nil {
+		return false
+	}
+	return *in
+}
+
 func dereferenceInt(in *int) int {
 	if in == nil {
 		return 0
