@@ -1,3 +1,6 @@
+// Copyright (c) HashiCorp, Inc.
+// SPDX-License-Identifier: MPL-2.0
+
 package command
 
 import (
@@ -415,6 +418,14 @@ func Commands(metaPtr *Meta, agentUi cli.Ui) map[string]cli.CommandFactory {
 				Meta: meta,
 			}, nil
 		},
+		"job restart": func() (cli.Command, error) {
+			// Use a *cli.ConcurrentUi because this command spawns several
+			// goroutines that write to the terminal concurrently.
+			meta.Ui = &cli.ConcurrentUi{Ui: meta.Ui}
+			return &JobRestartCommand{
+				Meta: meta,
+			}, nil
+		},
 		"job deployments": func() (cli.Command, error) {
 			return &JobDeploymentsCommand{
 				Meta: meta,
@@ -605,6 +616,46 @@ func Commands(metaPtr *Meta, agentUi cli.Ui) map[string]cli.CommandFactory {
 				Meta: meta,
 			}, nil
 		},
+		"node pool": func() (cli.Command, error) {
+			return &NodePoolCommand{
+				Meta: meta,
+			}, nil
+		},
+		"node pool apply": func() (cli.Command, error) {
+			return &NodePoolApplyCommand{
+				Meta: meta,
+			}, nil
+		},
+		"node pool delete": func() (cli.Command, error) {
+			return &NodePoolDeleteCommand{
+				Meta: meta,
+			}, nil
+		},
+		"node pool info": func() (cli.Command, error) {
+			return &NodePoolInfoCommand{
+				Meta: meta,
+			}, nil
+		},
+		"node pool init": func() (cli.Command, error) {
+			return &NodePoolInitCommand{
+				Meta: meta,
+			}, nil
+		},
+		"node pool jobs": func() (cli.Command, error) {
+			return &NodePoolJobsCommand{
+				Meta: meta,
+			}, nil
+		},
+		"node pool list": func() (cli.Command, error) {
+			return &NodePoolListCommand{
+				Meta: meta,
+			}, nil
+		},
+		"node pool nodes": func() (cli.Command, error) {
+			return &NodePoolNodesCommand{
+				Meta: meta,
+			}, nil
+		},
 		"operator": func() (cli.Command, error) {
 			return &OperatorCommand{
 				Meta: meta,
