@@ -1,3 +1,6 @@
+// Copyright (c) HashiCorp, Inc.
+// SPDX-License-Identifier: MPL-2.0
+
 package command
 
 import (
@@ -247,7 +250,7 @@ func (c *NodeStatusCommand) Run(args []string) int {
 			return 0
 		}
 
-		out[0] = "ID|DC|Name|Class|"
+		out[0] = "ID|Node Pool|DC|Name|Class|"
 
 		if c.os {
 			out[0] += "OS|"
@@ -264,8 +267,9 @@ func (c *NodeStatusCommand) Run(args []string) int {
 		}
 
 		for i, node := range nodes {
-			out[i+1] = fmt.Sprintf("%s|%s|%s|%s",
+			out[i+1] = fmt.Sprintf("%s|%s|%s|%s|%s",
 				limit(node.ID, c.length),
+				node.NodePool,
 				node.Datacenter,
 				node.Name,
 				node.NodeClass)
@@ -477,6 +481,7 @@ func (c *NodeStatusCommand) formatNode(client *api.Client, node *api.Node) int {
 	basic := []string{
 		fmt.Sprintf("ID|%s", node.ID),
 		fmt.Sprintf("Name|%s", node.Name),
+		fmt.Sprintf("Node Pool|%s", node.NodePool),
 		fmt.Sprintf("Class|%s", node.NodeClass),
 		fmt.Sprintf("DC|%s", node.Datacenter),
 		fmt.Sprintf("Drain|%v", formatDrain(node)),
@@ -695,14 +700,15 @@ func (c *NodeStatusCommand) outputNodeCSIVolumeInfo(client *api.Client, node *ap
 
 		// Output the volumes in name order
 		output := make([]string, 0, len(names)+1)
-		output = append(output, "ID|Name|Plugin ID|Schedulable|Provider|Access Mode")
+		output = append(output, "ID|Name|Namespace|Plugin ID|Schedulable|Provider|Access Mode")
 		for _, name := range names {
 			v, ok := volumes[name]
 			if ok {
 				output = append(output, fmt.Sprintf(
-					"%s|%s|%s|%t|%s|%s",
+					"%s|%s|%s|%s|%t|%s|%s",
 					v.ID,
 					name,
+					v.Namespace,
 					v.PluginID,
 					v.Schedulable,
 					v.Provider,
