@@ -1,3 +1,6 @@
+// Copyright (c) HashiCorp, Inc.
+// SPDX-License-Identifier: MPL-2.0
+
 package docker
 
 import (
@@ -204,7 +207,7 @@ var (
 			"type":   hclspec.NewAttr("type", "string", false),
 			"config": hclspec.NewBlockAttrs("config", "string", false),
 		})), hclspec.NewLiteral(`{
-			type = "json-file" 
+			type = "json-file"
 			config = {
 				max-file = "2"
 				max-size = "2m"
@@ -357,6 +360,7 @@ var (
 		"entrypoint":         hclspec.NewAttr("entrypoint", "list(string)", false),
 		"extra_hosts":        hclspec.NewAttr("extra_hosts", "list(string)", false),
 		"force_pull":         hclspec.NewAttr("force_pull", "bool", false),
+		"group_add":          hclspec.NewAttr("group_add", "list(string)", false),
 		"healthchecks":       hclspec.NewBlock("healthchecks", false, healthchecksBodySpec),
 		"hostname":           hclspec.NewAttr("hostname", "string", false),
 		"init":               hclspec.NewAttr("init", "bool", false),
@@ -440,6 +444,7 @@ type TaskConfig struct {
 	Entrypoint        []string           `codec:"entrypoint"`
 	ExtraHosts        []string           `codec:"extra_hosts"`
 	ForcePull         bool               `codec:"force_pull"`
+	GroupAdd          []string           `codec:"group_add"`
 	Healthchecks      DockerHealthchecks `codec:"healthchecks"`
 	Hostname          string             `codec:"hostname"`
 	Init              bool               `codec:"init"`
@@ -504,6 +509,11 @@ func (d DockerDevice) toDockerDevice() (docker.Device, error) {
 		return dd, fmt.Errorf("host path must be set in configuration for devices")
 	}
 
+	// Docker's CLI defaults to HostPath in this case. See #16754
+	if dd.PathInContainer == "" {
+		dd.PathInContainer = d.HostPath
+	}
+
 	if dd.CgroupPermissions == "" {
 		dd.CgroupPermissions = "rwm"
 	}
@@ -755,7 +765,7 @@ func (d *Driver) SetConfig(c *base.Config) error {
 		d.clientConfig = c.AgentConfig.Driver
 	}
 
-	dockerClient, _, err := d.dockerClients()
+	dockerClient, err := d.getDockerClient()
 	if err != nil {
 		return fmt.Errorf("failed to get docker client: %v", err)
 	}
@@ -773,6 +783,8 @@ func (d *Driver) SetConfig(c *base.Config) error {
 
 	d.cpusetFixer = newCpusetFixer(d)
 
+	go d.recoverPauseContainers(d.ctx)
+
 	return nil
 }
 
@@ -783,13 +795,6 @@ func (d *Driver) TaskConfigSchema() (*hclspec.Spec, error) {
 // Capabilities is returned by the Capabilities RPC and indicates what optional
 // features this driver supports.
 func (d *Driver) Capabilities() (*drivers.Capabilities, error) {
+	driverCapabilities.DisableLogCollection = d.config != nil && d.config.DisableLogCollection
 	return driverCapabilities, nil
 }
-
-var _ drivers.InternalCapabilitiesDriver = (*Driver)(nil)
-
-func (d *Driver) InternalCapabilities() drivers.InternalCapabilities {
-	return drivers.InternalCapabilities{
-		DisableLogCollection: d.config != nil && d.config.DisableLogCollection,
-	}
-}
