@@ -1,3 +1,6 @@
+// Copyright (c) HashiCorp, Inc.
+// SPDX-License-Identifier: MPL-2.0
+
 package docker
 
 import (
@@ -5,7 +8,6 @@ import (
 	"context"
 	"encoding/json"
 	"fmt"
-	"io/ioutil"
 	"net"
 	"os"
 	"path/filepath"
@@ -28,27 +30,16 @@ import (
 	"github.com/hashicorp/nomad/drivers/shared/eventer"
 	"github.com/hashicorp/nomad/drivers/shared/hostnames"
 	"github.com/hashicorp/nomad/drivers/shared/resolvconf"
+	"github.com/hashicorp/nomad/helper"
 	nstructs "github.com/hashicorp/nomad/nomad/structs"
 	"github.com/hashicorp/nomad/plugins/base"
 	"github.com/hashicorp/nomad/plugins/drivers"
 	pstructs "github.com/hashicorp/nomad/plugins/shared/structs"
 	"github.com/ryanuber/go-glob"
+	"golang.org/x/exp/slices"
 )
 
 var (
-	// createClientsLock is a lock that protects reading/writing global client
-	// variables
-	createClientsLock sync.Mutex
-
-	// client is a docker client with a timeout of 5 minutes. This is for doing
-	// all operations with the docker daemon besides which are not long running
-	// such as creating, killing containers, etc.
-	client *docker.Client
-
-	// waitClient is a docker client with no timeouts. This is used for long
-	// running operations such as waiting on containers and collect stats
-	waitClient *docker.Client
-
 	dockerTransientErrs = []string{
 		"Client.Timeout exceeded while awaiting headers",
 		"EOF",
@@ -83,6 +74,7 @@ const (
 	dockerLabelNamespace     = "com.hashicorp.nomad.namespace"
 	dockerLabelNodeName      = "com.hashicorp.nomad.node_name"
 	dockerLabelNodeID        = "com.hashicorp.nomad.node_id"
+	dockerLabelParentJobID   = "com.hashicorp.nomad.parent_job_id"
 )
 
 type pauseContainerStore struct {
@@ -156,6 +148,10 @@ type Driver struct {
 	detected     bool
 	detectedLock sync.RWMutex
 
+	dockerClientLock sync.Mutex
+	dockerClient     *docker.Client // for most docker api calls (use getDockerClient())
+	infinityClient   *docker.Client // for wait and stop calls (use getInfinityClient())
+
 	danglingReconciler *containerReconciler
 	cpusetFixer        CpusetFixer
 }
@@ -171,7 +167,7 @@ func NewDockerDriver(ctx context.Context, logger hclog.Logger) drivers.DriverPlu
 		ctx:             ctx,
 		logger:          logger,
 	}
-	go driver.recoverPauseContainers(ctx)
+
 	return driver
 }
 
@@ -226,12 +222,17 @@ func (d *Driver) RecoverTask(handle *drivers.TaskHandle) error {
 		return fmt.Errorf("failed to decode driver task state: %v", err)
 	}
 
-	client, _, err := d.dockerClients()
+	dockerClient, err := d.getDockerClient()
 	if err != nil {
-		return fmt.Errorf("failed to get docker client: %v", err)
+		return fmt.Errorf("failed to get docker client: %w", err)
 	}
 
-	container, err := client.InspectContainerWithOptions(docker.InspectContainerOptions{
+	infinityClient, err := d.getInfinityClient()
+	if err != nil {
+		return fmt.Errorf("failed to get docker long operations client: %w", err)
+	}
+
+	container, err := dockerClient.InspectContainerWithOptions(docker.InspectContainerOptions{
 		ID: handleState.ContainerID,
 	})
 	if err != nil {
@@ -239,8 +240,8 @@ func (d *Driver) RecoverTask(handle *drivers.TaskHandle) error {
 	}
 
 	h := &taskHandle{
-		client:                client,
-		waitClient:            waitClient,
+		dockerClient:          dockerClient,
+		infinityClient:        infinityClient,
 		logger:                d.logger.With("container_id", container.ID),
 		task:                  handle.Config,
 		containerID:           container.ID,
@@ -251,21 +252,21 @@ func (d *Driver) RecoverTask(handle *drivers.TaskHandle) error {
 		net:                   handleState.DriverNetwork,
 	}
 
-	if !d.config.DisableLogCollection {
+	if loggingIsEnabled(d.config, handle.Config) {
 		h.dlogger, h.dloggerPluginClient, err = d.reattachToDockerLogger(handleState.ReattachConfig)
 		if err != nil {
 			d.logger.Warn("failed to reattach to docker logger process", "error", err)
 
 			h.dlogger, h.dloggerPluginClient, err = d.setupNewDockerLogger(container, handle.Config, time.Now())
 			if err != nil {
-				if err := client.StopContainer(handleState.ContainerID, 0); err != nil {
+				if err := dockerClient.StopContainer(handleState.ContainerID, 0); err != nil {
 					d.logger.Warn("failed to stop container during cleanup", "container_id", handleState.ContainerID, "error", err)
 				}
 				return fmt.Errorf("failed to setup replacement docker logger: %v", err)
 			}
 
 			if err := handle.SetDriverState(h.buildState()); err != nil {
-				if err := client.StopContainer(handleState.ContainerID, 0); err != nil {
+				if err := dockerClient.StopContainer(handleState.ContainerID, 0); err != nil {
 					d.logger.Warn("failed to stop container during cleanup", "container_id", handleState.ContainerID, "error", err)
 				}
 				return fmt.Errorf("failed to store driver state: %v", err)
@@ -282,6 +283,16 @@ func (d *Driver) RecoverTask(handle *drivers.TaskHandle) error {
 	return nil
 }
 
+func loggingIsEnabled(driverCfg *DriverConfig, taskCfg *drivers.TaskConfig) bool {
+	if driverCfg.DisableLogCollection {
+		return false
+	}
+	if taskCfg.StderrPath == os.DevNull && taskCfg.StdoutPath == os.DevNull {
+		return false
+	}
+	return true
+}
+
 func (d *Driver) StartTask(cfg *drivers.TaskConfig) (*drivers.TaskHandle, *drivers.DriverNetwork, error) {
 	if _, ok := d.tasks.Get(cfg.ID); ok {
 		return nil, nil, fmt.Errorf("task with ID %q already started", cfg.ID)
@@ -302,13 +313,19 @@ func (d *Driver) StartTask(cfg *drivers.TaskConfig) (*drivers.TaskHandle, *drive
 	handle := drivers.NewTaskHandle(taskHandleVersion)
 	handle.Config = cfg
 
-	// Initialize docker API clients
-	client, _, err := d.dockerClients()
+	// we'll need the normal docker client
+	dockerClient, err := d.getDockerClient()
 	if err != nil {
-		return nil, nil, fmt.Errorf("Failed to connect to docker daemon: %s", err)
+		return nil, nil, fmt.Errorf("Failed to create docker client: %v", err)
 	}
 
-	id, err := d.createImage(cfg, &driverConfig, client)
+	// and also the long operations client
+	infinityClient, err := d.getInfinityClient()
+	if err != nil {
+		return nil, nil, fmt.Errorf("Failed to create long operations docker client: %v", err)
+	}
+
+	id, err := d.createImage(cfg, &driverConfig, dockerClient)
 	if err != nil {
 		return nil, nil, err
 	}
@@ -329,10 +346,10 @@ func (d *Driver) StartTask(cfg *drivers.TaskConfig) (*drivers.TaskHandle, *drive
 
 	startAttempts := 0
 CREATE:
-	container, err := d.createContainer(client, containerCfg, driverConfig.Image)
+	container, err := d.createContainer(dockerClient, containerCfg, driverConfig.Image)
 	if err != nil {
 		d.logger.Error("failed to create container", "error", err)
-		client.RemoveContainer(docker.RemoveContainerOptions{
+		dockerClient.RemoveContainer(docker.RemoveContainerOptions{
 			ID:    containerCfg.Name,
 			Force: true,
 		})
@@ -348,7 +365,7 @@ CREATE:
 		// Start the container
 		if err := d.startContainer(container); err != nil {
 			d.logger.Error("failed to start container", "container_id", container.ID, "error", err)
-			client.RemoveContainer(docker.RemoveContainerOptions{
+			dockerClient.RemoveContainer(docker.RemoveContainerOptions{
 				ID:    container.ID,
 				Force: true,
 			})
@@ -363,17 +380,17 @@ CREATE:
 
 		// Inspect container to get all of the container metadata as much of the
 		// metadata (eg networking) isn't populated until the container is started
-		runningContainer, err := client.InspectContainerWithOptions(docker.InspectContainerOptions{
+		runningContainer, err := dockerClient.InspectContainerWithOptions(docker.InspectContainerOptions{
 			ID: container.ID,
 		})
 		if err != nil {
-			client.RemoveContainer(docker.RemoveContainerOptions{
+			dockerClient.RemoveContainer(docker.RemoveContainerOptions{
 				ID:    container.ID,
 				Force: true,
 			})
 			msg := "failed to inspect started container"
 			d.logger.Error(msg, "error", err)
-			client.RemoveContainer(docker.RemoveContainerOptions{
+			dockerClient.RemoveContainer(docker.RemoveContainerOptions{
 				ID:    container.ID,
 				Force: true,
 			})
@@ -397,7 +414,7 @@ CREATE:
 		}
 	}
 
-	collectingLogs := !d.config.DisableLogCollection
+	collectingLogs := loggingIsEnabled(d.config, cfg)
 
 	var dlogger docklog.DockerLogger
 	var pluginClient *plugin.Client
@@ -406,7 +423,7 @@ CREATE:
 		dlogger, pluginClient, err = d.setupNewDockerLogger(container, cfg, time.Unix(0, 0))
 		if err != nil {
 			d.logger.Error("an error occurred after container startup, terminating container", "container_id", container.ID)
-			client.RemoveContainer(docker.RemoveContainerOptions{ID: container.ID, Force: true})
+			dockerClient.RemoveContainer(docker.RemoveContainerOptions{ID: container.ID, Force: true})
 			return nil, nil, err
 		}
 	}
@@ -422,8 +439,8 @@ CREATE:
 
 	// Return a driver handle
 	h := &taskHandle{
-		client:                client,
-		waitClient:            waitClient,
+		dockerClient:          dockerClient,
+		infinityClient:        infinityClient,
 		dlogger:               dlogger,
 		dloggerPluginClient:   pluginClient,
 		logger:                d.logger.With("container_id", container.ID),
@@ -442,7 +459,7 @@ CREATE:
 			dlogger.Stop()
 			pluginClient.Kill()
 		}
-		client.RemoveContainer(docker.RemoveContainerOptions{ID: container.ID, Force: true})
+		dockerClient.RemoveContainer(docker.RemoveContainerOptions{ID: container.ID, Force: true})
 		return nil, nil, err
 	}
 
@@ -535,10 +552,15 @@ CREATE:
 // startContainer starts the passed container. It attempts to handle any
 // transient Docker errors.
 func (d *Driver) startContainer(c *docker.Container) error {
+	dockerClient, err := d.getDockerClient()
+	if err != nil {
+		return err
+	}
+
 	// Start a container
 	attempted := 0
 START:
-	startErr := client.StartContainer(c.ID, c.HostConfig)
+	startErr := dockerClient.StartContainer(c.ID, c.HostConfig)
 	if startErr == nil || strings.Contains(startErr.Error(), "Container already running") {
 		return nil
 	}
@@ -678,7 +700,12 @@ func (d *Driver) loadImage(task *drivers.TaskConfig, driverConfig *TaskConfig, c
 }
 
 func (d *Driver) convertAllocPathsForWindowsLCOW(task *drivers.TaskConfig, image string) error {
-	imageConfig, err := client.InspectImage(image)
+	dockerClient, err := d.getDockerClient()
+	if err != nil {
+		return err
+	}
+
+	imageConfig, err := dockerClient.InspectImage(image)
 	if err != nil {
 		return fmt.Errorf("the image does not exist: %v", err)
 	}
@@ -748,12 +775,45 @@ func (d *Driver) containerBinds(task *drivers.TaskConfig, driverConfig *TaskConf
 	return binds, nil
 }
 
-func (d *Driver) recoverPauseContainers(ctx context.Context) {
-	// On Client restart, we must rebuild the set of pause containers
-	// we are tracking. Basically just scan all containers and pull the ID from
-	// anything that has the Nomad Label and has Name with prefix "/nomad_init_".
+func (d *Driver) findPauseContainer(allocID string) (string, error) {
 
-	_, dockerClient, err := d.dockerClients()
+	dockerClient, err := d.getDockerClient()
+	if err != nil {
+		return "", err
+	}
+
+	containers, listErr := dockerClient.ListContainers(docker.ListContainersOptions{
+		Context: d.ctx,
+		All:     false, // running only
+		Filters: map[string][]string{
+			"label": {dockerLabelAllocID},
+		},
+	})
+	if listErr != nil {
+		d.logger.Error("failed to list pause containers for recovery", "error", listErr)
+		return "", listErr
+	}
+
+	for _, c := range containers {
+		if !slices.ContainsFunc(c.Names, func(s string) bool {
+			return strings.HasPrefix(s, "/nomad_init_")
+		}) {
+			continue
+		}
+		if c.Labels[dockerLabelAllocID] == allocID {
+			return c.ID, nil
+		}
+	}
+
+	return "", nil
+}
+
+// recoverPauseContainers gets called when we start up the plugin. On client
+// restarts we need to rebuild the set of pause containers we are
+// tracking. Basically just scan all containers and pull the ID from anything
+// that has the Nomad Label and has Name with prefix "/nomad_init_".
+func (d *Driver) recoverPauseContainers(ctx context.Context) {
+	dockerClient, err := d.getDockerClient()
 	if err != nil {
 		d.logger.Error("failed to recover pause containers", "error", err)
 		return
@@ -766,8 +826,8 @@ func (d *Driver) recoverPauseContainers(ctx context.Context) {
 			"label": {dockerLabelAllocID},
 		},
 	})
-	if listErr != nil {
-		d.logger.Error("failed to list pause containers", "error", err)
+	if listErr != nil && listErr != ctx.Err() {
+		d.logger.Error("failed to list pause containers for recovery", "error", listErr)
 		return
 	}
 
@@ -805,7 +865,7 @@ func parseSecurityOpts(securityOpts []string) ([]string, error) {
 			}
 		}
 		if con[0] == "seccomp" && con[1] != "unconfined" {
-			f, err := ioutil.ReadFile(con[1])
+			f, err := os.ReadFile(con[1])
 			if err != nil {
 				return securityOpts, fmt.Errorf("opening seccomp profile (%s) failed: %v", con[1], err)
 			}
@@ -950,7 +1010,8 @@ func (d *Driver) createContainerConfig(task *drivers.TaskConfig, driverConfig *T
 
 		PidsLimit: &pidsLimit,
 
-		Runtime: containerRuntime,
+		Runtime:  containerRuntime,
+		GroupAdd: driverConfig.GroupAdd,
 	}
 
 	// This translates to docker create/run --cpuset-cpus option.
@@ -1036,7 +1097,7 @@ func (d *Driver) createContainerConfig(task *drivers.TaskConfig, driverConfig *T
 		hostConfig.ShmSize = driverConfig.ShmSize
 	}
 
-	// Setup devices
+	// Setup devices from Docker-specific config
 	for _, device := range driverConfig.Devices {
 		dd, err := device.toDockerDevice()
 		if err != nil {
@@ -1044,6 +1105,8 @@ func (d *Driver) createContainerConfig(task *drivers.TaskConfig, driverConfig *T
 		}
 		hostConfig.Devices = append(hostConfig.Devices, dd)
 	}
+
+	// Setup devices from Nomad device plugins
 	for _, device := range task.Devices {
 		hostConfig.Devices = append(hostConfig.Devices, docker.Device{
 			PathOnHost:        device.HostPath,
@@ -1262,6 +1325,9 @@ func (d *Driver) createContainerConfig(task *drivers.TaskConfig, driverConfig *T
 		if glob.Glob(configurationExtraLabel, "job_id") {
 			labels[dockerLabelJobID] = task.JobID
 		}
+		if glob.Glob(configurationExtraLabel, "parent_job_id") && len(task.ParentJobID) > 0 {
+			labels[dockerLabelParentJobID] = task.ParentJobID
+		}
 		if glob.Glob(configurationExtraLabel, "task_group_name") {
 			labels[dockerLabelTaskGroupName] = task.TaskGroupName
 		}
@@ -1410,11 +1476,11 @@ func (d *Driver) detectIP(c *docker.Container, driverConfig *TaskConfig) (string
 // if the container is dead or can't be found.
 func (d *Driver) containerByName(name string) (*docker.Container, error) {
 
-	client, _, err := d.dockerClients()
+	dockerClient, err := d.getDockerClient()
 	if err != nil {
 		return nil, err
 	}
-	containers, err := client.ListContainers(docker.ListContainersOptions{
+	containers, err := dockerClient.ListContainers(docker.ListContainersOptions{
 		All: true,
 	})
 	if err != nil {
@@ -1446,7 +1512,7 @@ OUTER:
 		return nil, nil
 	}
 
-	container, err := client.InspectContainerWithOptions(docker.InspectContainerOptions{
+	container, err := dockerClient.InspectContainerWithOptions(docker.InspectContainerOptions{
 		ID: shimContainer.ID,
 	})
 	if err != nil {
@@ -1514,7 +1580,12 @@ func (d *Driver) DestroyTask(taskID string, force bool) error {
 		return drivers.ErrTaskNotFound
 	}
 
-	c, err := client.InspectContainerWithOptions(docker.InspectContainerOptions{
+	dockerClient, err := d.getDockerClient()
+	if err != nil {
+		return err
+	}
+
+	c, err := dockerClient.InspectContainerWithOptions(docker.InspectContainerOptions{
 		ID: h.containerID,
 	})
 	if err != nil {
@@ -1530,13 +1601,13 @@ func (d *Driver) DestroyTask(taskID string, force bool) error {
 			if !force {
 				return fmt.Errorf("must call StopTask for the given task before Destroy or set force to true")
 			}
-			if err := h.client.StopContainer(h.containerID, 0); err != nil {
+			if err := dockerClient.StopContainer(h.containerID, 0); err != nil {
 				h.logger.Warn("failed to stop container during destroy", "error", err)
 			}
 		}
 
 		if h.removeContainerOnExit {
-			if err := h.client.RemoveContainer(docker.RemoveContainerOptions{ID: h.containerID, RemoveVolumes: true, Force: true}); err != nil {
+			if err := dockerClient.RemoveContainer(docker.RemoveContainerOptions{ID: h.containerID, RemoveVolumes: true, Force: true}); err != nil {
 				h.logger.Error("error removing container", "error", err)
 			}
 		} else {
@@ -1572,7 +1643,12 @@ func (d *Driver) InspectTask(taskID string) (*drivers.TaskStatus, error) {
 		return nil, drivers.ErrTaskNotFound
 	}
 
-	container, err := client.InspectContainerWithOptions(docker.InspectContainerOptions{
+	dockerClient, err := d.getDockerClient()
+	if err != nil {
+		return nil, err
+	}
+
+	container, err := dockerClient.InspectContainerWithOptions(docker.InspectContainerOptions{
 		ID: h.containerID,
 	})
 	if err != nil {
@@ -1674,7 +1750,13 @@ func (d *Driver) ExecTaskStreaming(ctx context.Context, taskID string, opts *dri
 		Container:    h.containerID,
 		Context:      ctx,
 	}
-	exec, err := h.client.CreateExec(createExecOpts)
+
+	dockerClient, err := d.getDockerClient()
+	if err != nil {
+		return nil, err
+	}
+
+	exec, err := dockerClient.CreateExec(createExecOpts)
 	if err != nil {
 		return nil, fmt.Errorf("failed to create exec object: %v", err)
 	}
@@ -1690,7 +1772,7 @@ func (d *Driver) ExecTaskStreaming(ctx context.Context, taskID string, opts *dri
 				if !ok {
 					return
 				}
-				client.ResizeExecTTY(exec.ID, s.Height, s.Width)
+				dockerClient.ResizeExecTTY(exec.ID, s.Height, s.Width)
 			}
 		}
 	}()
@@ -1709,7 +1791,7 @@ func (d *Driver) ExecTaskStreaming(ctx context.Context, taskID string, opts *dri
 		ErrorStream:  opts.Stderr,
 		Context:      ctx,
 	}
-	if err := client.StartExec(exec.ID, startOpts); err != nil {
+	if err := dockerClient.StartExec(exec.ID, startOpts); err != nil {
 		return nil, fmt.Errorf("failed to start exec: %v", err)
 	}
 
@@ -1720,7 +1802,7 @@ func (d *Driver) ExecTaskStreaming(ctx context.Context, taskID string, opts *dri
 	start := time.Now()
 	var res *docker.ExecInspect
 	for (res == nil || res.Running) && time.Since(start) <= execTerminatingTimeout {
-		res, err = client.InspectExec(exec.ID)
+		res, err = dockerClient.InspectExec(exec.ID)
 		if err != nil {
 			return nil, fmt.Errorf("failed to inspect exec result: %v", err)
 		}
@@ -1736,37 +1818,37 @@ func (d *Driver) ExecTaskStreaming(ctx context.Context, taskID string, opts *dri
 	}, nil
 }
 
-// dockerClients creates two *docker.Client, one for long running operations and
-// the other for shorter operations. In test / dev mode we can use ENV vars to
-// connect to the docker daemon. In production mode we will read docker.endpoint
-// from the config file.
-func (d *Driver) dockerClients() (*docker.Client, *docker.Client, error) {
-	createClientsLock.Lock()
-	defer createClientsLock.Unlock()
+func (d *Driver) getOrCreateClient(timeout time.Duration) (*docker.Client, error) {
+	var (
+		client *docker.Client
+		err    error
+	)
 
-	if client != nil && waitClient != nil {
-		return client, waitClient, nil
-	}
-
-	var err error
-
-	// Only initialize the client if it hasn't yet been done
-	if client == nil {
-		client, err = d.newDockerClient(dockerTimeout)
-		if err != nil {
-			return nil, nil, err
+	helper.WithLock(&d.dockerClientLock, func() {
+		if timeout == 0 {
+			if d.infinityClient == nil {
+				d.infinityClient, err = d.newDockerClient(0)
+			}
+			client = d.infinityClient
+		} else {
+			if d.dockerClient == nil {
+				d.dockerClient, err = d.newDockerClient(timeout)
+			}
+			client = d.dockerClient
 		}
-	}
+	})
 
-	// Only initialize the waitClient if it hasn't yet been done
-	if waitClient == nil {
-		waitClient, err = d.newDockerClient(0 * time.Minute)
-		if err != nil {
-			return nil, nil, err
-		}
-	}
+	return client, err
+}
 
-	return client, waitClient, nil
+// getInfinityClient creates a docker API client with no timeout.
+func (d *Driver) getInfinityClient() (*docker.Client, error) {
+	return d.getOrCreateClient(0)
+}
+
+// getDockerClient creates a docker API client with a hard-coded timeout.
+func (d *Driver) getDockerClient() (*docker.Client, error) {
+	return d.getOrCreateClient(dockerTimeout)
 }
 
 // newDockerClient creates a new *docker.Client with a configurable timeout
