@@ -1,10 +1,13 @@
+// Copyright (c) HashiCorp, Inc.
+// SPDX-License-Identifier: MPL-2.0
+
 package docker
 
 import (
 	"context"
 	"fmt"
-	"io/ioutil"
 	"math/rand"
+	"os"
 	"path/filepath"
 	"reflect"
 	"runtime"
@@ -17,9 +20,14 @@ import (
 
 	docker "github.com/fsouza/go-dockerclient"
 	hclog "github.com/hashicorp/go-hclog"
+	"github.com/shoenig/test/must"
+	"github.com/stretchr/testify/assert"
+	"github.com/stretchr/testify/require"
+
 	"github.com/hashicorp/nomad/ci"
 	"github.com/hashicorp/nomad/client/taskenv"
 	"github.com/hashicorp/nomad/client/testutil"
+	"github.com/hashicorp/nomad/drivers/shared/capabilities"
 	"github.com/hashicorp/nomad/helper/pluginutils/hclspecutils"
 	"github.com/hashicorp/nomad/helper/pluginutils/hclutils"
 	"github.com/hashicorp/nomad/helper/pluginutils/loader"
@@ -30,9 +38,6 @@ import (
 	"github.com/hashicorp/nomad/plugins/drivers"
 	dtestutil "github.com/hashicorp/nomad/plugins/drivers/testutils"
 	tu "github.com/hashicorp/nomad/testutil"
-	"github.com/shoenig/test/must"
-	"github.com/stretchr/testify/assert"
-	"github.com/stretchr/testify/require"
 )
 
 var (
@@ -130,7 +135,7 @@ func dockerTask(t *testing.T) (*drivers.TaskConfig, *TaskConfig, []int) {
 func dockerSetup(t *testing.T, task *drivers.TaskConfig, driverCfg map[string]interface{}) (*docker.Client, *dtestutil.DriverHarness, *taskHandle, func()) {
 	client := newTestDockerClient(t)
 	driver := dockerDriverHarness(t, driverCfg)
-	cleanup := driver.MkAllocDir(task, true)
+	cleanup := driver.MkAllocDir(task, loggingIsEnabled(&DriverConfig{}, task))
 
 	copyImage(t, task.TaskDir(), "busybox.tar")
 	_, _, err := driver.StartTask(task)
@@ -187,6 +192,14 @@ func dockerDriverHarness(t *testing.T, cfg map[string]interface{}) *dtestutil.Dr
 			},
 		}
 	}
+
+	// If on windows, "allow" (don't attempt to drop) linux capabilities.
+	// https://github.com/hashicorp/nomad/issues/15181
+	// TODO: this should instead get fixed properly in capabilities package.
+	if _, ok := cfg["allow_caps"]; !ok && runtime.GOOS == "windows" {
+		cfg["allow_caps"] = capabilities.DockerDefaults().Slice(false)
+	}
+
 	plugLoader, err := loader.NewPluginLoader(&loader.PluginLoaderConfig{
 		Logger:            logger,
 		PluginDir:         "./plugins",
@@ -397,7 +410,7 @@ func TestDockerDriver_Start_LoadImage(t *testing.T) {
 
 	// Check that data was written to the shared alloc directory.
 	outputFile := filepath.Join(task.TaskDir().LocalDir, "output")
-	act, err := ioutil.ReadFile(outputFile)
+	act, err := os.ReadFile(outputFile)
 	if err != nil {
 		t.Fatalf("Couldn't read expected output: %v", err)
 	}
@@ -524,7 +537,7 @@ func TestDockerDriver_Start_Wait_AllocDir(t *testing.T) {
 
 	// Check that data was written to the shared alloc directory.
 	outputFile := filepath.Join(task.TaskDir().SharedAllocDir, file)
-	act, err := ioutil.ReadFile(outputFile)
+	act, err := os.ReadFile(outputFile)
 	if err != nil {
 		t.Fatalf("Couldn't read expected output: %v", err)
 	}
@@ -838,6 +851,37 @@ func TestDockerDriver_LoggingConfiguration(t *testing.T) {
 	require.Equal(t, loggerConfig, container.HostConfig.LogConfig.Config)
 }
 
+// TestDockerDriver_LogCollectionDisabled ensures that logmon isn't configured
+// when log collection is disable, but out-of-band Docker log shipping still
+// works as expected
+func TestDockerDriver_LogCollectionDisabled(t *testing.T) {
+	ci.Parallel(t)
+	testutil.DockerCompatible(t)
+
+	task, cfg, _ := dockerTask(t)
+	task.StdoutPath = os.DevNull
+	task.StderrPath = os.DevNull
+
+	must.NoError(t, task.EncodeConcreteDriverConfig(cfg))
+
+	dockerClientConfig := make(map[string]interface{})
+	loggerConfig := map[string]string{"gelf-address": "udp://1.2.3.4:12201", "tag": "gelf"}
+
+	dockerClientConfig["logging"] = LoggingConfig{
+		Type:   "gelf",
+		Config: loggerConfig,
+	}
+	client, d, handle, cleanup := dockerSetup(t, task, dockerClientConfig)
+	t.Cleanup(cleanup)
+	must.NoError(t, d.WaitUntilStarted(task.ID, 5*time.Second))
+	container, err := client.InspectContainer(handle.containerID)
+	must.NoError(t, err)
+	must.Nil(t, handle.dlogger)
+
+	must.Eq(t, "gelf", container.HostConfig.LogConfig.Type)
+	must.Eq(t, loggerConfig, container.HostConfig.LogConfig.Config)
+}
+
 func TestDockerDriver_HealthchecksDisable(t *testing.T) {
 	ci.Parallel(t)
 	testutil.DockerCompatible(t)
@@ -1479,10 +1523,10 @@ func TestDockerDriver_DNS(t *testing.T) {
 		require.NoError(t, task.EncodeConcreteDriverConfig(cfg))
 
 		_, d, _, cleanup := dockerSetup(t, task, nil)
-		defer cleanup()
+		t.Cleanup(cleanup)
 
 		require.NoError(t, d.WaitUntilStarted(task.ID, 5*time.Second))
-		defer d.DestroyTask(task.ID, true)
+		t.Cleanup(func() { _ = d.DestroyTask(task.ID, true) })
 
 		dtestutil.TestTaskDNSConfig(t, d, task.ID, c.cfg)
 	}
@@ -2143,7 +2187,7 @@ func TestDockerDriver_VolumesDisabled(t *testing.T) {
 			t.Fatalf("timeout")
 		}
 
-		if _, err := ioutil.ReadFile(filepath.Join(task.TaskDir().Dir, fn)); err != nil {
+		if _, err := os.ReadFile(filepath.Join(task.TaskDir().Dir, fn)); err != nil {
 			t.Fatalf("unexpected error reading %s: %v", fn, err)
 		}
 	}
@@ -2201,7 +2245,7 @@ func TestDockerDriver_VolumesEnabled(t *testing.T) {
 		t.Fatalf("timeout")
 	}
 
-	if _, err := ioutil.ReadFile(hostpath); err != nil {
+	if _, err := os.ReadFile(hostpath); err != nil {
 		t.Fatalf("unexpected error reading %s: %v", hostpath, err)
 	}
 }
@@ -2471,34 +2515,56 @@ func TestDockerDriver_Device_Success(t *testing.T) {
 		t.Skip("test device mounts only on linux")
 	}
 
-	hostPath := "/dev/random"
-	containerPath := "/dev/myrandom"
-	perms := "rwm"
-
-	expectedDevice := docker.Device{
-		PathOnHost:        hostPath,
-		PathInContainer:   containerPath,
-		CgroupPermissions: perms,
-	}
-	config := DockerDevice{
-		HostPath:      hostPath,
-		ContainerPath: containerPath,
+	cases := []struct {
+		Name     string
+		Input    DockerDevice
+		Expected docker.Device
+	}{
+		{
+			Name: "AllSet",
+			Input: DockerDevice{
+				HostPath:          "/dev/random",
+				ContainerPath:     "/dev/hostrandom",
+				CgroupPermissions: "rwm",
+			},
+			Expected: docker.Device{
+				PathOnHost:        "/dev/random",
+				PathInContainer:   "/dev/hostrandom",
+				CgroupPermissions: "rwm",
+			},
+		},
+		{
+			Name: "OnlyHost",
+			Input: DockerDevice{
+				HostPath: "/dev/random",
+			},
+			Expected: docker.Device{
+				PathOnHost:        "/dev/random",
+				PathInContainer:   "/dev/random",
+				CgroupPermissions: "rwm",
+			},
+		},
 	}
 
-	task, cfg, _ := dockerTask(t)
+	for i := range cases {
+		tc := cases[i]
+		t.Run(tc.Name, func(t *testing.T) {
+			task, cfg, _ := dockerTask(t)
 
-	cfg.Devices = []DockerDevice{config}
-	require.NoError(t, task.EncodeConcreteDriverConfig(cfg))
+			cfg.Devices = []DockerDevice{tc.Input}
+			require.NoError(t, task.EncodeConcreteDriverConfig(cfg))
 
-	client, driver, handle, cleanup := dockerSetup(t, task, nil)
-	defer cleanup()
-	require.NoError(t, driver.WaitUntilStarted(task.ID, 5*time.Second))
+			client, driver, handle, cleanup := dockerSetup(t, task, nil)
+			defer cleanup()
+			require.NoError(t, driver.WaitUntilStarted(task.ID, 5*time.Second))
 
-	container, err := client.InspectContainer(handle.containerID)
-	require.NoError(t, err)
+			container, err := client.InspectContainer(handle.containerID)
+			require.NoError(t, err)
 
-	require.NotEmpty(t, container.HostConfig.Devices, "Expected one device")
-	require.Equal(t, expectedDevice, container.HostConfig.Devices[0], "Incorrect device ")
+			require.NotEmpty(t, container.HostConfig.Devices, "Expected one device")
+			require.Equal(t, tc.Expected, container.HostConfig.Devices[0], "Incorrect device ")
+		})
+	}
 }
 
 func TestDockerDriver_Entrypoint(t *testing.T) {
@@ -3023,3 +3089,23 @@ func TestDockerDriver_StopSignal(t *testing.T) {
 		})
 	}
 }
+
+func TestDockerDriver_GroupAdd(t *testing.T) {
+	if !tu.IsCI() {
+		t.Parallel()
+	}
+	testutil.DockerCompatible(t)
+
+	task, cfg, _ := dockerTask(t)
+	cfg.GroupAdd = []string{"12345", "9999"}
+	require.NoError(t, task.EncodeConcreteDriverConfig(cfg))
+
+	client, d, handle, cleanup := dockerSetup(t, task, nil)
+	defer cleanup()
+	require.NoError(t, d.WaitUntilStarted(task.ID, 5*time.Second))
+
+	container, err := client.InspectContainer(handle.containerID)
+	require.NoError(t, err)
+
+	require.Exactly(t, cfg.GroupAdd, container.HostConfig.GroupAdd)
+}
