@@ -1,3 +1,6 @@
+// Copyright (c) HashiCorp, Inc.
+// SPDX-License-Identifier: MPL-2.0
+
 //go:build windows
 
 package docker
@@ -6,18 +9,13 @@ import (
 	"testing"
 
 	"github.com/hashicorp/nomad/client/allocdir"
-	tu "github.com/hashicorp/nomad/testutil"
+	"github.com/hashicorp/nomad/testutil"
 )
 
 func newTaskConfig(variant string, command []string) TaskConfig {
 	// busyboxImageID is an id of an image containing nanoserver windows and
 	// a busybox exe.
-	busyboxImageID := "hashicorpdev/busybox-windows:server2016-0.1"
-
-	if tu.IsCI() {
-		// In CI, use HashiCorp Mirror to avoid DockerHub rate limiting
-		busyboxImageID = "docker.mirror.hashicorp.services/" + busyboxImageID
-	}
+	busyboxImageID := testutil.TestBusyboxImage()
 
 	return TaskConfig{
 		Image:            busyboxImageID,
