@@ -1,3 +1,6 @@
+// Copyright (c) HashiCorp, Inc.
+// SPDX-License-Identifier: MPL-2.0
+
 package nomad
 
 import (
@@ -61,6 +64,7 @@ const (
 	ACLRoleSnapshot                      SnapshotType = 25
 	ACLAuthMethodSnapshot                SnapshotType = 26
 	ACLBindingRuleSnapshot               SnapshotType = 27
+	NodePoolSnapshot                     SnapshotType = 28
 
 	// Namespace appliers were moved from enterprise and therefore start at 64
 	NamespaceSnapshot SnapshotType = 64
@@ -224,6 +228,10 @@ func (n *nomadFSM) Apply(log *raft.Log) interface{} {
 		return n.applyStatusUpdate(msgType, buf[1:], log.Index)
 	case structs.NodeUpdateDrainRequestType:
 		return n.applyDrainUpdate(msgType, buf[1:], log.Index)
+	case structs.NodePoolUpsertRequestType:
+		return n.applyNodePoolUpsert(msgType, buf[1:], log.Index)
+	case structs.NodePoolDeleteRequestType:
+		return n.applyNodePoolDelete(msgType, buf[1:], log.Index)
 	case structs.JobRegisterRequestType:
 		return n.applyUpsertJob(msgType, buf[1:], log.Index)
 	case structs.JobDeregisterRequestType:
@@ -382,7 +390,13 @@ func (n *nomadFSM) applyUpsertNode(reqType structs.MessageType, buf []byte, inde
 	// Handle upgrade paths
 	req.Node.Canonicalize()
 
-	if err := n.state.UpsertNode(reqType, index, req.Node); err != nil {
+	// Upsert node.
+	var opts []state.NodeUpsertOption
+	if req.CreateNodePool {
+		opts = append(opts, state.NodeUpsertWithNodePool)
+	}
+
+	if err := n.state.UpsertNode(reqType, index, req.Node, opts...); err != nil {
 		n.logger.Error("UpsertNode failed", "error", err)
 		return err
 	}
@@ -470,10 +484,19 @@ func (n *nomadFSM) applyDrainUpdate(reqType structs.MessageType, buf []byte, ind
 			return fmt.Errorf("error looking up ACL token: %v", err)
 		}
 		if token == nil {
-			n.logger.Error("token did not exist during node drain update")
-			return fmt.Errorf("token did not exist during node drain update")
+			node, err := n.state.NodeBySecretID(nil, req.AuthToken)
+			if err != nil {
+				n.logger.Error("error looking up node for drain update", "error", err)
+				return fmt.Errorf("error looking up node for drain update: %v", err)
+			}
+			if node == nil {
+				n.logger.Error("token did not exist during node drain update")
+				return fmt.Errorf("token did not exist during node drain update")
+			}
+			accessorId = node.ID
+		} else {
+			accessorId = token.AccessorID
 		}
-		accessorId = token.AccessorID
 	}
 
 	if err := n.state.UpdateNodeDrain(reqType, index, req.NodeID, req.DrainStrategy, req.MarkEligible, req.UpdatedAt,
@@ -528,6 +551,36 @@ func (n *nomadFSM) applyNodeEligibilityUpdate(msgType structs.MessageType, buf [
 	return nil
 }
 
+func (n *nomadFSM) applyNodePoolUpsert(msgType structs.MessageType, buf []byte, index uint64) interface{} {
+	defer metrics.MeasureSince([]string{"nomad", "fsm", "apply_node_pool_upsert"}, time.Now())
+	var req structs.NodePoolUpsertRequest
+	if err := structs.Decode(buf, &req); err != nil {
+		panic(fmt.Errorf("failed to decode request: %v", err))
+	}
+
+	if err := n.state.UpsertNodePools(msgType, index, req.NodePools); err != nil {
+		n.logger.Error("UpsertNodePool failed", "error", err)
+		return err
+	}
+
+	return nil
+}
+
+func (n *nomadFSM) applyNodePoolDelete(msgType structs.MessageType, buf []byte, index uint64) interface{} {
+	defer metrics.MeasureSince([]string{"nomad", "fsm", "apply_node_pool_delete"}, time.Now())
+	var req structs.NodePoolDeleteRequest
+	if err := structs.Decode(buf, &req); err != nil {
+		panic(fmt.Errorf("failed to decode request: %v", err))
+	}
+
+	if err := n.state.DeleteNodePools(msgType, index, req.Names); err != nil {
+		n.logger.Error("DeleteNodePools failed", "error", err)
+		return err
+	}
+
+	return nil
+}
+
 func (n *nomadFSM) applyUpsertJob(msgType structs.MessageType, buf []byte, index uint64) interface{} {
 	defer metrics.MeasureSince([]string{"nomad", "fsm", "register_job"}, time.Now())
 	var req structs.JobRegisterRequest
@@ -544,7 +597,7 @@ func (n *nomadFSM) applyUpsertJob(msgType structs.MessageType, buf []byte, index
 	 */
 	req.Job.Canonicalize()
 
-	if err := n.state.UpsertJob(msgType, index, req.Job); err != nil {
+	if err := n.state.UpsertJob(msgType, index, req.Submission, req.Job); err != nil {
 		n.logger.Error("UpsertJob failed", "error", err)
 		return err
 	}
@@ -783,7 +836,7 @@ func (n *nomadFSM) handleJobDeregister(index uint64, jobID, namespace string, pu
 		stopped := current.Copy()
 		stopped.Stop = true
 
-		if err := n.state.UpsertJobTxn(index, stopped, tx); err != nil {
+		if err := n.state.UpsertJobTxn(index, nil, stopped, tx); err != nil {
 			return fmt.Errorf("UpsertJob failed: %w", err)
 		}
 	}
@@ -1809,6 +1862,18 @@ func (n *nomadFSM) restoreImpl(old io.ReadCloser, filter *FSMFilter) error {
 				return err
 			}
 
+		case NodePoolSnapshot:
+			pool := new(structs.NodePool)
+
+			if err := dec.Decode(pool); err != nil {
+				return err
+			}
+
+			// Perform the restoration.
+			if err := restore.NodePoolRestore(pool); err != nil {
+				return err
+			}
+
 		default:
 			// Check if this is an enterprise only object being restored
 			restorer, ok := n.enterpriseRestorers[snapType]
@@ -2267,6 +2332,10 @@ func (s *nomadSnapshot) Persist(sink raft.SnapshotSink) error {
 		sink.Cancel()
 		return err
 	}
+	if err := s.persistNodePools(sink, encoder); err != nil {
+		sink.Cancel()
+		return err
+	}
 	if err := s.persistJobs(sink, encoder); err != nil {
 		sink.Cancel()
 		return err
@@ -2429,6 +2498,27 @@ func (s *nomadSnapshot) persistNodes(sink raft.SnapshotSink,
 	return nil
 }
 
+func (s *nomadSnapshot) persistNodePools(sink raft.SnapshotSink,
+	encoder *codec.Encoder) error {
+	// Get all node pools.
+	ws := memdb.NewWatchSet()
+	pools, err := s.snap.NodePools(ws, state.SortDefault)
+	if err != nil {
+		return err
+	}
+
+	// Iterate over all node pools and persist them.
+	for raw := pools.Next(); raw != nil; raw = pools.Next() {
+		pool := raw.(*structs.NodePool)
+
+		sink.Write([]byte{byte(NodePoolSnapshot)})
+		if err := encoder.Encode(pool); err != nil {
+			return err
+		}
+	}
+	return nil
+}
+
 func (s *nomadSnapshot) persistJobs(sink raft.SnapshotSink,
 	encoder *codec.Encoder) error {
 	// Get all the jobs
