@@ -1,3 +1,6 @@
+// Copyright (c) HashiCorp, Inc.
+// SPDX-License-Identifier: MPL-2.0
+
 package nomad
 
 import (
@@ -10,6 +13,7 @@ import (
 	"time"
 
 	"github.com/google/go-cmp/cmp"
+	"github.com/google/go-cmp/cmp/cmpopts"
 	memdb "github.com/hashicorp/go-memdb"
 	"github.com/hashicorp/raft"
 	"github.com/kr/pretty"
@@ -237,6 +241,83 @@ func TestFSM_UpsertNode_Canonicalize_Ineligible(t *testing.T) {
 	require.Equal(structs.NodeSchedulingIneligible, n.SchedulingEligibility)
 }
 
+func TestFSM_UpsertNode_NodePool(t *testing.T) {
+	ci.Parallel(t)
+
+	testCases := []struct {
+		name       string
+		setupReqFn func(*structs.NodeRegisterRequest)
+		validateFn func(*testing.T, *structs.Node, *structs.NodePool)
+	}{
+		{
+			name: "node with empty node pool is placed in defualt",
+			setupReqFn: func(req *structs.NodeRegisterRequest) {
+				req.Node.NodePool = ""
+			},
+			validateFn: func(t *testing.T, node *structs.Node, pool *structs.NodePool) {
+				must.Eq(t, structs.NodePoolDefault, node.NodePool)
+				must.Eq(t, 1, pool.ModifyIndex)
+			},
+		},
+		{
+			name: "create new node pool with node",
+			setupReqFn: func(req *structs.NodeRegisterRequest) {
+				req.Node.NodePool = "new"
+				req.CreateNodePool = true
+			},
+			validateFn: func(t *testing.T, node *structs.Node, pool *structs.NodePool) {
+				must.NotNil(t, pool)
+				must.Eq(t, "new", pool.Name)
+				must.Eq(t, pool.Name, node.NodePool)
+				must.Eq(t, node.ModifyIndex, pool.CreateIndex)
+			},
+		},
+		{
+			name: "don't create new node pool with node",
+			setupReqFn: func(req *structs.NodeRegisterRequest) {
+				req.Node.NodePool = "new"
+				req.CreateNodePool = false
+			},
+			validateFn: func(t *testing.T, node *structs.Node, pool *structs.NodePool) {
+				must.Nil(t, pool)
+				must.Eq(t, "new", node.NodePool)
+			},
+		},
+	}
+
+	for _, tc := range testCases {
+		t.Run(tc.name, func(t *testing.T) {
+			fsm := testFSM(t)
+
+			node := mock.Node()
+			req := structs.NodeRegisterRequest{
+				Node: node,
+			}
+			if tc.setupReqFn != nil {
+				tc.setupReqFn(&req)
+			}
+			buf, err := structs.Encode(structs.NodeRegisterRequestType, req)
+			must.NoError(t, err)
+
+			resp := fsm.Apply(makeLog(buf))
+			must.Nil(t, resp)
+
+			// Snapshot the state.
+			s := fsm.State()
+
+			gotNode, err := s.NodeByID(nil, node.ID)
+			must.NoError(t, err)
+
+			gotPool, err := s.NodePoolByName(nil, gotNode.NodePool)
+			must.NoError(t, err)
+
+			if tc.validateFn != nil {
+				tc.validateFn(t, gotNode, gotPool)
+			}
+		})
+	}
+}
+
 func TestFSM_DeregisterNode(t *testing.T) {
 	ci.Parallel(t)
 	fsm := testFSM(t)
@@ -552,6 +633,105 @@ func TestFSM_UpdateNodeEligibility_Unblock(t *testing.T) {
 	})
 }
 
+func TestFSM_NodePoolDelete(t *testing.T) {
+	ci.Parallel(t)
+
+	// Create FSM and populate state.
+	fsm := testFSM(t)
+	pools := []*structs.NodePool{
+		mock.NodePool(),
+		mock.NodePool(),
+		mock.NodePool(),
+		mock.NodePool(),
+	}
+	err := fsm.State().UpsertNodePools(structs.MsgTypeTestSetup, 1000, pools)
+	must.NoError(t, err)
+
+	// Delete some of the node pools.
+	req := structs.NodePoolDeleteRequest{
+		Names: []string{pools[0].Name, pools[1].Name},
+	}
+	buf, err := structs.Encode(structs.NodePoolDeleteRequestType, req)
+	must.NoError(t, err)
+
+	resp := fsm.Apply(makeLog(buf))
+	must.Nil(t, resp)
+
+	// Verify selected node pools were deleted.
+	ws := memdb.NewWatchSet()
+	for i, pool := range pools {
+		got, err := fsm.State().NodePoolByName(ws, pool.Name)
+		must.NoError(t, err)
+
+		switch i {
+		// Node pools 0 and 1 were deleted.
+		case 0, 1:
+			must.Nil(t, got)
+		default:
+			must.NotNil(t, got)
+		}
+	}
+}
+
+func TestFSM_NodePoolUpsert(t *testing.T) {
+	ci.Parallel(t)
+
+	// Create FSM and create some node pools.
+	fsm := testFSM(t)
+	pools := []*structs.NodePool{
+		mock.NodePool(),
+		mock.NodePool(),
+		mock.NodePool(),
+	}
+	req := structs.NodePoolUpsertRequest{
+		NodePools: pools,
+	}
+	buf, err := structs.Encode(structs.NodePoolUpsertRequestType, req)
+	must.NoError(t, err)
+
+	resp := fsm.Apply(makeLog(buf))
+	must.Nil(t, resp)
+
+	// Verify node pools were created.
+	ws := memdb.NewWatchSet()
+	for _, pool := range pools {
+		got, err := fsm.State().NodePoolByName(ws, pool.Name)
+
+		must.NoError(t, err)
+		must.Eq(t, pool, got, must.Cmp(cmpopts.IgnoreFields(
+			structs.NodePool{},
+			"CreateIndex",
+			"ModifyIndex",
+		)))
+	}
+
+	// Update one of the node pools.
+	updatedPool := pools[0].Copy()
+	updatedPool.Description = "updated"
+	updatedPool.Meta = map[string]string{
+		"update": "true",
+	}
+
+	req = structs.NodePoolUpsertRequest{
+		NodePools: []*structs.NodePool{updatedPool},
+	}
+	buf, err = structs.Encode(structs.NodePoolUpsertRequestType, req)
+	must.NoError(t, err)
+
+	resp = fsm.Apply(makeLog(buf))
+	must.Nil(t, resp)
+
+	// Verify node pool was updated.
+	ws = memdb.NewWatchSet()
+	got, err := fsm.State().NodePoolByName(ws, updatedPool.Name)
+	must.NoError(t, err)
+	must.Eq(t, updatedPool, got, must.Cmp(cmpopts.IgnoreFields(
+		structs.NodePool{},
+		"CreateIndex",
+		"ModifyIndex",
+	)))
+}
+
 func TestFSM_RegisterJob(t *testing.T) {
 	ci.Parallel(t)
 	fsm := testFSM(t)
@@ -1763,7 +1943,7 @@ func TestFSM_JobStabilityUpdate(t *testing.T) {
 
 	// Upsert a deployment
 	job := mock.Job()
-	if err := state.UpsertJob(structs.MsgTypeTestSetup, 1, job); err != nil {
+	if err := state.UpsertJob(structs.MsgTypeTestSetup, 1, nil, job); err != nil {
 		t.Fatalf("bad: %v", err)
 	}
 
@@ -1808,7 +1988,7 @@ func TestFSM_DeploymentPromotion(t *testing.T) {
 	tg2 := tg1.Copy()
 	tg2.Name = "foo"
 	j.TaskGroups = append(j.TaskGroups, tg2)
-	if err := state.UpsertJob(structs.MsgTypeTestSetup, 1, j); err != nil {
+	if err := state.UpsertJob(structs.MsgTypeTestSetup, 1, nil, j); err != nil {
 		t.Fatalf("bad: %v", err)
 	}
 
@@ -2250,15 +2430,31 @@ func TestFSM_SnapshotRestore_Nodes(t *testing.T) {
 	}
 }
 
+func TestFSM_SnapshotRestore_NodePools(t *testing.T) {
+	ci.Parallel(t)
+
+	// Add some state
+	fsm := testFSM(t)
+	state := fsm.State()
+	pool := mock.NodePool()
+	state.UpsertNodePools(structs.MsgTypeTestSetup, 1000, []*structs.NodePool{pool})
+
+	// Verify the contents
+	fsm2 := testSnapshotRestore(t, fsm)
+	state2 := fsm2.State()
+	out, _ := state2.NodePoolByName(nil, pool.Name)
+	must.Eq(t, pool, out)
+}
+
 func TestFSM_SnapshotRestore_Jobs(t *testing.T) {
 	ci.Parallel(t)
 	// Add some state
 	fsm := testFSM(t)
 	state := fsm.State()
 	job1 := mock.Job()
-	state.UpsertJob(structs.MsgTypeTestSetup, 1000, job1)
+	state.UpsertJob(structs.MsgTypeTestSetup, 1000, nil, job1)
 	job2 := mock.Job()
-	state.UpsertJob(structs.MsgTypeTestSetup, 1001, job2)
+	state.UpsertJob(structs.MsgTypeTestSetup, 1001, nil, job2)
 
 	// Verify the contents
 	ws := memdb.NewWatchSet()
@@ -2436,12 +2632,12 @@ func TestFSM_SnapshotRestore_JobSummary(t *testing.T) {
 	state := fsm.State()
 
 	job1 := mock.Job()
-	state.UpsertJob(structs.MsgTypeTestSetup, 1000, job1)
+	state.UpsertJob(structs.MsgTypeTestSetup, 1000, nil, job1)
 	ws := memdb.NewWatchSet()
 	js1, _ := state.JobSummaryByID(ws, job1.Namespace, job1.ID)
 
 	job2 := mock.Job()
-	state.UpsertJob(structs.MsgTypeTestSetup, 1001, job2)
+	state.UpsertJob(structs.MsgTypeTestSetup, 1001, nil, job2)
 	js2, _ := state.JobSummaryByID(ws, job2.Namespace, job2.ID)
 
 	// Verify the contents
@@ -2486,10 +2682,10 @@ func TestFSM_SnapshotRestore_JobVersions(t *testing.T) {
 	fsm := testFSM(t)
 	state := fsm.State()
 	job1 := mock.Job()
-	state.UpsertJob(structs.MsgTypeTestSetup, 1000, job1)
+	state.UpsertJob(structs.MsgTypeTestSetup, 1000, nil, job1)
 	job2 := mock.Job()
 	job2.ID = job1.ID
-	state.UpsertJob(structs.MsgTypeTestSetup, 1001, job2)
+	state.UpsertJob(structs.MsgTypeTestSetup, 1001, nil, job2)
 
 	// Verify the contents
 	ws := memdb.NewWatchSet()
@@ -2520,7 +2716,7 @@ func TestFSM_SnapshotRestore_Deployments(t *testing.T) {
 	d1.JobID = j.ID
 	d2.JobID = j.ID
 
-	state.UpsertJob(structs.MsgTypeTestSetup, 999, j)
+	state.UpsertJob(structs.MsgTypeTestSetup, 999, nil, j)
 	state.UpsertDeployment(1000, d1)
 	state.UpsertDeployment(1001, d2)
 
@@ -2691,7 +2887,7 @@ func TestFSM_SnapshotRestore_ACLAuthMethods(t *testing.T) {
 	testState := fsm.State()
 
 	// Generate and upsert some ACL auth methods.
-	authMethods := []*structs.ACLAuthMethod{mock.ACLAuthMethod(), mock.ACLAuthMethod()}
+	authMethods := []*structs.ACLAuthMethod{mock.ACLOIDCAuthMethod(), mock.ACLOIDCAuthMethod()}
 	must.NoError(t, testState.UpsertACLAuthMethods(10, authMethods))
 
 	// Perform a snapshot restore.
@@ -2752,12 +2948,12 @@ func TestFSM_ReconcileSummaries(t *testing.T) {
 	// Make a job so that none of the tasks can be placed
 	job1 := mock.Job()
 	job1.TaskGroups[0].Tasks[0].Resources.CPU = 5000
-	require.NoError(t, state.UpsertJob(structs.MsgTypeTestSetup, 1000, job1))
+	require.NoError(t, state.UpsertJob(structs.MsgTypeTestSetup, 1000, nil, job1))
 
 	// make a job which can make partial progress
 	alloc := mock.Alloc()
 	alloc.NodeID = node.ID
-	require.NoError(t, state.UpsertJob(structs.MsgTypeTestSetup, 1010, alloc.Job))
+	require.NoError(t, state.UpsertJob(structs.MsgTypeTestSetup, 1010, nil, alloc.Job))
 	require.NoError(t, state.UpsertAllocs(structs.MsgTypeTestSetup, 1011, []*structs.Allocation{alloc}))
 
 	// Delete the summaries
@@ -2838,7 +3034,7 @@ func TestFSM_ReconcileParentJobSummary(t *testing.T) {
 		Payload: "random",
 	}
 	job1.TaskGroups[0].Count = 1
-	state.UpsertJob(structs.MsgTypeTestSetup, 1000, job1)
+	state.UpsertJob(structs.MsgTypeTestSetup, 1000, nil, job1)
 
 	// Make a child job
 	childJob := job1.Copy()
@@ -2854,7 +3050,7 @@ func TestFSM_ReconcileParentJobSummary(t *testing.T) {
 	alloc.JobID = childJob.ID
 	alloc.ClientStatus = structs.AllocClientStatusRunning
 
-	state.UpsertJob(structs.MsgTypeTestSetup, 1010, childJob)
+	state.UpsertJob(structs.MsgTypeTestSetup, 1010, nil, childJob)
 	state.UpsertAllocs(structs.MsgTypeTestSetup, 1011, []*structs.Allocation{alloc})
 
 	// Make the summary incorrect in the state store
@@ -3540,8 +3736,8 @@ func TestFSM_UpsertACLAuthMethods(t *testing.T) {
 	ci.Parallel(t)
 	fsm := testFSM(t)
 
-	am1 := mock.ACLAuthMethod()
-	am2 := mock.ACLAuthMethod()
+	am1 := mock.ACLOIDCAuthMethod()
+	am2 := mock.ACLOIDCAuthMethod()
 	req := structs.ACLAuthMethodUpsertRequest{
 		AuthMethods: []*structs.ACLAuthMethod{am1, am2},
 	}
@@ -3564,8 +3760,8 @@ func TestFSM_DeleteACLAuthMethods(t *testing.T) {
 	ci.Parallel(t)
 	fsm := testFSM(t)
 
-	am1 := mock.ACLAuthMethod()
-	am2 := mock.ACLAuthMethod()
+	am1 := mock.ACLOIDCAuthMethod()
+	am2 := mock.ACLOIDCAuthMethod()
 	must.Nil(t, fsm.State().UpsertACLAuthMethods(1000, []*structs.ACLAuthMethod{am1, am2}))
 
 	req := structs.ACLAuthMethodDeleteRequest{
@@ -3591,7 +3787,7 @@ func TestFSM_UpsertACLBindingRules(t *testing.T) {
 	fsm := testFSM(t)
 
 	// Create an auth method and upsert so the binding rules can link to this.
-	authMethod := mock.ACLAuthMethod()
+	authMethod := mock.ACLOIDCAuthMethod()
 	must.NoError(t, fsm.state.UpsertACLAuthMethods(10, []*structs.ACLAuthMethod{authMethod}))
 
 	aclBindingRule1 := mock.ACLBindingRule()
