@@ -1,3 +1,6 @@
+// Copyright (c) HashiCorp, Inc.
+// SPDX-License-Identifier: MPL-2.0
+
 package nomad
 
 import (
@@ -49,6 +52,10 @@ const (
 	// NodeHeartbeatEventReregistered is the message used when the node becomes
 	// reregistered by the heartbeat.
 	NodeHeartbeatEventReregistered = "Node reregistered by heartbeat"
+
+	// NodeWaitingForNodePool is the message used when the node is waiting for
+	// its node pool to be created.
+	NodeWaitingForNodePool = "Node registered but waiting for node pool to be created"
 )
 
 // Node endpoint is used for client interactions
@@ -132,6 +139,15 @@ func (n *Node) Register(args *structs.NodeRegisterRequest, reply *structs.NodeUp
 	if args.Node.SecretID == "" {
 		return fmt.Errorf("missing node secret ID for client registration")
 	}
+	if args.Node.NodePool != "" {
+		err := structs.ValidateNodePoolName(args.Node.NodePool)
+		if err != nil {
+			return fmt.Errorf("invalid node pool: %v", err)
+		}
+		if args.Node.NodePool == structs.NodePoolAll {
+			return fmt.Errorf("node is not allowed to register in node pool %q", structs.NodePoolAll)
+		}
+	}
 
 	// Default the status if none is given
 	if args.Node.Status == "" {
@@ -146,6 +162,11 @@ func (n *Node) Register(args *structs.NodeRegisterRequest, reply *structs.NodeUp
 		args.Node.SchedulingEligibility = structs.NodeSchedulingEligible
 	}
 
+	// Default the node pool if none is given.
+	if args.Node.NodePool == "" {
+		args.Node.NodePool = structs.NodePoolDefault
+	}
+
 	// Set the timestamp when the node is registered
 	args.Node.StatusUpdatedAt = time.Now().Unix()
 
@@ -187,7 +208,18 @@ func (n *Node) Register(args *structs.NodeRegisterRequest, reply *structs.NodeUp
 		n.srv.addNodeConn(n.ctx)
 	}
 
-	// Commit this update via Raft
+	// Commit this update via Raft.
+	//
+	// Only the authoritative region is allowed to create the node pool for the
+	// node if it doesn't exist yet. This prevents non-authoritative regions
+	// from having to push their local state to the authoritative region.
+	//
+	// Nodes in non-authoritative regions that are registered with a new node
+	// pool are kept in the `initializing` status until the node pool is
+	// created and replicated.
+	if n.srv.Region() == n.srv.config.AuthoritativeRegion {
+		args.CreateNodePool = true
+	}
 	_, index, err := n.srv.raftApply(structs.NodeRegisterRequestType, args)
 	if err != nil {
 		n.logger.Error("register failed", "error", err)
@@ -283,7 +315,7 @@ func equalDevices(n1, n2 *structs.Node) bool {
 	return reflect.DeepEqual(n1.NodeResources.Devices, n2.NodeResources.Devices)
 }
 
-// updateNodeUpdateResponse assumes the n.srv.peerLock is held for reading.
+// constructNodeServerInfoResponse assumes the n.srv.peerLock is held for reading.
 func (n *Node) constructNodeServerInfoResponse(nodeID string, snap *state.StateSnapshot, reply *structs.NodeUpdateResponse) error {
 	reply.LeaderRPCAddr = string(n.srv.raft.Leader())
 
@@ -297,16 +329,30 @@ func (n *Node) constructNodeServerInfoResponse(nodeID string, snap *state.StateS
 			})
 	}
 
+	ws := memdb.NewWatchSet()
+
 	// Add ClientStatus information to heartbeat response.
-	node, _ := snap.NodeByID(nil, nodeID)
-	reply.SchedulingEligibility = node.SchedulingEligibility
+	if node, err := snap.NodeByID(ws, nodeID); err == nil && node != nil {
+		reply.SchedulingEligibility = node.SchedulingEligibility
+	} else if node == nil {
+
+		// If the node is not found, leave reply.SchedulingEligibility as
+		// the empty string. The response handler in the client treats this
+		// as a no-op. As there is no call to action for an operator, log it
+		// at debug level.
+		n.logger.Debug("constructNodeServerInfoResponse: node not found",
+			"node_id", nodeID)
+	} else {
+
+		// This case is likely only reached via a code error in state store
+		return err
+	}
 
 	// TODO(sean@): Use an indexed node count instead
 	//
 	// Snapshot is used only to iterate over all nodes to create a node
 	// count to send back to Nomad Clients in their heartbeat so Clients
 	// can estimate the size of the cluster.
-	ws := memdb.NewWatchSet()
 	iter, err := snap.Nodes(ws)
 	if err == nil {
 		for {
@@ -463,15 +509,24 @@ func (n *Node) deregister(args *structs.NodeBatchDeregisterRequest,
 // Clients with non-terminal allocations must first call UpdateAlloc to be able
 // to transition from the initializing status to ready.
 //
+// Clients node pool must exist for them to be able to transition from
+// initializing to ready.
+//
 //	                ┌────────────────────────────────────── No ───┐
 //	                │                                             │
 //	             ┌──▼───┐          ┌─────────────┐       ┌────────┴────────┐
 //	── Register ─► init ├─ ready ──► Has allocs? ├─ Yes ─► Allocs updated? │
-//	             └──▲───┘          └─────┬───────┘       └────────┬────────┘
-//	                │                    │                        │
-//	              ready                  └─ No ─┐  ┌─────── Yes ──┘
-//	                │                           │  │
-//	         ┌──────┴───────┐                ┌──▼──▼─┐         ┌──────┐
+//	             └──▲──▲┘          └─────┬───────┘       └────────┬────────┘
+//	                │  │                 │                        │
+//	                │  │                 └─ No ─┐  ┌─────── Yes ──┘
+//	                │  │                        │  │
+//	                │  │               ┌────────▼──▼───────┐
+//	                │  └──────────No───┤ Node pool exists? │
+//	                │                  └─────────┬─────────┘
+//	                │                            │
+//	              ready                         Yes
+//	                │                            │
+//	         ┌──────┴───────┐                ┌───▼───┐         ┌──────┐
 //	         │ disconnected ◄─ disconnected ─┤ ready ├─ down ──► down │
 //	         └──────────────┘                └───▲───┘         └──┬───┘
 //	                                             │                │
@@ -539,6 +594,8 @@ func (n *Node) UpdateStatus(args *structs.NodeUpdateStatusRequest, reply *struct
 	switch node.Status {
 	case structs.NodeStatusInit:
 		if args.Status == structs.NodeStatusReady {
+			// Keep node in the initializing status if it has allocations but
+			// they are not updated.
 			allocs, err := snap.AllocsByNodeTerminal(ws, args.NodeID, false)
 			if err != nil {
 				return fmt.Errorf("failed to query node allocs: %v", err)
@@ -546,8 +603,26 @@ func (n *Node) UpdateStatus(args *structs.NodeUpdateStatusRequest, reply *struct
 
 			allocsUpdated := node.LastAllocUpdateIndex > node.LastMissedHeartbeatIndex
 			if len(allocs) > 0 && !allocsUpdated {
+				n.logger.Debug(fmt.Sprintf("marking node as %s due to outdated allocation information", structs.NodeStatusInit))
 				args.Status = structs.NodeStatusInit
 			}
+
+			// Keep node in the initialing status if it's in a node pool that
+			// doesn't exist.
+			pool, err := snap.NodePoolByName(ws, node.NodePool)
+			if err != nil {
+				return fmt.Errorf("failed to query node pool: %v", err)
+			}
+			if pool == nil {
+				n.logger.Debug(fmt.Sprintf("marking node as %s due to missing node pool", structs.NodeStatusInit))
+				args.Status = structs.NodeStatusInit
+				if !node.HasEvent(NodeWaitingForNodePool) {
+					args.NodeEvent = structs.NewNodeEvent().
+						SetSubsystem(structs.NodeEventSubsystemCluster).
+						SetMessage(NodeWaitingForNodePool).
+						AddDetail("node_pool", node.NodePool)
+				}
+			}
 		}
 	case structs.NodeStatusDisconnected:
 		if args.Status == structs.NodeStatusReady {
@@ -557,7 +632,7 @@ func (n *Node) UpdateStatus(args *structs.NodeUpdateStatusRequest, reply *struct
 
 	// Commit this update via Raft
 	var index uint64
-	if node.Status != args.Status {
+	if node.Status != args.Status || args.NodeEvent != nil {
 		// Attach an event if we are updating the node status to ready when it
 		// is down via a heartbeat
 		if node.Status == structs.NodeStatusDown && args.NodeEvent == nil {
@@ -1615,12 +1690,11 @@ func (n *Node) createNodeEvals(node *structs.Node, nodeIndex uint64) ([]string,
 	var sysJobs []*structs.Job
 	for jobI := sysJobsIter.Next(); jobI != nil; jobI = sysJobsIter.Next() {
 		job := jobI.(*structs.Job)
-		// Avoid creating evals for jobs that don't run in this
-		// datacenter. We could perform an entire feasibility check
-		// here, but datacenter is a good optimization to start with as
-		// datacenter cardinality tends to be low so the check
-		// shouldn't add much work.
-		if node.IsInAnyDC(job.Datacenters) {
+		// Avoid creating evals for jobs that don't run in this datacenter or
+		// node pool. We could perform an entire feasibility check here, but
+		// datacenter/pool is a good optimization to start with as their
+		// cardinality tends to be low so the check shouldn't add much work.
+		if node.IsInPool(job.NodePool) && node.IsInAnyDC(job.Datacenters) {
 			sysJobs = append(sysJobs, job)
 		}
 	}
