@@ -1,3 +1,6 @@
+// Copyright (c) HashiCorp, Inc.
+// SPDX-License-Identifier: MPL-2.0
+
 package nomad
 
 import (
@@ -356,16 +359,16 @@ func TestPeriodicDispatch_Remove_TriggersUpdate(t *testing.T) {
 	}
 }
 
-func TestPeriodicDispatch_ForceRun_Untracked(t *testing.T) {
+func TestPeriodicDispatch_ForceEval_Untracked(t *testing.T) {
 	ci.Parallel(t)
 	p, _ := testPeriodicDispatcher(t)
 
-	if _, err := p.ForceRun("ns", "foo"); err == nil {
-		t.Fatal("ForceRun of untracked job should fail")
+	if _, err := p.ForceEval("ns", "foo"); err == nil {
+		t.Fatal("ForceEval of untracked job should fail")
 	}
 }
 
-func TestPeriodicDispatch_ForceRun_Tracked(t *testing.T) {
+func TestPeriodicDispatch_ForceEval_Tracked(t *testing.T) {
 	ci.Parallel(t)
 	p, m := testPeriodicDispatcher(t)
 
@@ -377,9 +380,9 @@ func TestPeriodicDispatch_ForceRun_Tracked(t *testing.T) {
 		t.Fatalf("Add failed %v", err)
 	}
 
-	// ForceRun the job
-	if _, err := p.ForceRun(job.Namespace, job.ID); err != nil {
-		t.Fatalf("ForceRun failed %v", err)
+	// ForceEval the job
+	if _, err := p.ForceEval(job.Namespace, job.ID); err != nil {
+		t.Fatalf("ForceEval failed %v", err)
 	}
 
 	// Check that job was launched correctly.
@@ -672,7 +675,7 @@ func TestPeriodicDispatch_RunningChildren_NoEvals(t *testing.T) {
 	// Insert job.
 	state := s1.fsm.State()
 	job := mock.PeriodicJob()
-	if err := state.UpsertJob(structs.MsgTypeTestSetup, 1000, job); err != nil {
+	if err := state.UpsertJob(structs.MsgTypeTestSetup, 1000, nil, job); err != nil {
 		t.Fatalf("UpsertJob failed: %v", err)
 	}
 
@@ -696,12 +699,12 @@ func TestPeriodicDispatch_RunningChildren_ActiveEvals(t *testing.T) {
 	// Insert periodic job and child.
 	state := s1.fsm.State()
 	job := mock.PeriodicJob()
-	if err := state.UpsertJob(structs.MsgTypeTestSetup, 1000, job); err != nil {
+	if err := state.UpsertJob(structs.MsgTypeTestSetup, 1000, nil, job); err != nil {
 		t.Fatalf("UpsertJob failed: %v", err)
 	}
 
 	childjob := deriveChildJob(job)
-	if err := state.UpsertJob(structs.MsgTypeTestSetup, 1001, childjob); err != nil {
+	if err := state.UpsertJob(structs.MsgTypeTestSetup, 1001, nil, childjob); err != nil {
 		t.Fatalf("UpsertJob failed: %v", err)
 	}
 
@@ -733,12 +736,12 @@ func TestPeriodicDispatch_RunningChildren_ActiveAllocs(t *testing.T) {
 	// Insert periodic job and child.
 	state := s1.fsm.State()
 	job := mock.PeriodicJob()
-	if err := state.UpsertJob(structs.MsgTypeTestSetup, 1000, job); err != nil {
+	if err := state.UpsertJob(structs.MsgTypeTestSetup, 1000, nil, job); err != nil {
 		t.Fatalf("UpsertJob failed: %v", err)
 	}
 
 	childjob := deriveChildJob(job)
-	if err := state.UpsertJob(structs.MsgTypeTestSetup, 1001, childjob); err != nil {
+	if err := state.UpsertJob(structs.MsgTypeTestSetup, 1001, nil, childjob); err != nil {
 		t.Fatalf("UpsertJob failed: %v", err)
 	}
 
