@@ -1,3 +1,6 @@
+// Copyright (c) HashiCorp, Inc.
+// SPDX-License-Identifier: MPL-2.0
+
 package nomad
 
 import (
@@ -30,6 +33,7 @@ var (
 		structs.Allocs,
 		structs.Jobs,
 		structs.Nodes,
+		structs.NodePools,
 		structs.Evals,
 		structs.Deployments,
 		structs.Plugins,
@@ -72,6 +76,8 @@ func (s *Search) getPrefixMatches(iter memdb.ResultIterator, prefix string) ([]s
 			id = t.ID
 		case *structs.Node:
 			id = t.ID
+		case *structs.NodePool:
+			id = t.Name
 		case *structs.Deployment:
 			id = t.ID
 		case *structs.CSIPlugin:
@@ -213,6 +219,10 @@ func (s *Search) fuzzyMatchSingle(raw interface{}, text string) (structs.Context
 		name = t.Name
 		scope = []string{t.ID}
 		ctx = structs.Nodes
+	case *structs.NodePool:
+		name = t.Name
+		scope = []string{t.Name}
+		ctx = structs.NodePools
 	case *structs.Namespace:
 		name = t.Name
 		ctx = structs.Namespaces
@@ -378,6 +388,15 @@ func getResourceIter(context structs.Context, aclObj *acl.ACL, namespace, prefix
 		return store.AllocsByIDPrefix(ws, namespace, prefix, state.SortDefault)
 	case structs.Nodes:
 		return store.NodesByIDPrefix(ws, prefix)
+	case structs.NodePools:
+		iter, err := store.NodePoolsByNamePrefix(ws, prefix, state.SortDefault)
+		if err != nil {
+			return nil, err
+		}
+		if aclObj == nil || aclObj.IsManagement() {
+			return iter, nil
+		}
+		return memdb.NewFilterIterator(iter, nodePoolCapFilter(aclObj)), nil
 	case structs.Deployments:
 		return store.DeploymentsByIDPrefix(ws, namespace, prefix, state.SortDefault)
 	case structs.Plugins:
@@ -446,6 +465,17 @@ func getFuzzyResourceIterator(context structs.Context, aclObj *acl.ACL, namespac
 		}
 		return store.Nodes(ws)
 
+	case structs.NodePools:
+		iter, err := store.NodePools(ws, state.SortDefault)
+		if err != nil {
+			return nil, err
+		}
+
+		if aclObj == nil || aclObj.IsManagement() {
+			return iter, nil
+		}
+		return memdb.NewFilterIterator(iter, nodePoolCapFilter(aclObj)), nil
+
 	case structs.Plugins:
 		if wildcard(namespace) {
 			iter, err := store.CSIPlugins(ws)
@@ -504,6 +534,15 @@ func nsCapFilter(aclObj *acl.ACL) memdb.FilterFunc {
 	}
 }
 
+// nodePoolCapFilter produces a memdb.FilterFunc for removing node pools not
+// accessible by aclObj during a table scan.
+func nodePoolCapFilter(aclObj *acl.ACL) memdb.FilterFunc {
+	return func(v interface{}) bool {
+		pool := v.(*structs.NodePool)
+		return !aclObj.AllowNodePoolOperation(pool.Name, acl.NodePoolCapabilityRead)
+	}
+}
+
 // If the length of a prefix is odd, return a subset to the last even character
 // This only applies to UUIDs, jobs are excluded
 func roundUUIDDownIfOdd(prefix string, context structs.Context) string {
@@ -620,8 +659,11 @@ func (s *Search) PrefixSearch(args *structs.SearchRequest, reply *structs.Search
 	return s.srv.blockingRPC(&opts)
 }
 
-// sufficientSearchPerms returns true if the provided ACL has access to any
-// capabilities required for prefix searching.
+// sufficientSearchPerms returns true if the provided ACL has access to *any*
+// capabilities required for prefix searching. This is intended as a performance
+// improvement so that we don't do expensive queries and then filter the results
+// if the user will never get any results. The caller still needs to filter
+// anything it gets from the state store.
 //
 // Returns true if aclObj is nil or is for a management token
 func sufficientSearchPerms(aclObj *acl.ACL, namespace string, context structs.Context) bool {
@@ -629,25 +671,23 @@ func sufficientSearchPerms(aclObj *acl.ACL, namespace string, context structs.Co
 		return true
 	}
 
-	nodeRead := aclObj.AllowNodeRead()
-	allowNS := aclObj.AllowNamespace(namespace)
-	jobRead := aclObj.AllowNsOp(namespace, acl.NamespaceCapabilityReadJob)
-	allowEnt := sufficientSearchPermsEnt(aclObj)
-
-	if !nodeRead && !allowNS && !allowEnt && !jobRead {
-		return false
-	}
-
 	// Reject requests that explicitly specify a disallowed context. This
 	// should give the user better feedback than simply filtering out all
 	// results and returning an empty list.
 	switch context {
 	case structs.Nodes:
-		return nodeRead
+		return aclObj.AllowNodeRead()
+	case structs.NodePools:
+		// The search term alone is not enough to determine if the token is
+		// allowed to access the given prefix since it may not match node pool
+		// label in the policy. Node pools will be filtered when iterating over
+		// the results.
+		return aclObj.AllowNodePoolSearch()
 	case structs.Namespaces:
-		return allowNS
-	case structs.Allocs, structs.Deployments, structs.Evals, structs.Jobs:
-		return jobRead
+		return aclObj.AllowNamespace(namespace)
+	case structs.Allocs, structs.Deployments, structs.Evals, structs.Jobs,
+		structs.ScalingPolicies, structs.Recommendations:
+		return aclObj.AllowNsOp(namespace, acl.NamespaceCapabilityReadJob)
 	case structs.Volumes:
 		return acl.NamespaceValidator(acl.NamespaceCapabilityCSIListVolume,
 			acl.NamespaceCapabilityCSIReadVolume,
@@ -655,6 +695,10 @@ func sufficientSearchPerms(aclObj *acl.ACL, namespace string, context structs.Co
 			acl.NamespaceCapabilityReadJob)(aclObj, namespace)
 	case structs.Variables:
 		return aclObj.AllowVariableSearch(namespace)
+	case structs.Plugins:
+		return aclObj.AllowPluginList()
+	case structs.Quotas:
+		return aclObj.AllowQuotaRead()
 	}
 
 	return true
@@ -670,7 +714,7 @@ func sufficientSearchPerms(aclObj *acl.ACL, namespace string, context structs.Co
 //
 // These types are available for fuzzy searching:
 //
-//	Nodes, Namespaces, Jobs, Allocs, Plugins
+//	Nodes, Node Pools, Namespaces, Jobs, Allocs, Plugins, Variables
 //
 // Jobs are a special case that expand into multiple types, and whose return
 // values include Scope which is a descending list of IDs of parent objects,
@@ -881,17 +925,25 @@ func filteredSearchContexts(aclObj *acl.ACL, namespace string, context structs.C
 				available = append(available, c)
 			}
 		case structs.Variables:
-			if jobRead {
+			if aclObj.AllowVariableSearch(namespace) {
 				available = append(available, c)
 			}
 		case structs.Nodes:
 			if aclObj.AllowNodeRead() {
 				available = append(available, c)
 			}
+		case structs.NodePools:
+			if aclObj.AllowNodePoolSearch() {
+				available = append(available, c)
+			}
 		case structs.Volumes:
 			if volRead {
 				available = append(available, c)
 			}
+		case structs.Plugins:
+			if aclObj.AllowPluginList() {
+				available = append(available, c)
+			}
 		default:
 			if ok := filteredSearchContextsEnt(aclObj, namespace, c); ok {
 				available = append(available, c)
