@@ -1,3 +1,6 @@
+// Copyright (c) HashiCorp, Inc.
+// SPDX-License-Identifier: MPL-2.0
+
 package nomad
 
 import (
@@ -215,6 +218,13 @@ type Server struct {
 	// volumeWatcher is used to release volume claims
 	volumeWatcher *volumewatcher.Watcher
 
+	// volumeControllerFutures is a map of plugin IDs to pending controller RPCs. If
+	// no RPC is pending for a given plugin, this may be nil.
+	volumeControllerFutures map[string]context.Context
+
+	// volumeControllerLock synchronizes access controllerFutures map
+	volumeControllerLock sync.Mutex
+
 	// keyringReplicator is used to replicate root encryption keys from the
 	// leader
 	keyringReplicator *KeyringReplicator
@@ -285,36 +295,6 @@ type Server struct {
 	shutdownCh     <-chan struct{}
 }
 
-// Holds the RPC endpoints
-type endpoints struct {
-	Status              *Status
-	Node                *Node
-	Job                 *Job
-	CSIVolume           *CSIVolume
-	CSIPlugin           *CSIPlugin
-	Deployment          *Deployment
-	Region              *Region
-	Search              *Search
-	Periodic            *Periodic
-	System              *System
-	Operator            *Operator
-	ACL                 *ACL
-	Scaling             *Scaling
-	Enterprise          *EnterpriseEndpoints
-	Event               *Event
-	Namespace           *Namespace
-	Variables           *Variables
-	Keyring             *Keyring
-	ServiceRegistration *ServiceRegistration
-
-	// Client endpoints
-	ClientStats       *ClientStats
-	FileSystem        *FileSystem
-	Agent             *Agent
-	ClientAllocations *ClientAllocations
-	ClientCSI         *ClientCSI
-}
-
 // NewServer is used to construct a new Nomad server from the
 // configuration, potentially returning an error
 func NewServer(config *Config, consulCatalog consul.CatalogAPI, consulConfigEntries consul.ConfigAPI, consulACLs consul.ACLsAPI) (*Server, error) {
@@ -345,6 +325,11 @@ func NewServer(config *Config, consulCatalog consul.CatalogAPI, consulConfigEntr
 	// Create the logger
 	logger := config.Logger.ResetNamedIntercept("nomad")
 
+	// Validate enterprise license before anything stateful happens
+	if err = config.LicenseConfig.Validate(); err != nil {
+		return nil, err
+	}
+
 	// Create the server
 	s := &Server{
 		config:                  config,
@@ -467,6 +452,7 @@ func NewServer(config *Config, consulCatalog consul.CatalogAPI, consulConfigEntr
 		s.logger.Error("failed to create volume watcher", "error", err)
 		return nil, fmt.Errorf("failed to create volume watcher: %v", err)
 	}
+	s.volumeControllerFutures = map[string]context.Context{}
 
 	// Start the eval broker notification system so any subscribers can get
 	// updates when the processes SetEnabled is triggered.
@@ -868,8 +854,11 @@ func (s *Server) Reload(newConfig *Config) error {
 		}
 	}
 
-	if newConfig.LicenseEnv != "" || newConfig.LicensePath != "" {
-		s.EnterpriseState.ReloadLicense(newConfig)
+	if newConfig.LicenseConfig.LicenseEnvBytes != "" || newConfig.LicenseConfig.LicensePath != "" {
+		if err = s.EnterpriseState.ReloadLicense(newConfig); err != nil {
+			s.logger.Error("error reloading license", "error", err)
+			_ = multierror.Append(&mErr, err)
+		}
 	}
 
 	// Because this is a new configuration, we extract the worker pool arguments without acquiring a lock
@@ -1275,6 +1264,7 @@ func (s *Server) setupRpcServer(server *rpc.Server, ctx *RPCContext) {
 	_ = server.Register(NewKeyringEndpoint(s, ctx, s.encrypter))
 	_ = server.Register(NewNamespaceEndpoint(s, ctx))
 	_ = server.Register(NewNodeEndpoint(s, ctx))
+	_ = server.Register(NewNodePoolEndpoint(s, ctx))
 	_ = server.Register(NewPeriodicEndpoint(s, ctx))
 	_ = server.Register(NewPlanEndpoint(s, ctx))
 	_ = server.Register(NewRegionEndpoint(s, ctx))
