@@ -1,3 +1,6 @@
+// Copyright (c) HashiCorp, Inc.
+// SPDX-License-Identifier: MPL-2.0
+
 package structs
 
 import (
@@ -350,6 +353,17 @@ func TestServiceCheck_validateNomad(t *testing.T) {
 				Body:     "this is a request payload!",
 			},
 		},
+		{
+			name: "http with tls_server_name",
+			sc: &ServiceCheck{
+				Type:          ServiceCheckHTTP,
+				Interval:      3 * time.Second,
+				Timeout:       1 * time.Second,
+				Path:          "/health",
+				TLSServerName: "foo",
+			},
+			exp: `tls_server_name may only be set for Consul service checks`,
+		},
 	}
 
 	for _, testCase := range testCases {
@@ -390,6 +404,9 @@ func TestService_Hash(t *testing.T) {
 						Config:               map[string]any{"foo": "bar"},
 					}},
 				},
+				Meta: map[string]string{
+					"test-key": "test-value",
+				},
 			},
 			// SidecarTask: nil // not hashed
 		}}
@@ -530,6 +547,9 @@ func TestConsulConnect_CopyEqual(t *testing.T) {
 					"foo": 1,
 				},
 			},
+			Meta: map[string]string{
+				"test-key": "test-value",
+			},
 		},
 	}
 
@@ -833,12 +853,18 @@ func TestConsulSidecarService_Copy(t *testing.T) {
 			Tags:  []string{"foo", "bar"},
 			Port:  "port1",
 			Proxy: &ConsulProxy{LocalServiceAddress: "10.0.0.1"},
+			Meta: map[string]string{
+				"test-key": "test-value",
+			},
 		}
 		result := s.Copy()
 		require.Equal(t, &ConsulSidecarService{
 			Tags:  []string{"foo", "bar"},
 			Port:  "port1",
 			Proxy: &ConsulProxy{LocalServiceAddress: "10.0.0.1"},
+			Meta: map[string]string{
+				"test-key": "test-value",
+			},
 		}, result)
 	})
 }
