@@ -1,9 +1,16 @@
+/**
+ * Copyright (c) HashiCorp, Inc.
+ * SPDX-License-Identifier: MPL-2.0
+ */
+
+// @ts-check
 import Controller from '@ember/controller';
 import { alias } from '@ember/object/computed';
 import { inject as service } from '@ember/service';
 import WithNamespaceResetting from 'nomad-ui/mixins/with-namespace-resetting';
 import classic from 'ember-classic-decorator';
 import { action } from '@ember/object';
+import { tracked } from '@glimmer/tracking';
 @classic
 export default class IndexController extends Controller.extend(
   WithNamespaceResetting
@@ -21,6 +28,7 @@ export default class IndexController extends Controller.extend(
       sortDescending: 'desc',
     },
     'activeTask',
+    'statusMode',
   ];
 
   currentPage = 1;
@@ -29,14 +37,29 @@ export default class IndexController extends Controller.extend(
 
   sortProperty = 'name';
   sortDescending = false;
-  activeTask = null;
+
+  @tracked activeTask = null;
+
+  /**
+   * @type {('current'|'historical')}
+   */
+  @tracked
+  statusMode = 'current';
 
   @action
   setActiveTaskQueryParam(task) {
     if (task) {
-      this.set('activeTask', `${task.allocation.id}-${task.name}`);
+      this.activeTask = `${task.allocation.id}-${task.name}`;
     } else {
-      this.set('activeTask', null);
+      this.activeTask = null;
     }
   }
+
+  /**
+   * @param {('current'|'historical')} mode
+   */
+  @action
+  setStatusMode(mode) {
+    this.statusMode = mode;
+  }
 }
