@@ -1,3 +1,8 @@
+/**
+ * Copyright (c) HashiCorp, Inc.
+ * SPDX-License-Identifier: MPL-2.0
+ */
+
 import Controller, { inject as controller } from '@ember/controller';
 import { inject as service } from '@ember/service';
 import { action } from '@ember/object';
@@ -15,6 +20,10 @@ export default class VariablesIndexController extends Controller {
     this.router.transitionTo('variables.variable', variable.path);
   }
 
+  @action goToNewVariable() {
+    this.router.transitionTo('variables.new');
+  }
+
   @controller variables;
 
   @action
