@@ -1,9 +1,17 @@
+/**
+ * Copyright (c) HashiCorp, Inc.
+ * SPDX-License-Identifier: MPL-2.0
+ */
+
 import Controller, { inject as controller } from '@ember/controller';
+import { inject as service } from '@ember/service';
 import { action } from '@ember/object';
 
 const ALL_NAMESPACE_WILDCARD = '*';
 
 export default class VariablesPathController extends Controller {
+  @service router;
+
   get absolutePath() {
     return this.model?.absolutePath || '';
   }
@@ -23,6 +31,12 @@ export default class VariablesPathController extends Controller {
     }
   }
 
+  @action goToNewVariable() {
+    this.router.transitionTo('variables.new', {
+      queryParams: { path: `${this.absolutePath}/` },
+    });
+  }
+
   @controller variables;
 
   @action
