@@ -1,3 +1,8 @@
+/**
+ * Copyright (c) HashiCorp, Inc.
+ * SPDX-License-Identifier: MPL-2.0
+ */
+
 import Mixin from '@ember/object/mixin';
 import { computed } from '@ember/object';
 import { assert } from '@ember/debug';
@@ -36,13 +41,16 @@ export default Mixin.create(WithVisibilityDetection, {
   actions: {
     willTransition(transition) {
       // Don't cancel watchers if transitioning into a sub-route
+      // Make sure, if it starts with the route name, that it's not the same route
       if (
         !transition.intent.name ||
-        !transition.intent.name.startsWith(this.routeName)
+        !(
+          transition.intent.name.startsWith(this.routeName) &&
+          this.routeName !== transition.intent.name
+        )
       ) {
         this.cancelAllWatchers();
       }
-
       // Bubble the action up to the application route
       return true;
     },
