@@ -1,3 +1,8 @@
+/**
+ * Copyright (c) HashiCorp, Inc.
+ * SPDX-License-Identifier: MPL-2.0
+ */
+
 import { inject as service } from '@ember/service';
 import Route from '@ember/routing/route';
 import RSVP from 'rsvp';
@@ -25,6 +30,7 @@ export default class IndexRoute extends Route.extend(
         .query('job', { namespace: params.qpNamespace, meta: true })
         .catch(notifyForbidden(this)),
       namespaces: this.store.findAll('namespace'),
+      nodePools: this.store.findAll('node-pool'),
     });
   }
 
