@@ -1,3 +1,8 @@
+/**
+ * Copyright (c) HashiCorp, Inc.
+ * SPDX-License-Identifier: MPL-2.0
+ */
+
 import Ember from 'ember';
 import Response from 'ember-cli-mirage/response';
 import { HOSTS } from './common';
@@ -107,6 +112,19 @@ export default function () {
     return new Response(200, {}, this.serialize(job));
   });
 
+  this.get('/job/:id/submission', function (schema, req) {
+    return new Response(
+      200,
+      {},
+      JSON.stringify({
+        Source: 'the job source v0',
+        Format: 'hcl2',
+        VariableFlags: { X: 'x', Y: '42', Z: 'true' },
+        Variables: 'var file content',
+      })
+    );
+  });
+
   this.post('/job/:id/plan', function (schema, req) {
     const body = JSON.parse(req.requestBody);
 
@@ -320,6 +338,10 @@ export default function () {
     return this.serialize(nodes.find(params.id));
   });
 
+  this.get('/node/pools', function ({ nodePools }) {
+    return this.serialize(nodePools.all());
+  });
+
   this.get('/allocations');
 
   this.get('/allocation/:id');
@@ -477,6 +499,23 @@ export default function () {
     return new Response(400, {}, null);
   });
 
+  this.post('/acl/login', function (schema, { requestBody }) {
+    const { LoginToken } = JSON.parse(requestBody);
+    const tokenType = LoginToken.endsWith('management')
+      ? 'management'
+      : 'client';
+    const isBad = LoginToken.endsWith('bad');
+
+    if (isBad) {
+      return new Response(403, {}, null);
+    } else {
+      const token = schema.tokens
+        .all()
+        .models.find((token) => token.type === tokenType);
+      return this.serialize(token);
+    }
+  });
+
   this.get('/acl/token/:id', function ({ tokens }, req) {
     const token = tokens.find(req.params.id);
     const secret = req.requestHeaders['X-Nomad-Token'];
