@@ -1,3 +1,8 @@
+/**
+ * Copyright (c) HashiCorp, Inc.
+ * SPDX-License-Identifier: MPL-2.0
+ */
+
 import { assign } from '@ember/polyfills';
 import config from 'nomad-ui/config/environment';
 import * as topoScenarios from './topo';
@@ -25,7 +30,7 @@ export const allScenarios = {
   ...sysbatchScenarios,
 };
 
-const scenario =
+export const scenario =
   getScenarioQueryParameter() ||
   getConfigValue('mirageScenario', 'emptyCluster');
 
@@ -39,9 +44,12 @@ export default function (server) {
     );
   }
 
+  // Make sure built-in node pools exist.
+  createBuiltInNodePools(server);
   if (withNamespaces) createNamespaces(server);
   if (withTokens) createTokens(server);
   if (withRegions) createRegions(server);
+
   activeScenario(server);
 }
 
@@ -51,6 +59,7 @@ function smallCluster(server) {
   faker.seed(1);
   server.create('feature', { name: 'Dynamic Application Sizing' });
   server.createList('agent', 3, 'withConsulLink', 'withVaultLink');
+  server.createList('node-pool', 2);
   server.createList('node', 5);
   server.create(
     'node',
@@ -68,6 +77,105 @@ function smallCluster(server) {
     id: 'service-haver',
     namespaceId: 'default',
   });
+  server.create('job', {
+    createAllocations: true,
+    groupTaskCount: 150,
+    shallow: true,
+    allocStatusDistribution: {
+      running: 0.5,
+      failed: 0.05,
+      unknown: 0.2,
+      lost: 0.1,
+      complete: 0.1,
+      pending: 0.05,
+    },
+    name: 'mixed-alloc-job',
+    id: 'mixed-alloc-job',
+    namespaceId: 'default',
+    type: 'service',
+    activeDeployment: true,
+  });
+
+  //#region Active Deployment
+
+  const activelyDeployingJobGroups = 2;
+  const activelyDeployingTasksPerGroup = 100;
+
+  const activelyDeployingJob = server.create('job', {
+    createAllocations: true,
+    groupTaskCount: activelyDeployingTasksPerGroup,
+    shallow: true,
+    resourceSpec: Array(activelyDeployingJobGroups).fill(['M: 257, C: 500']),
+    noDeployments: true, // manually created below
+    activeDeployment: true,
+    allocStatusDistribution: {
+      running: 0.6,
+      failed: 0.05,
+      unknown: 0.05,
+      lost: 0,
+      complete: 0,
+      pending: 0.3,
+    },
+    name: 'actively-deploying-job',
+    id: 'actively-deploying-job',
+    namespaceId: 'default',
+    type: 'service',
+  });
+
+  server.create('deployment', false, 'active', {
+    jobId: activelyDeployingJob.id,
+    groupDesiredTotal: activelyDeployingTasksPerGroup,
+    versionNumber: 1,
+    status: 'running',
+  });
+  server.createList('allocation', 25, {
+    jobId: activelyDeployingJob.id,
+    jobVersion: 0,
+    clientStatus: 'running',
+  });
+
+  // Manipulate the above job to show a nice distribution of running, canary, etc. allocs
+  let activelyDeployingJobAllocs = server.schema.allocations
+    .all()
+    .filter((a) => a.jobId === activelyDeployingJob.id);
+  activelyDeployingJobAllocs.models
+    .filter((a) => a.clientStatus === 'running')
+    .slice(0, 10)
+    .forEach((a) =>
+      a.update({ deploymentStatus: { Healthy: false, Canary: true } })
+    );
+  activelyDeployingJobAllocs.models
+    .filter((a) => a.clientStatus === 'running')
+    .slice(10, 20)
+    .forEach((a) =>
+      a.update({ deploymentStatus: { Healthy: true, Canary: true } })
+    );
+  activelyDeployingJobAllocs.models
+    .filter((a) => a.clientStatus === 'running')
+    .slice(20, 65)
+    .forEach((a) =>
+      a.update({ deploymentStatus: { Healthy: true, Canary: false } })
+    );
+  activelyDeployingJobAllocs.models
+    .filter((a) => a.clientStatus === 'pending')
+    .slice(0, 10)
+    .forEach((a) =>
+      a.update({ deploymentStatus: { Healthy: true, Canary: true } })
+    );
+  activelyDeployingJobAllocs.models
+    .filter((a) => a.clientStatus === 'failed')
+    .slice(0, 5)
+    .forEach((a) =>
+      a.update({ deploymentStatus: { Healthy: true, Canary: false } })
+    );
+
+  //#endregion Active Deployment
+
+  server.create('job', {
+    name: 'hcl-definition-job',
+    id: 'display-hcl',
+    namespaceId: 'default',
+  });
   server.createList('allocFile', 5);
   server.create('allocFile', 'dir', { depth: 2 });
   server.createList('csi-plugin', 2);
@@ -238,10 +346,12 @@ function smallCluster(server) {
   server.create('auth-method', { name: 'vault' });
   server.create('auth-method', { name: 'auth0' });
   server.create('auth-method', { name: 'cognito' });
+  server.create('auth-method', { name: 'JWT-Local', type: 'JWT' });
 }
 
 function mediumCluster(server) {
   server.createList('agent', 3, 'withConsulLink', 'withVaultLink');
+  server.createList('node-pool', 5);
   server.createList('node', 50);
   server.createList('job', 25);
 }
@@ -251,6 +361,7 @@ function variableTestCluster(server) {
   createTokens(server);
   createNamespaces(server);
   server.createList('agent', 3, 'withConsulLink', 'withVaultLink');
+  server.createList('node-pool', 3);
   server.createList('node', 5);
   server.createList('job', 3);
   server.createList('variable', 3);
@@ -326,6 +437,7 @@ function servicesTestCluster(server) {
   faker.seed(1);
   server.create('feature', { name: 'Dynamic Application Sizing' });
   server.createList('agent', 3, 'withConsulLink', 'withVaultLink');
+  server.createList('node-pool', 3);
   server.createList('node', 5);
   server.createList('job', 1, { createRecommendations: true });
   server.create('job', {
@@ -458,12 +570,14 @@ function servicesTestCluster(server) {
 // Due to Mirage performance, large cluster scenarios will be slow
 function largeCluster(server) {
   server.createList('agent', 5);
+  server.createList('node-pool', 10);
   server.createList('node', 1000);
   server.createList('job', 100);
 }
 
 function massiveCluster(server) {
   server.createList('agent', 7);
+  server.createList('node-pool', 100);
   server.createList('node', 5000);
   server.createList('job', 2000);
 }
@@ -497,6 +611,7 @@ function allNodeTypes(server) {
 
 function everyFeature(server) {
   server.createList('agent', 3, 'withConsulLink', 'withVaultLink');
+  server.createList('node-pool', 3);
 
   server.create('node', 'forceIPv4');
   server.create('node', 'draining');
@@ -531,6 +646,11 @@ function emptyCluster(server) {
 
 // Behaviors
 
+function createBuiltInNodePools(server) {
+  server.create('node-pool', { name: 'default' });
+  server.create('node-pool', { name: 'all' });
+}
+
 function createTokens(server) {
   server.createList('token', 3);
   server.create('token', {
@@ -571,6 +691,11 @@ Accessor: ${token.accessorId}
 
 `);
   });
+
+  console.log(
+    'Alternatively, log in with a JWT. If it ends with `management`, you have full access. If it ends with `bad`, you`ll get an error. Otherwise, you`ll get a token with limited access.'
+  );
+  console.log('=====================================');
 }
 
 function getConfigValue(variableName, defaultValue) {
