@@ -1,3 +1,8 @@
+/**
+ * Copyright (c) HashiCorp, Inc.
+ * SPDX-License-Identifier: MPL-2.0
+ */
+
 import hbs from 'htmlbars-inline-precompile';
 
 export default {
@@ -429,6 +434,13 @@ export let DiffViewerWithManyChanges = () => {
                             Old: '',
                             Type: 'None',
                           },
+                          {
+                            Annotations: null,
+                            Name: 'TLSServerName',
+                            New: '',
+                            Old: '',
+                            Type: 'None',
+                          },
                           {
                             Annotations: null,
                             Name: 'TLSSkipVerify',
