@@ -1,14 +1,13 @@
+/**
+ * Copyright (c) HashiCorp, Inc.
+ * SPDX-License-Identifier: MPL-2.0
+ */
+
 /* eslint-disable qunit/require-expect */
 /* Mirage fixtures are random so we can't expect a set number of assertions */
 import AdapterError from '@ember-data/adapter/error';
 import { run } from '@ember/runloop';
-import {
-  currentURL,
-  click,
-  visit,
-  triggerEvent,
-  waitFor,
-} from '@ember/test-helpers';
+import { currentURL, click, triggerEvent, waitFor } from '@ember/test-helpers';
 import { assign } from '@ember/polyfills';
 import { module, test } from 'qunit';
 import { setupApplicationTest } from 'ember-qunit';
@@ -17,6 +16,7 @@ import a11yAudit from 'nomad-ui/tests/helpers/a11y-audit';
 import Allocation from 'nomad-ui/tests/pages/allocations/detail';
 import moment from 'moment';
 import formatHost from 'nomad-ui/utils/format-host';
+import faker from 'nomad-ui/mirage/faker';
 
 let job;
 let node;
@@ -29,6 +29,7 @@ module('Acceptance | allocation detail', function (hooks) {
   hooks.beforeEach(async function () {
     server.create('agent');
 
+    server.create('node-pool');
     node = server.create('node');
     job = server.create('job', {
       groupsCount: 1,
@@ -78,10 +79,7 @@ module('Acceptance | allocation detail', function (hooks) {
     );
     assert.ok(Allocation.execButton.isPresent);
 
-    assert.equal(
-      document.title,
-      `Allocation ${allocation.name} - Mirage - Nomad`
-    );
+    assert.ok(document.title.includes(`Allocation ${allocation.name} `));
 
     await Allocation.details.visitJob();
     assert.equal(
@@ -211,7 +209,7 @@ module('Acceptance | allocation detail', function (hooks) {
 
       assert.equal(taskRow.name, task.name, 'Name');
       assert.equal(taskRow.state, task.state, 'State');
-      assert.equal(taskRow.message, event.displayMessage, 'Event Message');
+      assert.equal(taskRow.message, event.message, 'Event Message');
       assert.equal(
         taskRow.time,
         moment(event.time / 1000000).format("MMM DD, 'YY HH:mm:ss ZZ"),
@@ -464,11 +462,11 @@ module('Acceptance | allocation detail', function (hooks) {
 
     component.onClick();
 
-    await waitFor('.flash-message.alert-error');
+    await waitFor('.flash-message.alert-critical');
 
     assert.verifySteps(['Transition dispatched.']);
     assert
-      .dom('.flash-message.alert-error')
+      .dom('.flash-message.alert-critical')
       .exists('A toast error message pops up.');
 
     // Clean-up
@@ -483,6 +481,7 @@ module('Acceptance | allocation detail (rescheduled)', function (hooks) {
   hooks.beforeEach(async function () {
     server.create('agent');
 
+    server.create('node-pool');
     node = server.create('node');
     job = server.create('job', { createAllocations: false });
     allocation = server.create('allocation', 'rescheduled');
@@ -505,6 +504,7 @@ module('Acceptance | allocation detail (not running)', function (hooks) {
   hooks.beforeEach(async function () {
     server.create('agent');
 
+    server.create('node-pool');
     node = server.create('node');
     job = server.create('job', { createAllocations: false });
     allocation = server.create('allocation', { clientStatus: 'pending' });
@@ -535,6 +535,7 @@ module('Acceptance | allocation detail (preemptions)', function (hooks) {
 
   hooks.beforeEach(async function () {
     server.create('agent');
+    server.create('node-pool');
     node = server.create('node');
     job = server.create('job', { createAllocations: false });
   });
@@ -673,6 +674,7 @@ module('Acceptance | allocation detail (services)', function (hooks) {
   hooks.beforeEach(async function () {
     server.create('feature', { name: 'Dynamic Application Sizing' });
     server.createList('agent', 3, 'withConsulLink', 'withVaultLink');
+    server.createList('node-pool', 3);
     server.createList('node', 5);
     server.createList('job', 1, { createRecommendations: true });
     const job = server.create('job', {
@@ -683,7 +685,11 @@ module('Acceptance | allocation detail (services)', function (hooks) {
       namespaceId: 'default',
     });
 
-    const currentAlloc = server.db.allocations.findBy({ jobId: job.id });
+    const runningAlloc = server.create('allocation', {
+      jobId: job.id,
+      forceRunningClientStatus: true,
+      clientStatus: 'running',
+    });
     const otherAlloc = server.db.allocations.reject((j) => j.jobId !== job.id);
 
     server.db.serviceFragments.update({
@@ -692,7 +698,7 @@ module('Acceptance | allocation detail (services)', function (hooks) {
           Status: 'success',
           Check: 'check1',
           Timestamp: 99,
-          Alloc: currentAlloc.id,
+          Alloc: runningAlloc.id,
         },
         {
           Status: 'failure',
@@ -701,21 +707,21 @@ module('Acceptance | allocation detail (services)', function (hooks) {
           propThatDoesntMatter:
             'this object will be ignored, since it shared a Check name with a later one.',
           Timestamp: 98,
-          Alloc: currentAlloc.id,
+          Alloc: runningAlloc.id,
         },
         {
           Status: 'success',
           Check: 'check2',
           Output: 'Two',
           Timestamp: 99,
-          Alloc: currentAlloc.id,
+          Alloc: runningAlloc.id,
         },
         {
           Status: 'failure',
           Check: 'check3',
           Output: 'Oh no!',
           Timestamp: 99,
-          Alloc: currentAlloc.id,
+          Alloc: runningAlloc.id,
         },
         {
           Status: 'success',
@@ -731,14 +737,20 @@ module('Acceptance | allocation detail (services)', function (hooks) {
   });
 
   test('Allocation has a list of services with active checks', async function (assert) {
-    await visit('jobs/service-haver@default');
-    await click('.allocation-row');
+    faker.seed(1);
+    const runningAlloc = server.db.allocations.findBy({
+      jobId: 'service-haver',
+      forceRunningClientStatus: true,
+      clientStatus: 'running',
+    });
+    await Allocation.visit({ id: runningAlloc.id });
     assert.dom('[data-test-service]').exists();
     assert.dom('.service-sidebar').exists();
     assert.dom('.service-sidebar').doesNotHaveClass('open');
     assert
       .dom('[data-test-service-status-bar]')
       .exists('At least one allocation has service health');
+
     await click('[data-test-service-status-bar]');
     assert.dom('.service-sidebar').hasClass('open');
     assert
