@@ -1,3 +1,8 @@
+/**
+ * Copyright (c) HashiCorp, Inc.
+ * SPDX-License-Identifier: MPL-2.0
+ */
+
 /* eslint-disable qunit/require-expect */
 import { module, skip, test } from 'qunit';
 import { currentURL, settled } from '@ember/test-helpers';
@@ -21,6 +26,7 @@ module('Acceptance | exec', function (hooks) {
     faker.seed(1);
 
     server.create('agent');
+    server.create('node-pool');
     server.create('node');
 
     this.job = server.create('job', {
@@ -67,7 +73,7 @@ module('Acceptance | exec', function (hooks) {
       region: 'region-2',
     });
 
-    assert.equal(document.title, 'Exec - region-2 - Mirage - Nomad');
+    assert.ok(document.title.includes('Exec - region-2'));
 
     assert.equal(Exec.header.region.text, this.job.region);
     assert.equal(Exec.header.namespace.text, this.job.namespace);
