@@ -1,3 +1,8 @@
+/**
+ * Copyright (c) HashiCorp, Inc.
+ * SPDX-License-Identifier: MPL-2.0
+ */
+
 import AdapterError from '@ember-data/adapter/error';
 import {
   click,
@@ -61,6 +66,7 @@ module('Acceptance | job run', function (hooks) {
 
   hooks.beforeEach(function () {
     // Required for placing allocations (a result of creating jobs)
+    server.create('node-pool');
     server.create('node');
 
     managementToken = server.create('token');
@@ -352,7 +358,7 @@ module('Acceptance | job run', function (hooks) {
         'We do not navigate away from the page if an error is returned by the API.'
       );
       assert
-        .dom('.flash-message.alert-error')
+        .dom('.flash-message.alert-critical')
         .exists('A toast error message pops up.');
     });
 
