@@ -1,3 +1,8 @@
+/**
+ * Copyright (c) HashiCorp, Inc.
+ * SPDX-License-Identifier: MPL-2.0
+ */
+
 import { next } from '@ember/runloop';
 import { assign } from '@ember/polyfills';
 import { settled } from '@ember/test-helpers';
@@ -7,6 +12,9 @@ import { startMirage } from 'nomad-ui/initializers/ember-cli-mirage';
 import { AbortController } from 'fetch';
 import { TextEncoderLite } from 'text-encoder-lite';
 import base64js from 'base64-js';
+import addToPath from 'nomad-ui/utils/add-to-path';
+import sinon from 'sinon';
+import { resolve } from 'rsvp';
 
 module('Unit | Adapter | Job', function (hooks) {
   setupTest(hooks);
@@ -26,6 +34,7 @@ module('Unit | Adapter | Job', function (hooks) {
 
       this.server.create('namespace');
       this.server.create('namespace', { id: 'some-namespace' });
+      this.server.create('node-pool');
       this.server.create('node');
       this.server.create('job', { id: 'job-1', namespaceId: 'default' });
       this.server.create('job', { id: 'job-2', namespaceId: 'some-namespace' });
@@ -605,6 +614,71 @@ module('Unit | Adapter | Job', function (hooks) {
     );
     assert.equal(request.method, 'POST');
   });
+
+  module('#fetchRawSpecification', function () {
+    test('it makes a GET request to the correct URL', async function (assert) {
+      const adapter = this.owner.lookup('adapter:job');
+      const job = {
+        get: sinon.stub(),
+      };
+
+      job.get.withArgs('id').returns('["job-id"]');
+      job.get.withArgs('version').returns('job-version');
+
+      const expectedURL = addToPath(
+        adapter.urlForFindRecord('["job-id"]', 'job', null, 'submission'),
+        '',
+        'version=' + job.get('version')
+      );
+
+      // Stub the ajax method to avoid making real API calls
+      sinon.stub(adapter, 'ajax').callsFake(() => resolve({}));
+
+      await adapter.fetchRawSpecification(job);
+
+      assert.ok(adapter.ajax.calledOnce, 'The ajax method is called once');
+
+      assert.equal(
+        expectedURL,
+        '/v1/job/job-id/submission?version=job-version',
+        'it formats the URL correctly'
+      );
+    });
+
+    test('it formats namespaces correctly', async function (assert) {
+      const adapter = this.owner.lookup('adapter:job');
+      const job = {
+        get: sinon.stub(),
+      };
+
+      job.get.withArgs('id').returns('["job-id"]');
+      job.get.withArgs('version').returns('job-version');
+      job.get.withArgs('namespace').returns('zoey');
+
+      const expectedURL = addToPath(
+        adapter.urlForFindRecord(
+          '["job-id", "zoey"]',
+          'job',
+          null,
+          'submission'
+        ),
+        '',
+        'version=' + job.get('version')
+      );
+
+      // Stub the ajax method to avoid making real API calls
+      sinon.stub(adapter, 'ajax').callsFake(() => resolve({}));
+
+      await adapter.fetchRawSpecification(job);
+
+      assert.ok(adapter.ajax.calledOnce, 'The ajax method is called once');
+
+      assert.equal(
+        expectedURL,
+        '/v1/job/job-id/submission?namespace=zoey&version=job-version'
+      );
+    });
+  });
 });
 
 function makeMockModel(id, options) {
